@@ -95,9 +95,6 @@ func NewResourceFromConf(configMap map[string]string) (*Resource, error) {
 		if err != nil {
 			return nil, err
 		}
-		if intValue < 0 {
-			return nil, fmt.Errorf("negative resources not permitted: %v", configMap)
-		}
 		res.Resources[key] = intValue
 	}
 	return res, nil
@@ -133,21 +130,34 @@ func (r *Resource) ToProto() *si.Resource {
 	return proto
 }
 
-// Return a clone (copy) of the resource it is called on.
+// Clone returns a clone (copy) of the resource it is called on.
 // This provides a deep copy of the object with the exact same member set.
 // NOTE: this is a clone not a sparse copy of the original.
 func (r *Resource) Clone() *Resource {
-	ret := NewResource()
-	if r != nil {
-		for k, v := range r.Resources {
-			ret.Resources[k] = v
-		}
-		return ret
+	if r == nil {
+		return nil
 	}
-	return nil
+	ret := NewResource()
+	for k, v := range r.Resources {
+		ret.Resources[k] = v
+	}
+	return ret
 }
 
-// Add additional resource to the base updating the base resource
+// Prune removes any resource type that has a zero value set.
+// Note that a zero value set and a type not being set are interpreted differently for quotas.
+func (r *Resource) Prune() {
+	if r == nil {
+		return
+	}
+	for k, v := range r.Resources {
+		if v == 0 {
+			delete(r.Resources, k)
+		}
+	}
+}
+
+// AddTo adds the resource to the base updating the base resource
 // Should be used by temporary computation only
 // A nil base resource does not change
 // A nil passed in resource is treated as a zero valued resource and leaves base unchanged
@@ -353,18 +363,32 @@ func Sub(left, right *Resource) *Resource {
 	return out
 }
 
-// SubOnlyExisting subtract delta from defined resource.
-// Ignore any type not defined in the base resource (ie receiver).
-// Used as part of the headroom updates as undefined resources are unlimited
-func (r *Resource) SubOnlyExisting(delta *Resource) {
+// SubOnlyExisting subtracts delta from base resource, ignoring any type not defined in the base resource.
+func SubOnlyExisting(base, delta *Resource) *Resource {
 	// check nil inputs and shortcut
-	if r == nil || delta == nil {
-		return
+	if base == nil || delta == nil {
+		return base.Clone()
 	}
 	// neither are nil, subtract the delta
-	for k := range r.Resources {
-		r.Resources[k] = subVal(r.Resources[k], delta.Resources[k])
+	result := NewResource()
+	for k := range base.Resources {
+		result.Resources[k] = subVal(base.Resources[k], delta.Resources[k])
 	}
+	return result
+}
+
+// AddOnlyExisting adds delta to base resource, ignoring any type not defined in the base resource.
+func AddOnlyExisting(base, delta *Resource) *Resource {
+	// check nil inputs and shortcut
+	if base == nil || delta == nil {
+		return base.Clone()
+	}
+	// neither are nil, add the delta
+	result := NewResource()
+	for k := range base.Resources {
+		result.Resources[k] = addVal(base.Resources[k], delta.Resources[k])
+	}
+	return result
 }
 
 // SubEliminateNegative subtracts resource returning a new resource with the result
@@ -458,6 +482,62 @@ func (r *Resource) fitIn(smaller *Resource, skipUndef bool) bool {
 	return true
 }
 
+// getShareFairForDenominator attempts to computes the denominator for a queue's fair share ratio.
+// Here Resources can be either guaranteed Resources or fairmax Resources.
+// If the quanity is explicitly 0 or negative, we will check usage.  If usage >= 0, the share will be set to 1.0.  Otherwise, it will be set 0.0.
+func getShareFairForDenominator(resourceType string, allocated Quantity, denominatorResources *Resource) (float64, bool) {
+	if denominatorResources == nil {
+		return 0.0, false
+	}
+
+	denominator, ok := denominatorResources.Resources[resourceType]
+
+	switch {
+	case ok && denominator <= 0:
+		if allocated <= 0 {
+			// explicit 0 or negative value with NO usage
+			return 0.0, true
+		} else {
+			// explicit 0 or negative value with usage
+			return 1.0, true
+		}
+	case denominator > 0:
+		return (float64(allocated) / float64(denominator)), true
+	default:
+		// no denominator. ie. no guarantee or fairmax for resourceType
+		return 0.0, false
+	}
+}
+
+// getFairShare produces a ratio which represents it's current 'fair' share usage.
+// Iterate over all of the allocated resource types.  For each, compute the ratio, ultimately returning the max ratio encountered.
+// The numerator will be the allocated usage.
+// If guarantees are present, they will be used for the denominator, otherwise we will fallback to the 'maxfair' capacity of the cluster.
+func getFairShare(allocated, guaranteed, fair *Resource) float64 {
+	if allocated == nil || len(allocated.Resources) == 0 {
+		return 0.0
+	}
+
+	var maxShare float64
+	for k, v := range allocated.Resources {
+		var nextShare float64
+
+		// if usage <= 0, resource has no share
+		if allocated.Resources[k] < 0 {
+			continue
+		}
+
+		nextShare, found := getShareFairForDenominator(k, v, guaranteed)
+		if !found {
+			nextShare, found = getShareFairForDenominator(k, v, fair)
+		}
+		if found && nextShare > maxShare {
+			maxShare = nextShare
+		}
+	}
+	return maxShare
+}
+
 // Get the share of each resource quantity when compared to the total
 // resources quantity
 // NOTE: shares can be negative and positive in the current assumptions
@@ -522,24 +602,18 @@ func CompUsageRatio(left, right, total *Resource) int {
 // 0 for equal shares
 // 1 if the left share is larger
 // -1 if the right share is larger
-func CompUsageRatioSeparately(left, leftTotal, right, rightTotal *Resource) int {
-	lshares := getShares(left, leftTotal)
-	rshares := getShares(right, rightTotal)
+func CompUsageRatioSeparately(leftAllocated, leftGuaranteed, leftFairMax, rightAllocated, rightGuaranteed, rightFairMax *Resource) int {
+	lshare := getFairShare(leftAllocated, leftGuaranteed, leftFairMax)
+	rshare := getFairShare(rightAllocated, rightGuaranteed, rightFairMax)
 
-	return compareShares(lshares, rshares)
-}
-
-// Compare two resources usage shares and assumes a nil total resource.
-// The share is thus equivalent to the usage passed in.
-// This returns the same value as compareShares does:
-// 0 for equal shares
-// 1 if the left share is larger
-// -1 if the right share is larger
-func CompUsageShares(left, right *Resource) int {
-	lshares := getShares(left, nil)
-	rshares := getShares(right, nil)
-
-	return compareShares(lshares, rshares)
+	switch {
+	case lshare > rshare:
+		return 1
+	case lshare < rshare:
+		return -1
+	default:
+		return 0
+	}
 }
 
 // Get fairness ratio calculated by:
@@ -618,12 +692,11 @@ func compareShares(lshares, rshares []float64) int {
 	return 0
 }
 
-// Compare the resources equal returns the specific values for following cases:
-// left  right  return
-// nil   nil    true
-// nil   <set>  false
-// <set> nil    false
-// <set> <set>  true/false  *based on the individual Quantity values
+// Equals Compare the resources based on common resource type available in both left and right Resource
+// Resource type available in left Resource but not in right Resource and vice versa is not taken into account
+// False in case anyone of the resources is nil
+// False in case resource type value differs
+// True in case when resource type values of left Resource matches with right Resource if resource type is available
 func Equals(left, right *Resource) bool {
 	if left == right {
 		return true
@@ -638,16 +711,61 @@ func Equals(left, right *Resource) bool {
 			return false
 		}
 	}
-
 	for k, v := range right.Resources {
 		if left.Resources[k] != v {
 			return false
 		}
 	}
-
 	return true
 }
 
+// DeepEquals Compare the resources based on resource type existence and its values as well
+// False in case anyone of the resources is nil
+// False in case resource length differs
+// False in case resource type existed in left Resource not exist in right Resource
+// False in case resource type value differs
+// True in case when all resource type and its values of left Resource matches with right Resource
+func DeepEquals(left, right *Resource) bool {
+	if left == right {
+		return true
+	}
+	if left == nil || right == nil {
+		return false
+	}
+	if len(right.Resources) != len(left.Resources) {
+		return false
+	}
+	for k, v := range left.Resources {
+		if val, ok := right.Resources[k]; ok {
+			if val != v {
+				return false
+			}
+		} else {
+			return false
+		}
+	}
+	return true
+}
+
+// MatchAny returns true if at least one type in the defined resource exists in the other resource.
+// False if none of the types exist in the other resource.
+// A nil resource is treated as an empty resource (no types defined) and returns false
+// Values are not considered during the checks
+func (r *Resource) MatchAny(other *Resource) bool {
+	if r == nil || other == nil {
+		return false
+	}
+	if r == other {
+		return true
+	}
+	for k := range r.Resources {
+		if _, ok := other.Resources[k]; ok {
+			return true
+		}
+	}
+	return false
+}
+
 // Compare the resources equal returns the specific values for following cases:
 // left  right  return
 // nil   nil    true
@@ -754,6 +872,55 @@ func StrictlyGreaterThanOrEquals(larger, smaller *Resource) bool {
 	return true
 }
 
+// StrictlyGreaterThanOnlyExisting returns true if all quantities for types in the defined resource are greater than
+// the quantity for the same type in smaller.
+// Types defined in smaller that are not in the defined resource are ignored.
+// Two resources that are equal are not considered strictly larger than each other.
+func (r *Resource) StrictlyGreaterThanOnlyExisting(smaller *Resource) bool {
+	if r == nil {
+		r = Zero
+	}
+	if smaller == nil {
+		smaller = Zero
+	}
+
+	// keep track of the number of not equal values
+	notEqual := false
+
+	// Is larger and smaller completely disjoint?
+	atleastOneResourcePresent := false
+	// Is all resource in larger greater than zero?
+	isAllPositiveInLarger := true
+
+	for k, v := range r.Resources {
+		// even when smaller is empty, all resource type in larger should be greater than zero
+		if smaller.IsEmpty() && v <= 0 {
+			isAllPositiveInLarger = false
+		}
+		// when smaller is not empty
+		if val, ok := smaller.Resources[k]; ok {
+			// at least one common resource type is there
+			atleastOneResourcePresent = true
+			if val > v {
+				return false
+			}
+			if val != v {
+				notEqual = true
+			}
+		}
+	}
+
+	switch {
+	case smaller.IsEmpty() && !r.IsEmpty():
+		return isAllPositiveInLarger
+	case atleastOneResourcePresent:
+		return notEqual
+	default:
+		// larger and smaller is completely disjoint. none of the resource match.
+		return !r.IsEmpty() && !smaller.IsEmpty()
+	}
+}
+
 // Have at least one quantity > 0, and no quantities < 0
 // A nil resource is not strictly greater than zero.
 func StrictlyGreaterThanZero(larger *Resource) bool {
@@ -772,26 +939,10 @@ func StrictlyGreaterThanZero(larger *Resource) bool {
 	return greater
 }
 
-// Returns a new resource with the smallest value for each quantity in the resources
-// If either resource passed in is nil a zero resource is returned
-// If a resource type is missing from one of the Resource, it is considered 0
-func ComponentWiseMin(left, right *Resource) *Resource {
-	out := NewResource()
-	if left != nil && right != nil {
-		for k, v := range left.Resources {
-			out.Resources[k] = min(v, right.Resources[k])
-		}
-		for k, v := range right.Resources {
-			out.Resources[k] = min(v, left.Resources[k])
-		}
-	}
-	return out
-}
-
-// Returns a new Resource with the smallest value for each quantity in the Resources
+// ComponentWiseMin returns a new Resource with the smallest value for each quantity in the Resources
 // If either Resource passed in is nil the other Resource is returned
 // If a Resource type is missing from one of the Resource, it is considered empty and the quantity from the other Resource is returned
-func ComponentWiseMinPermissive(left, right *Resource) *Resource {
+func ComponentWiseMin(left, right *Resource) *Resource {
 	out := NewResource()
 	if right == nil && left == nil {
 		return nil
@@ -819,6 +970,52 @@ func ComponentWiseMinPermissive(left, right *Resource) *Resource {
 	return out
 }
 
+// MergeIfNotPresent Returns a new Resource by merging resource type values present in right with left
+// only if resource type not present in left.
+// If either Resource passed in is nil the other Resource is returned
+// If a Resource type is missing from one of the Resource, it is considered empty and the quantity from the other Resource is returned
+func MergeIfNotPresent(left, right *Resource) *Resource {
+	if right == nil && left == nil {
+		return nil
+	}
+	if left == nil {
+		return right.Clone()
+	}
+	if right == nil {
+		return left.Clone()
+	}
+	out := left.Clone()
+	for k, v := range right.Resources {
+		if _, ok := left.Resources[k]; !ok {
+			out.Resources[k] = v
+		}
+	}
+	return out
+}
+
+// ComponentWiseMinOnlyExisting Returns a new Resource with the smallest value for resource type
+// existing only in left but not vice versa.
+func ComponentWiseMinOnlyExisting(left, right *Resource) *Resource {
+	out := NewResource()
+	if right == nil && left == nil {
+		return nil
+	}
+	if left == nil {
+		return nil
+	}
+	if right == nil {
+		return left.Clone()
+	}
+	for k, v := range left.Resources {
+		if val, ok := right.Resources[k]; ok {
+			out.Resources[k] = min(v, val)
+		} else {
+			out.Resources[k] = v
+		}
+	}
+	return out
+}
+
 func (r *Resource) HasNegativeValue() bool {
 	if r == nil {
 		return false
