@@ -49,7 +49,7 @@ func TestAcceptStateTransition(t *testing.T) {
 	// accepted to failed
 	err = app.HandleApplicationEvent(FailApplication)
 	assert.NilError(t, err, "no error expected accepted to failing")
-	err = common.WaitFor(10*time.Microsecond, time.Millisecond*100, app.IsFailing)
+	err = common.WaitForCondition(10*time.Microsecond, time.Millisecond*100, app.IsFailing)
 	assert.NilError(t, err, "App should be in Failing state")
 }
 
@@ -74,44 +74,14 @@ func TestRejectStateTransition(t *testing.T) {
 	assert.Equal(t, app.CurrentState(), Rejected.String())
 }
 
-func TestStartStateTransition(t *testing.T) {
-	// starting only from accepted
-	appInfo := newApplication("app-00001", "default", "root.a")
-	assert.Equal(t, appInfo.CurrentState(), New.String())
-	err := appInfo.HandleApplicationEvent(RunApplication)
-	assert.NilError(t, err, "no error expected new to accepted (start test)")
-	assert.Equal(t, appInfo.CurrentState(), Accepted.String())
-
-	// start app
-	err = appInfo.HandleApplicationEvent(RunApplication)
-	assert.Assert(t, err, "no error expected new to starting")
-	assert.Equal(t, appInfo.CurrentState(), Starting.String())
-
-	// starting to rejected: error expected
-	err = appInfo.HandleApplicationEvent(RejectApplication)
-	assert.Assert(t, err != nil, "error expected starting to rejected")
-	assert.Equal(t, appInfo.CurrentState(), Starting.String())
-
-	// start to failing
-	err = appInfo.HandleApplicationEvent(FailApplication)
-	assert.NilError(t, err, "no error expected starting to failing")
-	err = common.WaitFor(10*time.Microsecond, time.Millisecond*100, appInfo.IsFailing)
-	assert.NilError(t, err, "App should be in Failing state")
-}
-
 func TestRunStateTransition(t *testing.T) {
-	// run only from starting
+	// run from new
 	appInfo := newApplication("app-00001", "default", "root.a")
 	assert.Equal(t, appInfo.CurrentState(), New.String())
 	err := appInfo.HandleApplicationEvent(RunApplication)
 	assert.NilError(t, err, "no error expected new to accepted (run test)")
 	err = appInfo.HandleApplicationEvent(RunApplication)
-	assert.NilError(t, err, "no error expected accepted to starting (run test)")
-	assert.Equal(t, appInfo.CurrentState(), Starting.String())
-
-	// run app
-	err = appInfo.HandleApplicationEvent(RunApplication)
-	assert.NilError(t, err, "no error expected starting to running")
+	assert.NilError(t, err, "no error expected accepted to running (run test)")
 	assert.Equal(t, appInfo.CurrentState(), Running.String())
 
 	// run app: same state is allowed for running
@@ -127,7 +97,7 @@ func TestRunStateTransition(t *testing.T) {
 	// run to failing
 	err = appInfo.HandleApplicationEvent(FailApplication)
 	assert.NilError(t, err, "no error expected running to failing")
-	err = common.WaitFor(10*time.Microsecond, time.Millisecond*100, appInfo.IsFailing)
+	err = common.WaitForCondition(10*time.Microsecond, time.Millisecond*100, appInfo.IsFailing)
 	assert.NilError(t, err, "App should be in Failing state")
 
 	// run fails from failing
@@ -143,9 +113,9 @@ func TestCompletedStateTransition(t *testing.T) {
 	err := appInfo1.HandleApplicationEvent(RunApplication)
 	assert.NilError(t, err, "no error expected new to accepted (completed test)")
 	err = appInfo1.HandleApplicationEvent(RunApplication)
-	assert.NilError(t, err, "no error expected accepted to starting (completed test)")
+	assert.NilError(t, err, "no error expected accepted to running (completed test)")
 	err = appInfo1.HandleApplicationEvent(RunApplication)
-	assert.NilError(t, err, "no error expected starting to running (completed test)")
+	assert.NilError(t, err, "no error expected running to running (completed test)")
 	assert.Equal(t, appInfo1.CurrentState(), Running.String())
 	// completed from run through completing
 	err = appInfo1.HandleApplicationEvent(CompleteApplication)
@@ -197,21 +167,21 @@ func TestCompletingStateTransition(t *testing.T) {
 	assert.NilError(t, err, "no error expected accepted to completing")
 	assert.Equal(t, appInfo1.CurrentState(), Completing.String())
 
-	// starting to completing
+	// running to completing
 	appInfo2 := newApplication("app-00002", "default", "root.a")
 	assert.Equal(t, appInfo2.CurrentState(), New.String())
 	err = appInfo2.HandleApplicationEvent(RunApplication)
 	assert.NilError(t, err, "no error expected new to accepted (completing test)")
 	err = appInfo2.HandleApplicationEvent(RunApplication)
-	assert.NilError(t, err, "no error expected accepted to starting")
-	assert.Equal(t, appInfo2.CurrentState(), Starting.String())
+	assert.NilError(t, err, "no error expected accepted to running")
+	assert.Equal(t, appInfo2.CurrentState(), Running.String())
 	err = appInfo2.HandleApplicationEvent(CompleteApplication)
-	assert.NilError(t, err, "no error expected starting to completing")
+	assert.NilError(t, err, "no error expected running to completing")
 	assert.Equal(t, appInfo2.CurrentState(), Completing.String())
 
 	// completing to run and back again
 	err = appInfo2.HandleApplicationEvent(RunApplication)
-	assert.NilError(t, err, "no error expected starting to running (completing test)")
+	assert.NilError(t, err, "no error expected completing to running (completing test)")
 	err = appInfo2.HandleApplicationEvent(CompleteApplication)
 	assert.NilError(t, err, "no error expected running to completing")
 	assert.Equal(t, appInfo2.CurrentState(), Completing.String())
@@ -225,7 +195,7 @@ func TestFailedStateTransition(t *testing.T) {
 	// new to failing
 	err := appInfo.HandleApplicationEvent(FailApplication)
 	assert.NilError(t, err, "no error expected new to failing")
-	err = common.WaitFor(10*time.Microsecond, time.Millisecond*100, appInfo.IsFailing)
+	err = common.WaitForCondition(10*time.Microsecond, time.Millisecond*100, appInfo.IsFailing)
 	assert.NilError(t, err, "App should be in Failing state")
 
 	// failing to failed
@@ -236,7 +206,7 @@ func TestFailedStateTransition(t *testing.T) {
 	// failed to rejected: error expected
 	err = appInfo.HandleApplicationEvent(RejectApplication)
 	assert.Assert(t, err != nil, "error expected failing to rejected")
-	err = common.WaitFor(10*time.Microsecond, time.Millisecond*100, appInfo.IsFailed)
+	err = common.WaitForCondition(10*time.Microsecond, time.Millisecond*100, appInfo.IsFailed)
 	assert.NilError(t, err, "App should be in Failed state")
 }
 
@@ -260,12 +230,12 @@ func TestAppStateTransitionEvents(t *testing.T) {
 
 	// completing to run
 	err = appInfo.HandleApplicationEvent(RunApplication)
-	assert.NilError(t, err, "no error expected starting to running (completing test)")
+	assert.NilError(t, err, "no error expected completing to running (completing test)")
 
 	// run to failing
 	err = appInfo.HandleApplicationEvent(FailApplication)
 	assert.NilError(t, err, "no error expected new to failing")
-	err = common.WaitFor(10*time.Microsecond, time.Millisecond*100, appInfo.IsFailing)
+	err = common.WaitForCondition(10*time.Microsecond, time.Millisecond*100, appInfo.IsFailing)
 	assert.NilError(t, err, "App should be in Failing state")
 
 	// failing to failed
@@ -285,7 +255,7 @@ func TestAppStateTransitionEvents(t *testing.T) {
 	assert.Assert(t, appInfo.IsResuming(), "App should be in Resuming state")
 
 	// Verify application events
-	err = common.WaitFor(10*time.Millisecond, time.Second, func() bool {
+	err = common.WaitForCondition(10*time.Millisecond, time.Second, func() bool {
 		fmt.Printf("checking event length: %d\n", eventSystem.Store.CountStoredEvents())
 		return eventSystem.Store.CountStoredEvents() == 8
 	})
@@ -306,105 +276,135 @@ func TestAppStateTransitionEvents(t *testing.T) {
 }
 
 // Test to verify metrics after applications state transition
-// app-00001: New -> Resuming -> Accepted -> Starting -> Running -> Completing-> Completed
-// app-00002: New -> Accepted -> Starting -> Completing -> Running -> Failing-> Failed
-// app-00003: New -> Accepted -> Starting -> Failing -> Failed
+// app-00001: New -> Resuming -> Accepted -> Running -> Completing-> Completed
+// app-00002: New -> Accepted -> Running -> Completing -> Running -> Failing-> Failed
+// app-00003: New -> Accepted -> Running -> Failing -> Failed
 // app-00004: New -> Rejected
-// Final metrics will be: 0 running, 3 accepted, 1 completed, 2 failed and 1 rejected applications
+// Final queue metrics will be: 0 new, 0 running, 0 accepted, 1 completed, 2 failed and 1 rejected applications
+// Final scheduler metrics will be: 4 new, 0 running, 3 accepted, 1 completed, 2 failed and 1 rejected applications
+// Because the scheduler app submission state (new, accepted, rejected)is counter based, and it will not be decremented when the state changes
 func TestAppStateTransitionMetrics(t *testing.T) { //nolint:funlen
-	queue := createQueue(t, "root.metrics")
+	queue := createQueue(t, "metrics")
 	metrics.GetSchedulerMetrics().Reset()
-	// app-00001: New -> Resuming -> Accepted --> Starting -> Running -> Completing-> Completed
+	// app-00001: New -> Resuming -> Accepted --> Running -> Completing-> Completed
 	app := newApplication("app-00001", "default", "root.metrics")
 	app.SetQueue(queue)
 	assertState(t, app, nil, New.String())
-	assertTotalAppsRunningMetrics(t, 0)
-	assertTotalAppsCompletedMetrics(t, 0)
-	assertTotalAppsRejectedMetrics(t, 0)
-	// New -> Resuming
-	err := app.HandleApplicationEvent(ResumeApplication)
-	assertState(t, app, err, Resuming.String())
+	assertTotalAppsNewMetrics(t, 1)
 	assertTotalAppsRunningMetrics(t, 0)
 	assertTotalAppsCompletedMetrics(t, 0)
 	assertTotalAppsRejectedMetrics(t, 0)
 	assertQueueRunningApps(t, app, 0)
 	assertQueueApplicationsRunningMetrics(t, app, 0)
-	assertQueueApplicationsAcceptedMetrics(t, app, 1)
+	assertQueueApplicationsAcceptedMetrics(t, app, 0)
 	assertQueueApplicationsRejectedMetrics(t, app, 0)
 	assertQueueApplicationsFailedMetrics(t, app, 0)
 	assertQueueApplicationsCompletedMetrics(t, app, 0)
+	assertQueueApplicationsNewMetrics(t, app, 1)
+
+	// New -> Resuming
+	err := app.HandleApplicationEvent(ResumeApplication)
+	assertState(t, app, err, Resuming.String())
+	assertTotalAppsNewMetrics(t, 1)
+	assertTotalAppsRunningMetrics(t, 0)
+	assertTotalAppsCompletedMetrics(t, 0)
+	assertTotalAppsRejectedMetrics(t, 0)
+	assertQueueRunningApps(t, app, 0)
+	assertQueueApplicationsRunningMetrics(t, app, 0)
+	assertQueueApplicationsAcceptedMetrics(t, app, 0)
+	assertQueueApplicationsRejectedMetrics(t, app, 0)
+	assertQueueApplicationsFailedMetrics(t, app, 0)
+	assertQueueApplicationsCompletedMetrics(t, app, 0)
+	assertQueueApplicationsResumingMetrics(t, app, 1)
+	assertQueueApplicationsNewMetrics(t, app, 0)
 	// Resuming -> Accepted
 	err = app.HandleApplicationEvent(RunApplication)
 	assertState(t, app, err, Accepted.String())
 	assertTotalAppsRunningMetrics(t, 0)
 	assertTotalAppsCompletedMetrics(t, 0)
 	assertTotalAppsRejectedMetrics(t, 0)
+	assertTotalAppsNewMetrics(t, 1)
+	assertTotalAppsAcceptedMetrics(t, 1)
 	assertQueueRunningApps(t, app, 0)
 	assertQueueApplicationsRunningMetrics(t, app, 0)
 	assertQueueApplicationsAcceptedMetrics(t, app, 1)
 	assertQueueApplicationsRejectedMetrics(t, app, 0)
 	assertQueueApplicationsFailedMetrics(t, app, 0)
 	assertQueueApplicationsCompletedMetrics(t, app, 0)
-	// Accepted -> Starting
-	err = app.HandleApplicationEvent(RunApplication)
-	assertState(t, app, err, Starting.String())
-	assertTotalAppsRunningMetrics(t, 1)
-	assertTotalAppsCompletedMetrics(t, 0)
-	assertTotalAppsRejectedMetrics(t, 0)
-	assertQueueRunningApps(t, app, 1)
-	assertQueueApplicationsRunningMetrics(t, app, 1)
-	assertQueueApplicationsAcceptedMetrics(t, app, 1)
-	assertQueueApplicationsRejectedMetrics(t, app, 0)
-	assertQueueApplicationsFailedMetrics(t, app, 0)
-	assertQueueApplicationsCompletedMetrics(t, app, 0)
-	// Starting -> Running
+	assertQueueApplicationsNewMetrics(t, app, 0)
+	// Accepted -> Running
 	err = app.HandleApplicationEvent(RunApplication)
 	assertState(t, app, err, Running.String())
 	assertTotalAppsRunningMetrics(t, 1)
 	assertTotalAppsCompletedMetrics(t, 0)
 	assertTotalAppsRejectedMetrics(t, 0)
+	assertTotalAppsNewMetrics(t, 1)
+	assertTotalAppsAcceptedMetrics(t, 1)
 	assertQueueRunningApps(t, app, 1)
 	assertQueueApplicationsRunningMetrics(t, app, 1)
-	assertQueueApplicationsAcceptedMetrics(t, app, 1)
+	assertQueueApplicationsAcceptedMetrics(t, app, 0)
 	assertQueueApplicationsRejectedMetrics(t, app, 0)
 	assertQueueApplicationsFailedMetrics(t, app, 0)
 	assertQueueApplicationsCompletedMetrics(t, app, 0)
+	assertQueueApplicationsNewMetrics(t, app, 0)
+	// Running -> Running
+	err = app.HandleApplicationEvent(RunApplication)
+	assertState(t, app, err, Running.String())
+	assertTotalAppsRunningMetrics(t, 1)
+	assertTotalAppsCompletedMetrics(t, 0)
+	assertTotalAppsRejectedMetrics(t, 0)
+	assertTotalAppsAcceptedMetrics(t, 1)
+	assertTotalAppsNewMetrics(t, 1)
+	assertTotalAppsAcceptedMetrics(t, 1)
+	assertQueueRunningApps(t, app, 1)
+	assertQueueApplicationsRunningMetrics(t, app, 1)
+	assertQueueApplicationsAcceptedMetrics(t, app, 0)
+	assertQueueApplicationsRejectedMetrics(t, app, 0)
+	assertQueueApplicationsFailedMetrics(t, app, 0)
+	assertQueueApplicationsCompletedMetrics(t, app, 0)
+	assertQueueApplicationsNewMetrics(t, app, 0)
 	// Running -> Completing
 	err = app.HandleApplicationEvent(CompleteApplication)
 	assertState(t, app, err, Completing.String())
 	assertTotalAppsRunningMetrics(t, 0)
 	assertTotalAppsCompletedMetrics(t, 0)
 	assertTotalAppsRejectedMetrics(t, 0)
+	assertTotalAppsNewMetrics(t, 1)
+	assertTotalAppsAcceptedMetrics(t, 1)
 	assertQueueRunningApps(t, app, 0)
 	assertQueueApplicationsRunningMetrics(t, app, 0)
-	assertQueueApplicationsAcceptedMetrics(t, app, 1)
+	assertQueueApplicationsAcceptedMetrics(t, app, 0)
 	assertQueueApplicationsRejectedMetrics(t, app, 0)
 	assertQueueApplicationsFailedMetrics(t, app, 0)
 	assertQueueApplicationsCompletedMetrics(t, app, 0)
+	assertQueueApplicationsCompletingMetrics(t, app, 1)
 	// Completing -> Completed
 	err = app.HandleApplicationEvent(CompleteApplication)
 	assertState(t, app, err, Completed.String())
 	assertTotalAppsRunningMetrics(t, 0)
 	assertTotalAppsCompletedMetrics(t, 1)
 	assertTotalAppsRejectedMetrics(t, 0)
+	assertTotalAppsNewMetrics(t, 1)
+	assertTotalAppsAcceptedMetrics(t, 1)
 	assertQueueRunningApps(t, app, 0)
 	assertQueueApplicationsRunningMetrics(t, app, 0)
-	assertQueueApplicationsAcceptedMetrics(t, app, 1)
+	assertQueueApplicationsAcceptedMetrics(t, app, 0)
 	assertQueueApplicationsRejectedMetrics(t, app, 0)
 	assertQueueApplicationsFailedMetrics(t, app, 0)
 	assertQueueApplicationsCompletedMetrics(t, app, 1)
+	assertQueueApplicationsCompletingMetrics(t, app, 0)
 
-	// app-00002: New -> Accepted -> Starting -> Completing -> Running -> Failing-> Failed
+	// app-00002: New -> Accepted -> Completing -> Running -> Failing-> Failed
 	app = newApplication("app-00002", "default", "root.metrics")
 	app.SetQueue(queue)
 	assertState(t, app, nil, New.String())
 	// New -> Accepted
 	err = app.HandleApplicationEvent(RunApplication)
 	assertState(t, app, err, Accepted.String())
-	// Accepted -> Starting
+	// Accepted -> Running
 	err = app.HandleApplicationEvent(RunApplication)
-	assertState(t, app, err, Starting.String())
-	// Starting -> Completing
+	assertState(t, app, err, Running.String())
+	// Running -> Completing
 	err = app.HandleApplicationEvent(CompleteApplication)
 	assertState(t, app, err, Completing.String())
 	// Completing -> Running
@@ -419,24 +419,26 @@ func TestAppStateTransitionMetrics(t *testing.T) { //nolint:funlen
 	assertTotalAppsRunningMetrics(t, 0)
 	assertTotalAppsCompletedMetrics(t, 1)
 	assertTotalAppsRejectedMetrics(t, 0)
+	assertTotalAppsNewMetrics(t, 2)
+	assertTotalAppsAcceptedMetrics(t, 2)
 	assertQueueRunningApps(t, app, 0)
 	assertQueueApplicationsRunningMetrics(t, app, 0)
-	assertQueueApplicationsAcceptedMetrics(t, app, 2)
+	assertQueueApplicationsAcceptedMetrics(t, app, 0)
 	assertQueueApplicationsRejectedMetrics(t, app, 0)
 	assertQueueApplicationsFailedMetrics(t, app, 1)
 	assertQueueApplicationsCompletedMetrics(t, app, 1)
 
-	// app-00003: New -> Accepted -> Starting -> Failing -> Failed
+	// app-00003: New -> Accepted -> Running -> Failing -> Failed
 	app = newApplication("app-00003", "default", "root.metrics")
 	app.SetQueue(queue)
 	assertState(t, app, nil, New.String())
 	// New -> Accepted
 	err = app.HandleApplicationEvent(RunApplication)
 	assertState(t, app, err, Accepted.String())
-	// Accepted -> Starting
+	// Accepted -> Running
 	err = app.HandleApplicationEvent(RunApplication)
-	assertState(t, app, err, Starting.String())
-	// Starting -> Failing
+	assertState(t, app, err, Running.String())
+	// Running -> Failing
 	err = app.HandleApplicationEvent(FailApplication)
 	assertState(t, app, err, Failing.String())
 	// Failing -> Failed
@@ -445,9 +447,11 @@ func TestAppStateTransitionMetrics(t *testing.T) { //nolint:funlen
 	assertTotalAppsRunningMetrics(t, 0)
 	assertTotalAppsCompletedMetrics(t, 1)
 	assertTotalAppsRejectedMetrics(t, 0)
+	assertTotalAppsNewMetrics(t, 3)
+	assertTotalAppsAcceptedMetrics(t, 3)
 	assertQueueRunningApps(t, app, 0)
 	assertQueueApplicationsRunningMetrics(t, app, 0)
-	assertQueueApplicationsAcceptedMetrics(t, app, 3)
+	assertQueueApplicationsAcceptedMetrics(t, app, 0)
 	assertQueueApplicationsRejectedMetrics(t, app, 0)
 	assertQueueApplicationsFailedMetrics(t, app, 2)
 	assertQueueApplicationsCompletedMetrics(t, app, 1)
@@ -462,9 +466,12 @@ func TestAppStateTransitionMetrics(t *testing.T) { //nolint:funlen
 	assertTotalAppsRunningMetrics(t, 0)
 	assertTotalAppsCompletedMetrics(t, 1)
 	assertTotalAppsRejectedMetrics(t, 1)
+	assertTotalAppsNewMetrics(t, 4)
+	assertTotalAppsAcceptedMetrics(t, 3)
 	assertQueueRunningApps(t, app, 0)
+	assertQueueApplicationsNewMetrics(t, app, 0)
 	assertQueueApplicationsRunningMetrics(t, app, 0)
-	assertQueueApplicationsAcceptedMetrics(t, app, 3)
+	assertQueueApplicationsAcceptedMetrics(t, app, 0)
 	assertQueueApplicationsRejectedMetrics(t, app, 1)
 	assertQueueApplicationsFailedMetrics(t, app, 2)
 	assertQueueApplicationsCompletedMetrics(t, app, 1)
@@ -476,6 +483,13 @@ func assertState(t testing.TB, app *Application, err error, expected string) {
 	assert.Equal(t, app.CurrentState(), expected, "application not in expected state.")
 }
 
+func assertTotalAppsNewMetrics(t testing.TB, expected int) {
+	t.Helper()
+	totalAppsNew, err := metrics.GetSchedulerMetrics().GetTotalApplicationsNew()
+	assert.NilError(t, err, "no error expected when getting total new application count.")
+	assert.Equal(t, totalAppsNew, expected, "total new application metrics is not as expected.")
+}
+
 func assertTotalAppsRunningMetrics(t testing.TB, expected int) {
 	t.Helper()
 	totalAppsRunning, err := metrics.GetSchedulerMetrics().GetTotalApplicationsRunning()
@@ -483,6 +497,13 @@ func assertTotalAppsRunningMetrics(t testing.TB, expected int) {
 	assert.Equal(t, totalAppsRunning, expected, "total running application metrics is not as expected.")
 }
 
+func assertTotalAppsAcceptedMetrics(t testing.TB, expected int) {
+	t.Helper()
+	totalAppsAccepted, err := metrics.GetSchedulerMetrics().GetTotalApplicationsAccepted()
+	assert.NilError(t, err, "no error expected when getting total accepted application count.")
+	assert.Equal(t, totalAppsAccepted, expected, "total accepted application metrics is not as expected.")
+}
+
 func assertTotalAppsCompletedMetrics(t testing.TB, expected int) {
 	t.Helper()
 	totalAppsCompleted, err := metrics.GetSchedulerMetrics().GetTotalApplicationsCompleted()
@@ -531,6 +552,13 @@ func assertQueueApplicationsFailedMetrics(t testing.TB, app *Application, expect
 	assert.Equal(t, queueApplicationsFailed, expected, "total failed application metrics in queue is not as expected.")
 }
 
+func assertQueueApplicationsResumingMetrics(t testing.TB, app *Application, expected int) {
+	t.Helper()
+	queueApplicationsResuming, err := metrics.GetQueueMetrics(app.queuePath).GetQueueApplicationsResuming()
+	assert.NilError(t, err, "no error expected when getting total resuming application count in queue.")
+	assert.Equal(t, queueApplicationsResuming, expected, "total resuming application metrics in queue is not as expected.")
+}
+
 func assertQueueApplicationsCompletedMetrics(t testing.TB, app *Application, expected int) {
 	t.Helper()
 	queueApplicationsCompleted, err := metrics.GetQueueMetrics(app.queuePath).GetQueueApplicationsCompleted()
@@ -538,6 +566,20 @@ func assertQueueApplicationsCompletedMetrics(t testing.TB, app *Application, exp
 	assert.Equal(t, queueApplicationsCompleted, expected, "total completed application metrics in queue is not as expected.")
 }
 
+func assertQueueApplicationsCompletingMetrics(t testing.TB, app *Application, expected int) {
+	t.Helper()
+	queueApplicationsCompleting, err := metrics.GetQueueMetrics(app.queuePath).GetQueueApplicationsCompleting()
+	assert.NilError(t, err, "no error expected when getting total completing application count in queue.")
+	assert.Equal(t, queueApplicationsCompleting, expected, "total completing application metrics in queue is not as expected.")
+}
+
+func assertQueueApplicationsNewMetrics(t testing.TB, app *Application, expected int) {
+	t.Helper()
+	queueApplicationsNew, err := metrics.GetQueueMetrics(app.queuePath).GetQueueApplicationsNew()
+	assert.NilError(t, err, "no error expected when getting total new applications in queue.")
+	assert.Equal(t, queueApplicationsNew, expected, "total new applications in queue is not as expected.")
+}
+
 func createQueue(t *testing.T, queueName string) *Queue {
 	root, err := createRootQueue(nil)
 	assert.NilError(t, err, "failed to create queue: %v", err)
@@ -545,3 +587,10 @@ func createQueue(t *testing.T, queueName string) *Queue {
 	assert.NilError(t, err, "failed to create queue: %v", err)
 	return queue
 }
+
+func isStateChangeEvent(t *testing.T, app *Application, changeDetail si.EventRecord_ChangeDetail, record *si.EventRecord) {
+	assert.Equal(t, si.EventRecord_APP, record.Type, "incorrect event type, expect app")
+	assert.Equal(t, app.ApplicationID, record.ObjectID, "incorrect object ID, expected application ID")
+	assert.Equal(t, si.EventRecord_SET, record.EventChangeType, "incorrect change type, expected set")
+	assert.Equal(t, changeDetail, record.EventChangeDetail, "incorrect change detail")
+}
