@@ -20,7 +20,6 @@ package objects
 
 import (
 	"fmt"
-	"strconv"
 	"strings"
 
 	"go.uber.org/zap"
@@ -30,6 +29,7 @@ import (
 	"github.com/apache/yunikorn-core/pkg/locking"
 	"github.com/apache/yunikorn-core/pkg/log"
 	"github.com/apache/yunikorn-core/pkg/plugins"
+	schedEvt "github.com/apache/yunikorn-core/pkg/scheduler/objects/events"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/common"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
 )
@@ -54,28 +54,20 @@ type Node struct {
 	availableResource *resources.Resource
 	allocations       map[string]*Allocation
 	schedulable       bool
-	ready             bool
 
 	reservations map[string]*reservation // a map of reservations
 	listeners    []NodeListener          // a list of node listeners
-	nodeEvents   *nodeEvents
+	nodeEvents   *schedEvt.NodeEvents
 
 	locking.RWMutex
 }
 
 func NewNode(proto *si.NodeInfo) *Node {
-	// safe guard against panic
+	// safeguard against panic
 	if proto == nil {
 		return nil
 	}
 
-	var ready bool
-	var err error
-	if ready, err = strconv.ParseBool(proto.Attributes[common.NodeReadyAttribute]); err != nil {
-		log.Log(log.SchedNode).Debug("Could not parse ready flag, assuming true",
-			zap.String("nodeID", proto.NodeID))
-		ready = true
-	}
 	sn := &Node{
 		NodeID:            proto.NodeID,
 		reservations:      make(map[string]*reservation),
@@ -85,10 +77,10 @@ func NewNode(proto *si.NodeInfo) *Node {
 		allocations:       make(map[string]*Allocation),
 		schedulable:       true,
 		listeners:         make([]NodeListener, 0),
-		ready:             ready,
 	}
-	sn.nodeEvents = newNodeEvents(sn, events.GetEventSystem())
+	sn.nodeEvents = schedEvt.NewNodeEvents(events.GetEventSystem())
 	// initialise available resources
+	var err error
 	sn.availableResource, err = resources.SubErrorNegative(sn.totalResource, sn.occupiedResource)
 	if err != nil {
 		log.Log(log.SchedNode).Error("New node created with no available resources",
@@ -163,18 +155,25 @@ func (sn *Node) GetCapacity() *resources.Resource {
 	return sn.totalResource.Clone()
 }
 
+// SetCapacity changes the node resource capacity and returns the resource delta.
+// The delta is positive for an increased capacity and negative for a decrease.
 func (sn *Node) SetCapacity(newCapacity *resources.Resource) *resources.Resource {
-	defer sn.notifyListeners()
+	var delta *resources.Resource
+	defer func() {
+		if delta != nil {
+			sn.notifyListeners()
+		}
+	}()
 	sn.Lock()
 	defer sn.Unlock()
 	if resources.Equals(sn.totalResource, newCapacity) {
 		log.Log(log.SchedNode).Debug("skip updating capacity, not changed")
 		return nil
 	}
-	delta := resources.Sub(newCapacity, sn.totalResource)
+	delta = resources.Sub(newCapacity, sn.totalResource)
 	sn.totalResource = newCapacity
 	sn.refreshAvailableResource()
-	sn.nodeEvents.sendNodeCapacityChangedEvent()
+	sn.nodeEvents.SendNodeCapacityChangedEvent(sn.NodeID, sn.totalResource.Clone())
 	return delta
 }
 
@@ -184,6 +183,14 @@ func (sn *Node) GetOccupiedResource() *resources.Resource {
 	return sn.occupiedResource.Clone()
 }
 
+func (sn *Node) UpdateAllocatedResource(delta *resources.Resource) {
+	sn.Lock()
+	defer sn.Unlock()
+	sn.allocatedResource.AddTo(delta)
+	sn.allocatedResource.Prune()
+	sn.refreshAvailableResource()
+}
+
 func (sn *Node) SetOccupiedResource(occupiedResource *resources.Resource) {
 	defer sn.notifyListeners()
 	sn.Lock()
@@ -193,7 +200,7 @@ func (sn *Node) SetOccupiedResource(occupiedResource *resources.Resource) {
 		return
 	}
 	sn.occupiedResource = occupiedResource
-	sn.nodeEvents.sendNodeOccupiedResourceChangedEvent()
+	sn.nodeEvents.SendNodeOccupiedResourceChangedEvent(sn.NodeID, sn.occupiedResource.Clone())
 	sn.refreshAvailableResource()
 }
 
@@ -203,6 +210,7 @@ func (sn *Node) refreshAvailableResource() {
 	sn.availableResource = sn.totalResource.Clone()
 	sn.availableResource.SubFrom(sn.allocatedResource)
 	sn.availableResource.SubFrom(sn.occupiedResource)
+	sn.availableResource.Prune()
 	// check if any quantity is negative: a nil resource is all 0's
 	if !resources.StrictlyGreaterThanOrEquals(sn.availableResource, nil) {
 		log.Log(log.SchedNode).Warn("Node update triggered over allocated node",
@@ -213,13 +221,13 @@ func (sn *Node) refreshAvailableResource() {
 	}
 }
 
-// Return the allocation based on the allocationID of the allocation.
+// Return the allocation based on the allocationKey of the allocation.
 // returns nil if the allocation is not found
-func (sn *Node) GetAllocation(allocationID string) *Allocation {
+func (sn *Node) GetAllocation(allocationKey string) *Allocation {
 	sn.RLock()
 	defer sn.RUnlock()
 
-	return sn.allocations[allocationID]
+	return sn.allocations[allocationKey]
 }
 
 // Get a copy of the allocations on this node
@@ -243,7 +251,7 @@ func (sn *Node) SetSchedulable(schedulable bool) {
 	sn.Lock()
 	defer sn.Unlock()
 	sn.schedulable = schedulable
-	sn.nodeEvents.sendNodeSchedulableChangedEvent(sn.schedulable)
+	sn.nodeEvents.SendNodeSchedulableChangedEvent(sn.NodeID, sn.schedulable)
 }
 
 // Can this node be used in scheduling.
@@ -262,8 +270,8 @@ func (sn *Node) GetAllocatedResource() *resources.Resource {
 
 // Get the available resource on this node.
 func (sn *Node) GetAvailableResource() *resources.Resource {
-	sn.Lock()
-	defer sn.Unlock()
+	sn.RLock()
+	defer sn.RUnlock()
 	return sn.availableResource.Clone()
 }
 
@@ -303,65 +311,88 @@ func (sn *Node) FitInNode(resRequest *resources.Resource) bool {
 // Returns nil if the allocation was not found and no changes are made. If the allocation
 // is found the Allocation removed is returned. Used resources will decrease available
 // will increase as per the allocation removed.
-func (sn *Node) RemoveAllocation(allocationID string) *Allocation {
+func (sn *Node) RemoveAllocation(allocationKey string) *Allocation {
 	defer sn.notifyListeners()
 	sn.Lock()
 	defer sn.Unlock()
 
-	alloc := sn.allocations[allocationID]
+	alloc := sn.allocations[allocationKey]
 	if alloc != nil {
-		delete(sn.allocations, allocationID)
+		delete(sn.allocations, allocationKey)
 		sn.allocatedResource.SubFrom(alloc.GetAllocatedResource())
+		sn.allocatedResource.Prune()
 		sn.availableResource.AddTo(alloc.GetAllocatedResource())
-		sn.nodeEvents.sendAllocationRemovedEvent(alloc.allocationKey, alloc.allocatedResource)
+		sn.nodeEvents.SendAllocationRemovedEvent(sn.NodeID, alloc.allocationKey, alloc.GetAllocatedResource())
 		return alloc
 	}
 
 	return nil
 }
 
+// TryAddAllocation attempts to add the allocation to the node. Used resources will increase available will decrease.
+// A nil Allocation makes no changes. Preempted resources must have been released already.
+// Do a sanity check to make sure it still fits in the node and nothing has changed
+func (sn *Node) TryAddAllocation(alloc *Allocation) bool {
+	return sn.addAllocationInternal(alloc, false)
+}
+
 // AddAllocation adds the allocation to the node. Used resources will increase available will decrease.
 // A nil Allocation makes no changes. Preempted resources must have been released already.
 // Do a sanity check to make sure it still fits in the node and nothing has changed
-func (sn *Node) AddAllocation(alloc *Allocation) bool {
+func (sn *Node) AddAllocation(alloc *Allocation) {
+	_ = sn.addAllocationInternal(alloc, true)
+}
+
+func (sn *Node) addAllocationInternal(alloc *Allocation, force bool) bool {
 	if alloc == nil {
 		return false
 	}
-	defer sn.notifyListeners()
+	result := false
+	defer func() {
+		// check result to ensure we don't notify listeners unnecessarily
+		if result {
+			sn.notifyListeners()
+		}
+	}()
+
 	sn.Lock()
 	defer sn.Unlock()
 	// check if this still fits: it might have changed since pre-check
 	res := alloc.GetAllocatedResource()
-	if sn.availableResource.FitIn(res) {
-		sn.allocations[alloc.GetAllocationID()] = alloc
+	if force || sn.availableResource.FitIn(res) {
+		sn.allocations[alloc.GetAllocationKey()] = alloc
 		sn.allocatedResource.AddTo(res)
 		sn.availableResource.SubFrom(res)
-		sn.nodeEvents.sendAllocationAddedEvent(alloc.allocationKey, res)
-		return true
+		sn.availableResource.Prune()
+		sn.nodeEvents.SendAllocationAddedEvent(sn.NodeID, alloc.allocationKey, res)
+		result = true
+		return result
 	}
-	return false
+	result = false
+	return result
 }
 
 // ReplaceAllocation replaces the placeholder with the real allocation on the node.
 // The delta passed in is the difference in resource usage between placeholder and real allocation.
 // It should always be a negative value or zero: it is a decrease in usage or no change
-func (sn *Node) ReplaceAllocation(allocationID string, replace *Allocation, delta *resources.Resource) {
+func (sn *Node) ReplaceAllocation(allocationKey string, replace *Allocation, delta *resources.Resource) {
 	defer sn.notifyListeners()
 	sn.Lock()
 	defer sn.Unlock()
 
-	replace.SetPlaceholderCreateTime(sn.allocations[allocationID].GetCreateTime())
-	delete(sn.allocations, allocationID)
+	replace.SetPlaceholderCreateTime(sn.allocations[allocationKey].GetCreateTime())
+	delete(sn.allocations, allocationKey)
 	replace.SetPlaceholderUsed(true)
-	sn.allocations[replace.GetAllocationID()] = replace
+	sn.allocations[replace.GetAllocationKey()] = replace
 	before := sn.allocatedResource.Clone()
 	// The allocatedResource and availableResource should be updated in the same way
 	sn.allocatedResource.AddTo(delta)
 	sn.availableResource.SubFrom(delta)
+	sn.availableResource.Prune()
 	if !before.FitIn(sn.allocatedResource) {
 		log.Log(log.SchedNode).Warn("unexpected increase in node usage after placeholder replacement",
-			zap.String("placeholder allocationID", allocationID),
-			zap.String("allocation allocationID", replace.GetAllocationID()),
+			zap.String("placeholder allocationKey", allocationKey),
+			zap.String("allocation allocationKey", replace.GetAllocationKey()),
 			zap.Stringer("delta", delta))
 	}
 }
@@ -375,12 +406,12 @@ func (sn *Node) CanAllocate(res *resources.Resource) bool {
 }
 
 // Checking pre-conditions in the shim for an allocation.
-func (sn *Node) preAllocateConditions(ask *AllocationAsk) bool {
+func (sn *Node) preAllocateConditions(ask *Allocation) error {
 	return sn.preConditions(ask, true)
 }
 
 // Checking pre-conditions in the shim for a reservation.
-func (sn *Node) preReserveConditions(ask *AllocationAsk) bool {
+func (sn *Node) preReserveConditions(ask *Allocation) error {
 	return sn.preConditions(ask, false)
 }
 
@@ -390,30 +421,29 @@ func (sn *Node) preReserveConditions(ask *AllocationAsk) bool {
 // The caller must thus not rely on all plugins being executed.
 // This is a lock free call as it does not change the node and multiple predicate checks could be
 // run at the same time.
-func (sn *Node) preConditions(ask *AllocationAsk, allocate bool) bool {
+func (sn *Node) preConditions(ask *Allocation, allocate bool) error {
 	// Check the predicates plugin (k8shim)
-	allocID := ask.GetAllocationKey()
+	allocationKey := ask.GetAllocationKey()
 	if plugin := plugins.GetResourceManagerCallbackPlugin(); plugin != nil {
 		// checking predicates
 		if err := plugin.Predicates(&si.PredicatesArgs{
-			AllocationKey: allocID,
+			AllocationKey: allocationKey,
 			NodeID:        sn.NodeID,
 			Allocate:      allocate,
 		}); err != nil {
 			log.Log(log.SchedNode).Debug("running predicates failed",
-				zap.String("allocationKey", allocID),
+				zap.String("allocationKey", allocationKey),
 				zap.String("nodeID", sn.NodeID),
 				zap.Bool("allocateFlag", allocate),
 				zap.Error(err))
 			// running predicates failed
 			msg := err.Error()
 			ask.LogAllocationFailure(msg, allocate)
-			ask.SendPredicateFailedEvent(msg)
-			return false
+			return err
 		}
 	}
 	// all predicate plugins passed
-	return true
+	return nil
 }
 
 // preAllocateCheck checks if the node should be considered as a possible node to allocate on.
@@ -472,7 +502,7 @@ func (sn *Node) isReservedForApp(key string) bool {
 // Reserve the node for this application and ask combination, if not reserved yet.
 // The reservation is checked against the node resources.
 // If the reservation fails the function returns false, if the reservation is made it returns true.
-func (sn *Node) Reserve(app *Application, ask *AllocationAsk) error {
+func (sn *Node) Reserve(app *Application, ask *Allocation) error {
 	defer sn.notifyListeners()
 	sn.Lock()
 	defer sn.Unlock()
@@ -499,7 +529,7 @@ func (sn *Node) Reserve(app *Application, ask *AllocationAsk) error {
 		return fmt.Errorf("reservation does not fit on node %s, appID %s, ask %s", sn.NodeID, app.ApplicationID, ask.GetAllocatedResource().String())
 	}
 	sn.reservations[appReservation.getKey()] = appReservation
-	sn.nodeEvents.sendReservedEvent(ask.GetAllocatedResource(), ask.GetAllocationKey())
+	sn.nodeEvents.SendReservedEvent(sn.NodeID, ask.GetAllocatedResource(), ask.GetAllocationKey())
 	// reservation added successfully
 	return nil
 }
@@ -507,7 +537,7 @@ func (sn *Node) Reserve(app *Application, ask *AllocationAsk) error {
 // unReserve the node for this application and ask combination
 // If the reservation does not exist it returns 0 for reservations removed, if the reservation is removed it returns 1.
 // The error is set if the reservation key cannot be generated.
-func (sn *Node) unReserve(app *Application, ask *AllocationAsk) (int, error) {
+func (sn *Node) unReserve(app *Application, ask *Allocation) (int, error) {
 	defer sn.notifyListeners()
 	sn.Lock()
 	defer sn.Unlock()
@@ -521,7 +551,7 @@ func (sn *Node) unReserve(app *Application, ask *AllocationAsk) (int, error) {
 	}
 	if _, ok := sn.reservations[resKey]; ok {
 		delete(sn.reservations, resKey)
-		sn.nodeEvents.sendUnreservedEvent(ask.GetAllocatedResource(), ask.GetAllocationKey())
+		sn.nodeEvents.SendUnreservedEvent(sn.NodeID, ask.GetAllocatedResource(), ask.GetAllocationKey())
 		return 1, nil
 	}
 	// reservation was not found
@@ -595,23 +625,12 @@ func (sn *Node) getListeners() []NodeListener {
 	return list
 }
 
-func (sn *Node) IsReady() bool {
+func (sn *Node) SendNodeAddedEvent() {
 	sn.RLock()
 	defer sn.RUnlock()
-	return sn.ready
-}
-
-func (sn *Node) SetReady(ready bool) {
-	sn.Lock()
-	defer sn.Unlock()
-	sn.ready = ready
-	sn.nodeEvents.sendNodeReadyChangedEvent(sn.ready)
-}
-
-func (sn *Node) SendNodeAddedEvent() {
-	sn.nodeEvents.sendNodeAddedEvent()
+	sn.nodeEvents.SendNodeAddedEvent(sn.NodeID, sn.totalResource.Clone())
 }
 
 func (sn *Node) SendNodeRemovedEvent() {
-	sn.nodeEvents.sendNodeRemovedEvent()
+	sn.nodeEvents.SendNodeRemovedEvent(sn.NodeID)
 }
