@@ -28,6 +28,7 @@ import (
 	evtMock "github.com/apache/yunikorn-core/pkg/events/mock"
 	"github.com/apache/yunikorn-core/pkg/mock"
 	"github.com/apache/yunikorn-core/pkg/plugins"
+	schedEvt "github.com/apache/yunikorn-core/pkg/scheduler/objects/events"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/common"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
 )
@@ -103,11 +104,9 @@ func TestCheckConditions(t *testing.T) {
 	// Check if we can allocate on scheduling node (no plugins)
 	res := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 1})
 	ask := newAllocationAsk("test", "app001", res)
-	if !node.preAllocateConditions(ask) {
+	if node.preAllocateConditions(ask) != nil {
 		t.Error("node with scheduling set to true no plugins should allow allocation")
 	}
-
-	// TODO add mock for plugin to extend tests
 }
 
 func TestPreAllocateCheck(t *testing.T) {
@@ -186,7 +185,7 @@ func TestCanAllocate(t *testing.T) {
 			sn := &Node{
 				availableResource: tt.available,
 			}
-			assert.Equal(t, sn.CanAllocate(tt.request), tt.want, "unexpected node can run result")
+			assert.Equal(t, sn.CanAllocate(tt.request), tt.want, "unexpected node can run resultType")
 		})
 	}
 }
@@ -391,17 +390,48 @@ func TestAddAllocation(t *testing.T) {
 	}
 
 	// check nil alloc
-	assert.Assert(t, !node.AddAllocation(nil), "nil allocation should not have been added: %v", node)
+	node.AddAllocation(nil)
+	assert.Assert(t, resources.IsZero(node.GetAllocatedResource()), "nil allocation should not have been added: %v", node)
+
+	// check alloc that is over quota
+	large := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 200})
+	alloc := newAllocation(appID1, nodeID1, large)
+	node.AddAllocation(alloc)
+	if !resources.Equals(node.GetAllocatedResource(), large) {
+		t.Errorf("failed to add large allocation expected %v, got %v", large, node.GetAllocatedResource())
+	}
+	node.RemoveAllocation(alloc.GetAllocationKey())
+	assert.Assert(t, resources.IsZero(node.GetAllocatedResource()), "removal of large allocation should return to zero %v, got %v", node, node.GetAllocatedResource())
+
+	// check alloc that is over quota
+	half := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 50, "second": 100})
+	alloc = newAllocation(appID1, nodeID1, half)
+	node.AddAllocation(alloc)
+	if !resources.Equals(node.GetAllocatedResource(), half) {
+		t.Errorf("failed to add half allocation expected %v, got %v", half, node.GetAllocatedResource())
+	}
+	node.RemoveAllocation(alloc.GetAllocationKey())
+	assert.Assert(t, resources.IsZero(node.GetAllocatedResource()), "removal of half allocation should return to zero %v, got %v", node, node.GetAllocatedResource())
+}
+
+func TestTryAddAllocation(t *testing.T) {
+	node := newNode("node-123", map[string]resources.Quantity{"first": 100, "second": 200})
+	if !resources.IsZero(node.GetAllocatedResource()) {
+		t.Fatal("Failed to initialize resource")
+	}
+
+	// check nil alloc
+	assert.Assert(t, !node.TryAddAllocation(nil), "nil allocation should not have been added: %v", node)
 	// check alloc that does not match
 	unknown := resources.NewResourceFromMap(map[string]resources.Quantity{"unknown": 1})
 	alloc := newAllocation(appID1, nodeID1, unknown)
-	assert.Assert(t, !node.AddAllocation(alloc), "unmatched resource type in allocation should not have been added: %v", node)
+	assert.Assert(t, !node.TryAddAllocation(alloc), "unmatched resource type in allocation should not have been added: %v", node)
 
 	// allocate half of the resources available and check the calculation
 	half := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 50, "second": 100})
 	alloc = newAllocation(appID1, nodeID1, half)
-	assert.Assert(t, node.AddAllocation(alloc), "add allocation 1 should not have failed")
-	if node.GetAllocation(alloc.GetAllocationID()) == nil {
+	assert.Assert(t, node.TryAddAllocation(alloc), "add allocation 1 should not have failed")
+	if node.GetAllocation(alloc.GetAllocationKey()) == nil {
 		t.Fatal("failed to add allocations: allocation not returned")
 	}
 	if !resources.Equals(node.GetAllocatedResource(), half) {
@@ -417,8 +447,8 @@ func TestAddAllocation(t *testing.T) {
 	// second and check calculation
 	piece := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 25, "second": 50})
 	alloc = newAllocation(appID1, nodeID1, piece)
-	assert.Assert(t, node.AddAllocation(alloc), "add allocation 2 should not have failed")
-	if node.GetAllocation(alloc.GetAllocationID()) == nil {
+	assert.Assert(t, node.TryAddAllocation(alloc), "add allocation 2 should not have failed")
+	if node.GetAllocation(alloc.GetAllocationKey()) == nil {
 		t.Fatal("failed to add allocations: allocation not returned")
 	}
 	piece.AddTo(half)
@@ -445,7 +475,7 @@ func TestRemoveAllocation(t *testing.T) {
 	half := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 50, "second": 100})
 	alloc1 := newAllocation(appID1, nodeID1, half)
 	node.AddAllocation(alloc1)
-	if node.GetAllocation(alloc1.GetAllocationID()) == nil {
+	if node.GetAllocation(alloc1.GetAllocationKey()) == nil {
 		t.Fatal("failed to add allocations: allocation not returned")
 	}
 	// check empty alloc
@@ -463,10 +493,10 @@ func TestRemoveAllocation(t *testing.T) {
 	piece := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 25, "second": 50})
 	alloc2 := newAllocation(appID1, nodeID1, piece)
 	node.AddAllocation(alloc2)
-	if node.GetAllocation(alloc2.GetAllocationID()) == nil {
+	if node.GetAllocation(alloc2.GetAllocationKey()) == nil {
 		t.Fatal("failed to add allocations: allocation not returned")
 	}
-	alloc := node.RemoveAllocation(alloc1.GetAllocationID())
+	alloc := node.RemoveAllocation(alloc1.GetAllocationKey())
 	if alloc == nil {
 		t.Error("allocation should have been removed but was not")
 	}
@@ -489,9 +519,9 @@ func TestNodeReplaceAllocation(t *testing.T) {
 
 	// allocate half of the resources available and check the calculation
 	half := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 50, "second": 100})
-	ph := newPlaceholderAlloc(appID1, nodeID1, half)
+	ph := newPlaceholderAlloc(appID1, nodeID1, half, "tg")
 	node.AddAllocation(ph)
-	assert.Assert(t, node.GetAllocation(ph.GetAllocationID()) != nil, "failed to add placeholder allocation")
+	assert.Assert(t, node.GetAllocation(ph.GetAllocationKey()) != nil, "failed to add placeholder allocation")
 	assert.Assert(t, resources.Equals(node.GetAllocatedResource(), half), "allocated resource not set correctly %v got %v", half, node.GetAllocatedResource())
 	assert.Assert(t, resources.Equals(node.GetAvailableResource(), half), "available resource not set correctly %v got %v", half, node.GetAvailableResource())
 
@@ -501,17 +531,31 @@ func TestNodeReplaceAllocation(t *testing.T) {
 	delta := resources.Sub(piece, half)
 	assert.Assert(t, delta.HasNegativeValue(), "expected negative values in delta")
 	// swap and check the calculation
-	node.ReplaceAllocation(ph.GetAllocationID(), alloc, delta)
-	assert.Assert(t, node.GetAllocation(alloc.GetAllocationID()) != nil, "failed to replace allocation: allocation not returned")
+	node.ReplaceAllocation(ph.GetAllocationKey(), alloc, delta)
+	assert.Assert(t, node.GetAllocation(alloc.GetAllocationKey()) != nil, "failed to replace allocation: allocation not returned")
 	assert.Assert(t, resources.Equals(node.GetAllocatedResource(), piece), "allocated resource not set correctly %v got %v", piece, node.GetAllocatedResource())
 	assert.Assert(t, resources.Equals(node.GetAvailableResource(), resources.Sub(node.GetCapacity(), piece)), "available resource not set correctly %v got %v", resources.Sub(node.GetCapacity(), piece), node.GetAvailableResource())
 
 	// clean up all should be zero
-	assert.Assert(t, node.RemoveAllocation(alloc.GetAllocationID()) != nil, "allocation should have been removed but was not")
+	assert.Assert(t, node.RemoveAllocation(alloc.GetAllocationKey()) != nil, "allocation should have been removed but was not")
 	assert.Assert(t, resources.IsZero(node.GetAllocatedResource()), "allocated resource not updated correctly")
 	assert.Assert(t, resources.Equals(node.GetAvailableResource(), node.GetCapacity()), "available resource not set correctly %v got %v", node.GetCapacity(), node.GetAvailableResource())
 }
 
+func TestNodeUpdateAllocatedResource(t *testing.T) {
+	node := newNode("node-123", map[string]resources.Quantity{"first": 100, "second": 200})
+	assert.Assert(t, resources.IsZero(node.GetAllocatedResource()), "failed to initialize node")
+	piece := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 25, "second": 50})
+	alloc := newAllocation(appID1, nodeID1, piece)
+	node.AddAllocation(alloc)
+	assert.Assert(t, resources.Equals(piece, node.GetAllocatedResource()), "node resources not set correctly")
+
+	delta := resources.NewResourceFromMap(map[string]resources.Quantity{"first": -10, "second": 10})
+	diff := resources.Add(piece, delta)
+	node.UpdateAllocatedResource(delta)
+	assert.Assert(t, resources.Equals(diff, node.GetAllocatedResource()), "node resources not updated correctly")
+}
+
 func TestGetAllocation(t *testing.T) {
 	node := newNode("node-123", map[string]resources.Quantity{"first": 100, "second": 200})
 	if !resources.IsZero(node.GetAllocatedResource()) {
@@ -524,14 +568,14 @@ func TestGetAllocation(t *testing.T) {
 	}
 	alloc = newAllocation(appID1, nodeID1, nil)
 	node.AddAllocation(alloc)
-	alloc = node.GetAllocation(alloc.GetAllocationID())
+	alloc = node.GetAllocation(alloc.GetAllocationKey())
 	if alloc == nil {
 		t.Fatalf("allocation should have been found")
 	}
 	// unknown allocation get a nil
 	alloc = node.GetAllocation("fake")
 	if alloc != nil {
-		t.Fatalf("allocation should not have been found (fake ID)")
+		t.Fatalf("allocation should not have been found (fake key)")
 	}
 }
 
@@ -656,37 +700,6 @@ func TestAddRemoveListener(t *testing.T) {
 	assert.Equal(t, 1, tl.updateCount, "listener should not have fired again")
 }
 
-func TestReadyAttribute(t *testing.T) {
-	// missing
-	proto := newProto(testNode, nil, nil, nil)
-	node := NewNode(proto)
-	assert.Equal(t, true, node.ready, "Node should be in ready state")
-
-	// exists, but faulty
-	attr := map[string]string{
-		"readyX": "true",
-	}
-	proto = newProto(testNode, nil, nil, attr)
-	node = NewNode(proto)
-	assert.Equal(t, true, node.ready, "Node should be in ready state")
-
-	// exists, true
-	attr = map[string]string{
-		"ready": "true",
-	}
-	proto = newProto(testNode, nil, nil, attr)
-	node = NewNode(proto)
-	assert.Equal(t, true, node.ready, "Node should be in ready state")
-
-	// exists, false
-	attr = map[string]string{
-		"ready": "false",
-	}
-	proto = newProto(testNode, nil, nil, attr)
-	node = NewNode(proto)
-	assert.Equal(t, false, node.ready, "Node should not be in ready state")
-}
-
 func TestNodeEvents(t *testing.T) {
 	mockEvents := evtMock.NewEventSystem()
 	total := resources.NewResourceFromMap(map[string]resources.Quantity{"cpu": 100, "memory": 100})
@@ -695,7 +708,7 @@ func TestNodeEvents(t *testing.T) {
 		"ready": "true",
 	})
 	node := NewNode(proto)
-	node.nodeEvents = newNodeEvents(node, mockEvents)
+	node.nodeEvents = schedEvt.NewNodeEvents(mockEvents)
 
 	node.SendNodeAddedEvent()
 	assert.Equal(t, 1, len(mockEvents.Events))
@@ -710,19 +723,10 @@ func TestNodeEvents(t *testing.T) {
 	assert.Equal(t, si.EventRecord_NODE, event.Type)
 	assert.Equal(t, si.EventRecord_REMOVE, event.EventChangeType)
 
-	mockEvents.Reset()
-	node.SetReady(false)
-	assert.Equal(t, 1, len(mockEvents.Events))
-	event = mockEvents.Events[0]
-	assert.Equal(t, si.EventRecord_NODE, event.Type)
-	assert.Equal(t, si.EventRecord_SET, event.EventChangeType)
-	assert.Equal(t, si.EventRecord_NODE_READY, event.EventChangeDetail)
-
 	mockEvents.Reset()
 	node.AddAllocation(&Allocation{
 		allocatedResource: resources.NewResourceFromMap(map[string]resources.Quantity{"cpu": 1, "memory": 1}),
 		allocationKey:     aKey,
-		allocationID:      "allocationid-0",
 	})
 	assert.Equal(t, 1, len(mockEvents.Events))
 	event = mockEvents.Events[0]
@@ -731,7 +735,7 @@ func TestNodeEvents(t *testing.T) {
 	assert.Equal(t, si.EventRecord_NODE_ALLOC, event.EventChangeDetail)
 
 	mockEvents.Reset()
-	node.RemoveAllocation("allocationid-0")
+	node.RemoveAllocation(aKey)
 	event = mockEvents.Events[0]
 	assert.Equal(t, si.EventRecord_NODE, event.Type)
 	assert.Equal(t, si.EventRecord_REMOVE, event.EventChangeType)
@@ -805,16 +809,13 @@ func TestNode_FitInNode(t *testing.T) {
 			sn := &Node{
 				totalResource: tt.totalRes,
 			}
-			assert.Equal(t, sn.FitInNode(tt.resRequest), tt.want, "unexpected node fit result")
+			assert.Equal(t, sn.FitInNode(tt.resRequest), tt.want, "unexpected node fit resultType")
 		})
 	}
 }
 
 func TestPreconditions(t *testing.T) {
-	current := plugins.GetResourceManagerCallbackPlugin()
-	defer func() {
-		plugins.RegisterSchedulerPlugin(current)
-	}()
+	defer plugins.UnregisterSchedulerPlugins()
 
 	plugins.RegisterSchedulerPlugin(mock.NewPredicatePlugin(true, map[string]int{}))
 	total := resources.NewResourceFromMap(map[string]resources.Quantity{"cpu": 100, "memory": 100})
@@ -824,21 +825,17 @@ func TestPreconditions(t *testing.T) {
 	})
 	res := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 1})
 	ask := newAllocationAsk("test", "app001", res)
-	eventSystem := evtMock.NewEventSystem()
-	ask.askEvents = newAskEvents(ask, eventSystem)
 	node := NewNode(proto)
 
 	// failure
-	node.preConditions(ask, true)
-	assert.Equal(t, 1, len(eventSystem.Events))
-	assert.Equal(t, "Predicate failed for request 'test' with message: 'fake predicate plugin failed'", eventSystem.Events[0].Message)
+	err := node.preConditions(ask, true)
+	assert.ErrorContains(t, err, "fake predicate plugin failed")
 	assert.Equal(t, 1, len(ask.allocLog))
 	assert.Equal(t, "fake predicate plugin failed", ask.allocLog["fake predicate plugin failed"].Message)
 
 	// pass
-	eventSystem.Reset()
 	plugins.RegisterSchedulerPlugin(mock.NewPredicatePlugin(false, map[string]int{}))
-	node.preConditions(ask, true)
-	assert.Equal(t, 0, len(eventSystem.Events))
+	err = node.preConditions(ask, true)
+	assert.NilError(t, err)
 	assert.Equal(t, 1, len(ask.allocLog))
 }
