@@ -37,11 +37,11 @@ import (
 	"github.com/apache/yunikorn-core/pkg/locking"
 	"github.com/apache/yunikorn-core/pkg/log"
 	"github.com/apache/yunikorn-core/pkg/metrics"
+	schedEvt "github.com/apache/yunikorn-core/pkg/scheduler/objects/events"
 	"github.com/apache/yunikorn-core/pkg/scheduler/objects/template"
 	"github.com/apache/yunikorn-core/pkg/scheduler/policies"
 	"github.com/apache/yunikorn-core/pkg/scheduler/ugm"
 	"github.com/apache/yunikorn-core/pkg/webservice/dao"
-	siCommon "github.com/apache/yunikorn-scheduler-interface/lib/go/common"
 )
 
 var (
@@ -87,7 +87,7 @@ type Queue struct {
 	runningApps            uint64
 	allocatingAcceptedApps map[string]bool
 	template               *template.Template
-	queueEvents            *queueEvents
+	queueEvents            *schedEvt.QueueEvents
 
 	locking.RWMutex
 }
@@ -125,7 +125,7 @@ func NewConfiguredQueue(conf configs.QueueConfig, parent *Queue) (*Queue, error)
 
 	// update the properties
 	if err := sq.applyConf(conf); err != nil {
-		return nil, fmt.Errorf("configured queue creation failed: %w", err)
+		return nil, errors.Join(errors.New("configured queue creation failed: "), err)
 	}
 
 	// add to the parent, we might have an overall lock already
@@ -137,15 +137,15 @@ func NewConfiguredQueue(conf configs.QueueConfig, parent *Queue) (*Queue, error)
 		sq.QueuePath = parent.QueuePath + configs.DOT + sq.Name
 		err := parent.addChildQueue(sq)
 		if err != nil {
-			return nil, fmt.Errorf("configured queue creation failed: %w", err)
+			return nil, errors.Join(errors.New("configured queue creation failed: "), err)
 		}
 	} else {
 		sq.UpdateQueueProperties()
 	}
-	sq.queueEvents = newQueueEvents(events.GetEventSystem())
+	sq.queueEvents = schedEvt.NewQueueEvents(events.GetEventSystem())
 	log.Log(log.SchedQueue).Info("configured queue added to scheduler",
 		zap.String("queueName", sq.QueuePath))
-	sq.queueEvents.sendNewQueueEvent(sq.QueuePath, sq.isManaged)
+	sq.queueEvents.SendNewQueueEvent(sq.QueuePath, sq.isManaged)
 
 	return sq, nil
 }
@@ -178,8 +178,11 @@ func NewDynamicQueue(name string, leaf bool, parent *Queue) (*Queue, error) {
 		return nil, fmt.Errorf("dynamic queue can not be added without parent: %s", name)
 	}
 	// name might not be checked do it here
-	if !configs.QueueNameRegExp.MatchString(name) {
-		return nil, fmt.Errorf("invalid queue name '%s', a name must only have alphanumeric characters, - or _, and be no longer than 64 characters", name)
+	if err := configs.IsQueueNameValid(name); err != nil {
+		return nil, err
+	}
+	if name == common.RecoveryQueue {
+		return nil, fmt.Errorf("dynamic queue cannot be root.@recovery@")
 	}
 	return newDynamicQueueInternal(name, leaf, parent)
 }
@@ -196,14 +199,14 @@ func newDynamicQueueInternal(name string, leaf bool, parent *Queue) (*Queue, err
 	// still need to make sure we lock the parent so we do not interfere with scheduling
 	err := parent.addChildQueue(sq)
 	if err != nil {
-		return nil, fmt.Errorf("dynamic queue creation failed: %w", err)
+		return nil, errors.Join(errors.New("dynamic queue creation failed: "), err)
 	}
 
 	sq.UpdateQueueProperties()
-	sq.queueEvents = newQueueEvents(events.GetEventSystem())
+	sq.queueEvents = schedEvt.NewQueueEvents(events.GetEventSystem())
 	log.Log(log.SchedQueue).Info("dynamic queue added to scheduler",
 		zap.String("queueName", sq.QueuePath))
-	sq.queueEvents.sendNewQueueEvent(sq.QueuePath, sq.isManaged)
+	sq.queueEvents.SendNewQueueEvent(sq.QueuePath, sq.isManaged)
 
 	return sq, nil
 }
@@ -330,6 +333,14 @@ func (sq *Queue) applyConf(conf configs.QueueConfig) error {
 		sq.isManaged = true
 	}
 
+	// if the queue is marked for removal reverse that state
+	if !sq.IsRunning() {
+		err = sq.handleQueueEvent(Start)
+		if err != nil {
+			log.Log(log.SchedQueue).Info("managed queue state change failed",
+				zap.String("queue", sq.QueuePath))
+		}
+	}
 	prevLeaf := sq.isLeaf
 	sq.isLeaf = !conf.Parent
 	// Make sure the parent flag is set correctly: config might expect auto parent type creation
@@ -338,28 +349,29 @@ func (sq *Queue) applyConf(conf configs.QueueConfig) error {
 	}
 
 	if prevLeaf != sq.isLeaf && sq.queueEvents != nil {
-		sq.queueEvents.sendTypeChangedEvent(sq.QueuePath, sq.isLeaf)
+		sq.queueEvents.SendTypeChangedEvent(sq.QueuePath, sq.isLeaf)
 	}
 
 	if !sq.isLeaf {
 		if err = sq.setTemplate(conf.ChildTemplate); err != nil {
-			return nil
+			return err
 		}
 	}
 
-	// Load the max & guaranteed resources for all but the root queue
+	// Load the max & guaranteed resources and maxApps for all but the root queue
 	if sq.Name != configs.RootQueue {
-		if err = sq.setResources(conf.Resources); err != nil {
+		if err = sq.setResourcesFromConf(conf.Resources); err != nil {
 			return err
 		}
+		sq.maxRunningApps = conf.MaxApplications
 	}
 
 	sq.properties = conf.Properties
 	return nil
 }
 
-// setResources sets the maxResource and guaranteedResource of the queue from the config.
-func (sq *Queue) setResources(resource configs.Resources) error {
+// setResourcesFromConf sets the maxResource and guaranteedResource of the queue from the config.
+func (sq *Queue) setResourcesFromConf(resource configs.Resources) error {
 	maxResource, err := resources.NewResourceFromConf(resource.Max)
 	if err != nil {
 		log.Log(log.SchedQueue).Error("parsing failed on max resources this should not happen",
@@ -376,7 +388,11 @@ func (sq *Queue) setResources(resource configs.Resources) error {
 			zap.Error(err))
 		return err
 	}
+	sq.setResources(guaranteedResource, maxResource)
+	return nil
+}
 
+func (sq *Queue) setResources(guaranteedResource, maxResource *resources.Resource) {
 	switch {
 	case resources.StrictlyGreaterThanZero(maxResource):
 		log.Log(log.SchedQueue).Debug("setting max resources",
@@ -384,7 +400,7 @@ func (sq *Queue) setResources(resource configs.Resources) error {
 			zap.Stringer("current", sq.maxResource),
 			zap.Stringer("new", maxResource))
 		if !resources.Equals(sq.maxResource, maxResource) && sq.queueEvents != nil {
-			sq.queueEvents.sendMaxResourceChangedEvent(sq.QueuePath, maxResource)
+			sq.queueEvents.SendMaxResourceChangedEvent(sq.QueuePath, maxResource)
 		}
 		sq.maxResource = maxResource
 		sq.updateMaxResourceMetrics()
@@ -394,7 +410,7 @@ func (sq *Queue) setResources(resource configs.Resources) error {
 			zap.Stringer("current", sq.maxResource),
 			zap.Stringer("new", maxResource))
 		if sq.queueEvents != nil {
-			sq.queueEvents.sendMaxResourceChangedEvent(sq.QueuePath, maxResource)
+			sq.queueEvents.SendMaxResourceChangedEvent(sq.QueuePath, maxResource)
 		}
 		sq.maxResource = nil
 		sq.updateMaxResourceMetrics()
@@ -410,7 +426,7 @@ func (sq *Queue) setResources(resource configs.Resources) error {
 			zap.Stringer("current", sq.guaranteedResource),
 			zap.Stringer("new", guaranteedResource))
 		if !resources.Equals(sq.guaranteedResource, guaranteedResource) && sq.queueEvents != nil {
-			sq.queueEvents.sendGuaranteedResourceChangedEvent(sq.QueuePath, guaranteedResource)
+			sq.queueEvents.SendGuaranteedResourceChangedEvent(sq.QueuePath, guaranteedResource)
 		}
 		sq.guaranteedResource = guaranteedResource
 		sq.updateGuaranteedResourceMetrics()
@@ -420,7 +436,7 @@ func (sq *Queue) setResources(resource configs.Resources) error {
 			zap.Stringer("current", sq.guaranteedResource),
 			zap.Stringer("new", guaranteedResource))
 		if sq.queueEvents != nil {
-			sq.queueEvents.sendGuaranteedResourceChangedEvent(sq.QueuePath, guaranteedResource)
+			sq.queueEvents.SendGuaranteedResourceChangedEvent(sq.QueuePath, guaranteedResource)
 		}
 		sq.guaranteedResource = nil
 		sq.updateGuaranteedResourceMetrics()
@@ -428,7 +444,22 @@ func (sq *Queue) setResources(resource configs.Resources) error {
 		log.Log(log.SchedQueue).Debug("guaranteed resources setting ignored: cannot set zero guaranteed resources",
 			zap.String("queue", sq.QueuePath))
 	}
-	return nil
+}
+
+func (sq *Queue) SetResources(guaranteedResource, maxResource *resources.Resource) {
+	sq.Lock()
+	defer sq.Unlock()
+	sq.setResources(guaranteedResource, maxResource)
+}
+
+// SetMaxRunningApps allows setting the maximum running apps on a queue
+func (sq *Queue) SetMaxRunningApps(maxApps uint64) {
+	if sq == nil {
+		return
+	}
+	sq.Lock()
+	defer sq.Unlock()
+	sq.maxRunningApps = maxApps
 }
 
 // setTemplate sets the template on the queue based on the config.
@@ -579,6 +610,13 @@ func (sq *Queue) GetGuaranteedResource() *resources.Resource {
 	return sq.guaranteedResource
 }
 
+// GetMaxApps returns the maximum number of applications that can run in this queue.
+func (sq *Queue) GetMaxApps() uint64 {
+	sq.RLock()
+	defer sq.RUnlock()
+	return sq.maxRunningApps
+}
+
 // GetActualGuaranteedResources returns the actual (including parent) guaranteed resources for the queue.
 func (sq *Queue) GetActualGuaranteedResource() *resources.Resource {
 	if sq == nil {
@@ -587,7 +625,7 @@ func (sq *Queue) GetActualGuaranteedResource() *resources.Resource {
 	parentGuaranteed := sq.parent.GetActualGuaranteedResource()
 	sq.RLock()
 	defer sq.RUnlock()
-	return resources.ComponentWiseMinPermissive(sq.guaranteedResource, parentGuaranteed)
+	return resources.ComponentWiseMin(sq.guaranteedResource, parentGuaranteed)
 }
 
 func (sq *Queue) GetPreemptionDelay() time.Duration {
@@ -626,14 +664,14 @@ func (sq *Queue) CheckAdminAccess(user security.UserGroup) bool {
 }
 
 // GetPartitionQueueDAOInfo returns the queue hierarchy as an object for a REST call.
-// Exclude is true, which means that returns the specified queue object, but does not return the children of the specified queue.
-func (sq *Queue) GetPartitionQueueDAOInfo(exclude bool) dao.PartitionQueueDAOInfo {
+// Include is false, which means that returns the specified queue object, but does not return the children of the specified queue.
+func (sq *Queue) GetPartitionQueueDAOInfo(include bool) dao.PartitionQueueDAOInfo {
 	queueInfo := dao.PartitionQueueDAOInfo{}
 	children := sq.GetCopyOfChildren()
-	if !exclude {
+	if include {
 		queueInfo.Children = make([]dao.PartitionQueueDAOInfo, 0, len(children))
 		for _, child := range children {
-			queueInfo.Children = append(queueInfo.Children, child.GetPartitionQueueDAOInfo(false))
+			queueInfo.Children = append(queueInfo.Children, child.GetPartitionQueueDAOInfo(true))
 		}
 	}
 	// we have held the read lock so following method should not take lock again.
@@ -642,7 +680,7 @@ func (sq *Queue) GetPartitionQueueDAOInfo(exclude bool) dao.PartitionQueueDAOInf
 	defer sq.RUnlock()
 
 	for _, child := range children {
-		queueInfo.ChildrenNames = append(queueInfo.ChildrenNames, child.QueuePath)
+		queueInfo.ChildNames = append(queueInfo.ChildNames, child.QueuePath)
 	}
 	queueInfo.QueueName = sq.QueuePath
 	queueInfo.Status = sq.stateMachine.Current()
@@ -717,6 +755,11 @@ func (sq *Queue) decPendingResource(delta *resources.Resource) {
 			zap.Error(err))
 	} else {
 		sq.updatePendingResourceMetrics()
+		// We should update the metrics before pruning the resource.
+		// For example:
+		// If we prune the resource first and the resource become nil after pruning,
+		// the metrics will not be updated with nil resource, this is not expected.
+		sq.pending.Prune()
 	}
 }
 
@@ -728,69 +771,7 @@ func (sq *Queue) AddApplication(app *Application) {
 	defer sq.Unlock()
 	appID := app.ApplicationID
 	sq.applications[appID] = app
-	sq.queueEvents.sendNewApplicationEvent(sq.QueuePath, appID)
-	if common.IsRecoveryQueue(sq.QueuePath) {
-		// don't set tag-based resources on the recovery queue
-		return
-	}
-
-	// YUNIKORN-199: update the quota from the namespace
-	// get the tag with the quota
-	quota := app.GetTag(siCommon.AppTagNamespaceResourceQuota)
-	// get the tag with the guaranteed resource
-	guaranteed := app.GetTag(siCommon.AppTagNamespaceResourceGuaranteed)
-	if quota == "" && guaranteed == "" {
-		return
-	}
-
-	var quotaRes, guaranteedRes *resources.Resource
-	var quotaErr, guaranteedErr error
-
-	// need to set a quota: convert json string to resource
-	if quota != "" {
-		quotaRes, quotaErr = resources.NewResourceFromString(quota)
-		if quotaErr != nil {
-			log.Log(log.SchedQueue).Warn("application resource quota conversion failure",
-				zap.String("json quota string", quota),
-				zap.Error(quotaErr))
-		} else if !resources.StrictlyGreaterThanZero(quotaRes) {
-			log.Log(log.SchedQueue).Warn("Max resource quantities should be greater than zero: cannot set queue max resource",
-				zap.Stringer("maxResource", quotaRes))
-			quotaRes = nil // Skip setting quota if it has a value <= 0
-		}
-	}
-
-	// need to set guaranteed resource: convert json string to resource
-	if guaranteed != "" {
-		guaranteedRes, guaranteedErr = resources.NewResourceFromString(guaranteed)
-		if guaranteedErr != nil {
-			log.Log(log.SchedQueue).Warn("application guaranteed resource conversion failure",
-				zap.String("json guaranteed string", guaranteed),
-				zap.Error(guaranteedErr))
-			if quotaErr != nil {
-				return
-			}
-		} else if !resources.StrictlyGreaterThanZero(guaranteedRes) {
-			log.Log(log.SchedQueue).Warn("Guaranteed resource quantities should be greater than zero: cannot set queue guaranteed resource",
-				zap.Stringer("guaranteedResource", guaranteedRes))
-			guaranteedRes = nil // Skip setting guaranteed resource if it has a value <= 0
-		}
-	}
-
-	// set the quota
-	if sq.isManaged {
-		log.Log(log.SchedQueue).Warn("Trying to set max resources on a queue that is not an unmanaged leaf",
-			zap.String("queueName", sq.QueuePath))
-		return
-	}
-
-	if quotaRes != nil {
-		sq.maxResource = quotaRes
-	}
-
-	if guaranteedRes != nil {
-		sq.guaranteedResource = guaranteedRes
-	}
+	sq.queueEvents.SendNewApplicationEvent(sq.QueuePath, appID)
 }
 
 // RemoveApplication removes the app from the list of tracked applications. Make sure that the app
@@ -805,7 +786,7 @@ func (sq *Queue) RemoveApplication(app *Application) {
 			zap.String("applicationID", appID))
 		return
 	}
-	sq.queueEvents.sendRemoveApplicationEvent(sq.QueuePath, appID)
+	sq.queueEvents.SendRemoveApplicationEvent(sq.QueuePath, appID)
 	if appPending := app.GetPendingResource(); !resources.IsZero(appPending) {
 		sq.decPendingResource(appPending)
 	}
@@ -842,7 +823,7 @@ func (sq *Queue) RemoveApplication(app *Application) {
 	delete(sq.allocatingAcceptedApps, appID)
 	priority := sq.recalculatePriority()
 	sq.Unlock()
-	app.appEvents.sendRemoveApplicationEvent()
+	app.appEvents.SendRemoveApplicationEvent(appID)
 
 	sq.parent.UpdateQueuePriority(sq.Name, priority)
 
@@ -1007,7 +988,7 @@ func (sq *Queue) RemoveQueue() bool {
 	log.Log(log.SchedQueue).Info("removing queue", zap.String("queue", sq.QueuePath))
 	// root is always managed and is the only queue with a nil parent: no need to guard
 	sq.parent.removeChildQueue(sq.Name)
-	sq.queueEvents.sendRemoveQueueEvent(sq.QueuePath, sq.isManaged)
+	sq.queueEvents.SendRemoveQueueEvent(sq.QueuePath, sq.isManaged)
 	return true
 }
 
@@ -1030,18 +1011,17 @@ func (sq *Queue) isRoot() bool {
 	return sq.parent == nil
 }
 
-// IncAllocatedResource increments the allocated resources for this queue (recursively).
+// TryIncAllocatedResource increments the allocated resources for this queue (recursively).
 // Guard against going over max resources if set
-func (sq *Queue) IncAllocatedResource(alloc *resources.Resource, nodeReported bool) error {
+func (sq *Queue) TryIncAllocatedResource(alloc *resources.Resource) error {
 	// check this queue: failure stops checks if the allocation is not part of a node addition
-	fit, newAllocated := sq.allocatedResFits(alloc)
-	if !nodeReported && !fit {
+	if !sq.allocatedResFits(alloc) {
 		return fmt.Errorf("allocation (%v) puts queue '%s' over maximum allocation (%v), current usage (%v)",
 			alloc, sq.QueuePath, sq.maxResource, sq.allocatedResource)
 	}
 	// check the parent: need to pass before updating
 	if sq.parent != nil {
-		if err := sq.parent.IncAllocatedResource(alloc, nodeReported); err != nil {
+		if err := sq.parent.TryIncAllocatedResource(alloc); err != nil {
 			// only log the warning if we get to the leaf: otherwise we could spam the log with the same message
 			// each time we return from a recursive call. Worst case (hierarchy depth-1) times.
 			if sq.isLeaf {
@@ -1058,17 +1038,41 @@ func (sq *Queue) IncAllocatedResource(alloc *resources.Resource, nodeReported bo
 	sq.Lock()
 	defer sq.Unlock()
 	// all OK update this queue
-	sq.allocatedResource = newAllocated
+	sq.allocatedResource = resources.Add(sq.allocatedResource, alloc)
 	sq.updateAllocatedResourceMetrics()
 	return nil
 }
 
+// IncAllocatedResource increments the allocated resources for this queue (recursively). No queue limits are checked.
+func (sq *Queue) IncAllocatedResource(alloc *resources.Resource) {
+	// fall through if nil
+	if sq == nil {
+		return
+	}
+
+	// update parent
+	sq.parent.IncAllocatedResource(alloc)
+
+	// update this queue
+	sq.Lock()
+	defer sq.Unlock()
+	sq.allocatedResource = resources.Add(sq.allocatedResource, alloc)
+	sq.updateAllocatedResourceMetrics()
+}
+
+// allocatedResFits adds the passed in resource to the allocatedResource of the queue and checks if it still fits in the
+// queues' maximum. If the resource fits it returns true otherwise false.
 // small helper method to access sq.maxResource+sq.allocatedResource and avoid Clone() call
-func (sq *Queue) allocatedResFits(res *resources.Resource) (bool, *resources.Resource) {
+func (sq *Queue) allocatedResFits(alloc *resources.Resource) bool {
 	sq.RLock()
 	defer sq.RUnlock()
-	newAllocated := resources.Add(sq.allocatedResource, res)
-	return sq.maxResource.FitInMaxUndef(newAllocated), newAllocated
+	// on the root we want to reject a new allocation if it asks for resources not registered
+	// so do not use the "undefined" flag, also handles pruned max for root
+	if sq.isRoot() {
+		return sq.maxResource.FitIn(resources.AddOnlyExisting(alloc, sq.allocatedResource))
+	}
+	// any other queue undefined is always good
+	return sq.maxResource.FitInMaxUndef(resources.AddOnlyExisting(alloc, sq.allocatedResource))
 }
 
 // DecAllocatedResource decrement the allocated resources for this queue (recursively)
@@ -1103,7 +1107,12 @@ func (sq *Queue) DecAllocatedResource(alloc *resources.Resource) error {
 	defer sq.Unlock()
 	// all OK update the queue
 	sq.allocatedResource = resources.Sub(sq.allocatedResource, alloc)
+	// We should update the metrics before pruning the resource.
+	// For example:
+	// If we prune the resource first and the resource become nil after pruning,
+	// the metrics will not be updated with nil resource, this is not expected.
 	sq.updateAllocatedResourceMetrics()
+	sq.allocatedResource.Prune()
 	return nil
 }
 
@@ -1136,6 +1145,11 @@ func (sq *Queue) DecPreemptingResource(alloc *resources.Resource) {
 	sq.parent.DecPreemptingResource(alloc)
 	sq.preemptingResource = resources.Sub(sq.preemptingResource, alloc)
 	sq.updatePreemptingResourceMetrics()
+	// We should update the metrics before pruning the resource.
+	// For example:
+	// If we prune the resource first and the resource become nil after pruning,
+	// the metrics will not be updated with nil resource, this is not expected.
+	sq.preemptingResource.Prune()
 }
 
 func (sq *Queue) IsPrioritySortEnabled() bool {
@@ -1149,7 +1163,7 @@ func (sq *Queue) IsPrioritySortEnabled() bool {
 // Only applications with a pending resource request are considered.
 // Lock free call all locks are taken when needed in called functions
 // If withPlaceholdersOnly is true, then only applications with at least one placeholder allocation are considered.
-func (sq *Queue) sortApplications(filterApps, withPlaceholdersOnly bool) []*Application {
+func (sq *Queue) sortApplications(withPlaceholdersOnly bool) []*Application {
 	if !sq.IsLeafQueue() {
 		return nil
 	}
@@ -1167,16 +1181,7 @@ func (sq *Queue) sortApplications(filterApps, withPlaceholdersOnly bool) []*Appl
 	}
 
 	// sort applications based on the sorting policy
-	// some apps might be filtered out based on the policy specific conditions.
-	// currently, only the stateAware policy does the filtering (based on app state).
-	// if the filterApps flag is true, the app filtering is skipped while doing the sorting.
-	queueSortType := sq.getSortType()
-	if !filterApps && queueSortType == policies.StateAwarePolicy {
-		// fallback to FIFO policy if the filterApps flag is false
-		// this is to skip the app filtering in the StateAware policy sorting
-		queueSortType = policies.FifoSortPolicy
-	}
-	return sortApplications(apps, queueSortType, sq.IsPrioritySortEnabled(), sq.GetGuaranteedResource())
+	return sortApplications(apps, sq.getSortType(), sq.IsPrioritySortEnabled(), sq.GetGuaranteedResource())
 }
 
 // sortQueues returns a sorted shallow copy of the queues for this parent queue.
@@ -1189,6 +1194,7 @@ func (sq *Queue) sortQueues() []*Queue {
 	}
 	// Create a list of the queues with pending resources
 	sortedQueues := make([]*Queue, 0)
+	sortedMaxFairResources := make([]*resources.Resource, 0)
 	for _, child := range sq.GetCopyOfChildren() {
 		// a stopped queue cannot be scheduled
 		if child.IsStopped() {
@@ -1197,10 +1203,11 @@ func (sq *Queue) sortQueues() []*Queue {
 		// queue must have pending resources to be considered for scheduling
 		if resources.StrictlyGreaterThanZero(child.GetPendingResource()) {
 			sortedQueues = append(sortedQueues, child)
+			sortedMaxFairResources = append(sortedMaxFairResources, child.GetFairMaxResource())
 		}
 	}
 	// Sort the queues
-	sortQueue(sortedQueues, sq.getSortType(), sq.IsPrioritySortEnabled())
+	sortQueue(sortedQueues, sortedMaxFairResources, sq.getSortType(), sq.IsPrioritySortEnabled())
 
 	return sortedQueues
 }
@@ -1235,7 +1242,7 @@ func (sq *Queue) getMaxHeadRoom() *resources.Resource {
 func (sq *Queue) internalHeadRoom(parentHeadRoom *resources.Resource) *resources.Resource {
 	sq.RLock()
 	defer sq.RUnlock()
-	headRoom := sq.maxResource.Clone()
+	headRoom := sq.maxResource
 
 	// if we have no max set headroom is always the same as the parent
 	if headRoom == nil {
@@ -1244,14 +1251,14 @@ func (sq *Queue) internalHeadRoom(parentHeadRoom *resources.Resource) *resources
 
 	// calculate what we have left over after removing all allocation
 	// ignore unlimited resource types (ie the ones not defined in max)
-	headRoom.SubOnlyExisting(sq.allocatedResource)
+	headRoom = resources.SubOnlyExisting(headRoom, sq.allocatedResource)
 
 	// check the minimum of the two: parentHeadRoom is nil for root
 	if parentHeadRoom == nil {
 		return headRoom
 	}
 	// take the minimum value of *all* resource types defined
-	return resources.ComponentWiseMinPermissive(headRoom, parentHeadRoom)
+	return resources.ComponentWiseMin(headRoom, parentHeadRoom)
 }
 
 // GetMaxResource returns the max resource for the queue. The max resource should never be larger than the
@@ -1270,6 +1277,38 @@ func (sq *Queue) GetMaxResource() *resources.Resource {
 	return sq.internalGetMax(limit)
 }
 
+// GetFairMaxResource computes the fair max resources for a given queue.
+// Starting with the root, descend down to the target queue allowing children to override Resource values .
+// If the root includes an explicit 0 value for a Resource, do not include it in the accumulator and treat it as missing.
+// If no children provide a maximum capacity override, the resulting value will be the value found on the Root.
+// It is useful for fair-scheduling to allow a ratio to be produced representing the rough utilization % of a given queue.
+func (sq *Queue) GetFairMaxResource() *resources.Resource {
+	var limit *resources.Resource
+	if sq.parent == nil {
+		return sq.GetMaxResource().Clone()
+	}
+
+	limit = sq.parent.GetFairMaxResource()
+	return sq.internalGetFairMaxResource(limit)
+}
+
+func (sq *Queue) internalGetFairMaxResource(limit *resources.Resource) *resources.Resource {
+	sq.RLock()
+	defer sq.RUnlock()
+
+	out := limit.Clone()
+	if sq.maxResource.IsEmpty() || out.IsEmpty() {
+		return out
+	}
+
+	// perform merge. child wins every resources collision
+	for k, v := range sq.maxResource.Resources {
+		out.Resources[k] = v
+	}
+
+	return out
+}
+
 // GetMaxQueueSet returns the max resource for the queue. The max resource should never be larger than the
 // max resource of the parent. The cluster size, which defines the root limit, is not relevant for this call.
 // Contrary to the GetMaxResource call. This will return nil unless a limit is set.
@@ -1278,12 +1317,10 @@ func (sq *Queue) GetMaxResource() *resources.Resource {
 // When defining a limit you therefore should define all resource quantities.
 func (sq *Queue) GetMaxQueueSet() *resources.Resource {
 	// get the limit for the parent first and check against the queue's own
-	var limit *resources.Resource
 	if sq.parent == nil {
 		return nil
 	}
-	limit = sq.parent.GetMaxQueueSet()
-	return sq.internalGetMax(limit)
+	return sq.internalGetMax(sq.parent.GetMaxQueueSet())
 }
 
 // internalGetMax does the real max calculation.
@@ -1292,9 +1329,6 @@ func (sq *Queue) internalGetMax(parentLimit *resources.Resource) *resources.Reso
 	defer sq.RUnlock()
 	// no parent queue limit set, not even for root
 	if parentLimit == nil {
-		if sq.maxResource == nil {
-			return nil
-		}
 		return sq.maxResource.Clone()
 	}
 	// parent limit set, no queue limit return parent
@@ -1327,7 +1361,7 @@ func (sq *Queue) SetMaxResource(max *resources.Resource) {
 			zap.Stringer("current", sq.maxResource),
 			zap.Stringer("new", max))
 		if !resources.Equals(sq.maxResource, max) && sq.queueEvents != nil {
-			sq.queueEvents.sendMaxResourceChangedEvent(sq.QueuePath, sq.maxResource)
+			sq.queueEvents.SendMaxResourceChangedEvent(sq.QueuePath, sq.maxResource)
 		}
 		sq.maxResource = max.Clone()
 		sq.updateMaxResourceMetrics()
@@ -1337,7 +1371,7 @@ func (sq *Queue) SetMaxResource(max *resources.Resource) {
 			zap.Stringer("current", sq.maxResource),
 			zap.Stringer("new", max))
 		if sq.queueEvents != nil {
-			sq.queueEvents.sendMaxResourceChangedEvent(sq.QueuePath, sq.maxResource)
+			sq.queueEvents.SendMaxResourceChangedEvent(sq.QueuePath, sq.maxResource)
 		}
 		sq.maxResource = nil
 		sq.updateMaxResourceMetrics()
@@ -1375,7 +1409,7 @@ func (sq *Queue) canRunApp(appID string) bool {
 // resources are skipped.
 // Applications are sorted based on the application sortPolicy. Applications without pending resources are skipped.
 // Lock free call this all locks are taken when needed in called functions
-func (sq *Queue) TryAllocate(iterator func() NodeIterator, fullIterator func() NodeIterator, getnode func(string) *Node, allowPreemption bool) *Allocation {
+func (sq *Queue) TryAllocate(iterator func() NodeIterator, fullIterator func() NodeIterator, getnode func(string) *Node, allowPreemption bool) *AllocationResult {
 	if sq.IsLeafQueue() {
 		// get the headroom
 		headRoom := sq.getHeadRoom()
@@ -1383,33 +1417,34 @@ func (sq *Queue) TryAllocate(iterator func() NodeIterator, fullIterator func() N
 		preemptAttemptsRemaining := maxPreemptionsPerQueue
 
 		// process the apps (filters out app without pending requests)
-		for _, app := range sq.sortApplications(true, false) {
+		for _, app := range sq.sortApplications(false) {
 			runnableInQueue := sq.canRunApp(app.ApplicationID)
 			runnableByUserLimit := ugm.GetUserManager().CanRunApp(sq.QueuePath, app.ApplicationID, app.user)
 			app.updateRunnableStatus(runnableInQueue, runnableByUserLimit)
 			if app.IsAccepted() && (!runnableInQueue || !runnableByUserLimit) {
 				continue
 			}
-			alloc := app.tryAllocate(headRoom, allowPreemption, preemptionDelay, &preemptAttemptsRemaining, iterator, fullIterator, getnode)
-			if alloc != nil {
+			result := app.tryAllocate(headRoom, allowPreemption, preemptionDelay, &preemptAttemptsRemaining, iterator, fullIterator, getnode)
+			if result != nil {
 				log.Log(log.SchedQueue).Info("allocation found on queue",
 					zap.String("queueName", sq.QueuePath),
 					zap.String("appID", app.ApplicationID),
-					zap.Stringer("allocation", alloc))
+					zap.Stringer("resultType", result.ResultType),
+					zap.Stringer("allocation", result.Request))
 				// if the app is still in Accepted state we're allocating placeholders.
 				// we want to count these apps as running
 				if app.IsAccepted() {
 					sq.setAllocatingAccepted(app.ApplicationID)
 				}
-				return alloc
+				return result
 			}
 		}
 	} else {
 		// process the child queues (filters out queues without pending requests)
 		for _, child := range sq.sortQueues() {
-			alloc := child.TryAllocate(iterator, fullIterator, getnode, allowPreemption)
-			if alloc != nil {
-				return alloc
+			result := child.TryAllocate(iterator, fullIterator, getnode, allowPreemption)
+			if result != nil {
+				return result
 			}
 		}
 	}
@@ -1422,25 +1457,26 @@ func (sq *Queue) TryAllocate(iterator func() NodeIterator, fullIterator func() N
 // the configured queue sortPolicy. Queues without pending resources are skipped.
 // Applications are sorted based on the application sortPolicy. Applications without pending resources are skipped.
 // Lock free call this all locks are taken when needed in called functions
-func (sq *Queue) TryPlaceholderAllocate(iterator func() NodeIterator, getnode func(string) *Node) *Allocation {
+func (sq *Queue) TryPlaceholderAllocate(iterator func() NodeIterator, getnode func(string) *Node) *AllocationResult {
 	if sq.IsLeafQueue() {
 		// process the apps (filters out app without pending requests)
-		for _, app := range sq.sortApplications(true, true) {
-			alloc := app.tryPlaceholderAllocate(iterator, getnode)
-			if alloc != nil {
+		for _, app := range sq.sortApplications(true) {
+			result := app.tryPlaceholderAllocate(iterator, getnode)
+			if result != nil {
 				log.Log(log.SchedQueue).Info("allocation found on queue",
 					zap.String("queueName", sq.QueuePath),
 					zap.String("appID", app.ApplicationID),
-					zap.Stringer("allocation", alloc))
-				return alloc
+					zap.Stringer("resultType", result.ResultType),
+					zap.Stringer("allocation", result.Request))
+				return result
 			}
 		}
 	} else {
 		// process the child queues (filters out queues without pending requests)
 		for _, child := range sq.sortQueues() {
-			alloc := child.TryPlaceholderAllocate(iterator, getnode)
-			if alloc != nil {
-				return alloc
+			result := child.TryPlaceholderAllocate(iterator, getnode)
+			if result != nil {
+				return result
 			}
 		}
 	}
@@ -1448,14 +1484,12 @@ func (sq *Queue) TryPlaceholderAllocate(iterator func() NodeIterator, getnode fu
 }
 
 // GetQueueOutstandingRequests builds a slice of pending allocation asks that fits into the queue's headroom.
-func (sq *Queue) GetQueueOutstandingRequests(total *[]*AllocationAsk) {
+func (sq *Queue) GetQueueOutstandingRequests(total *[]*Allocation) {
 	if sq.IsLeafQueue() {
 		headRoom := sq.getMaxHeadRoom()
-		// while calculating outstanding requests, we do not need to filter apps.
-		// e.g. StateAware filters apps by state in order to schedule app one by one.
-		// we calculate all the requests that can fit into the queue's headroom,
+		// while calculating outstanding requests, we calculate all the requests that can fit into the queue's headroom,
 		// all these requests are qualified to trigger the up scaling.
-		for _, app := range sq.sortApplications(false, false) {
+		for _, app := range sq.sortApplications(false) {
 			// calculate the users' headroom
 			userHeadroom := ugm.GetUserManager().Headroom(app.queuePath, app.ApplicationID, app.user)
 			app.getOutstandingRequests(headRoom, userHeadroom, total)
@@ -1473,7 +1507,7 @@ func (sq *Queue) GetQueueOutstandingRequests(total *[]*AllocationAsk) {
 // the configured queue sortPolicy. Queues without pending resources are skipped.
 // Applications are currently NOT sorted and are iterated over in a random order.
 // Lock free call this all locks are taken when needed in called functions
-func (sq *Queue) TryReservedAllocate(iterator func() NodeIterator) *Allocation {
+func (sq *Queue) TryReservedAllocate(iterator func() NodeIterator) *AllocationResult {
 	if sq.IsLeafQueue() {
 		// skip if it has no reservations
 		reservedCopy := sq.GetReservedApps()
@@ -1497,28 +1531,29 @@ func (sq *Queue) TryReservedAllocate(iterator func() NodeIterator) *Allocation {
 				if app.IsAccepted() && (!sq.canRunApp(appID) || !ugm.GetUserManager().CanRunApp(sq.QueuePath, appID, app.user)) {
 					continue
 				}
-				alloc := app.tryReservedAllocate(headRoom, iterator)
-				if alloc != nil {
+				result := app.tryReservedAllocate(headRoom, iterator)
+				if result != nil {
 					log.Log(log.SchedQueue).Info("reservation found for allocation found on queue",
 						zap.String("queueName", sq.QueuePath),
 						zap.String("appID", appID),
-						zap.Stringer("allocation", alloc),
+						zap.Stringer("resultType", result.ResultType),
+						zap.Stringer("allocation", result.Request),
 						zap.String("appStatus", app.CurrentState()))
 					// if the app is still in Accepted state we're allocating placeholders.
 					// we want to count these apps as running
 					if app.IsAccepted() {
 						sq.setAllocatingAccepted(app.ApplicationID)
 					}
-					return alloc
+					return result
 				}
 			}
 		}
 	} else {
 		// process the child queues (filters out queues that have no pending requests)
 		for _, child := range sq.sortQueues() {
-			alloc := child.TryReservedAllocate(iterator)
-			if alloc != nil {
-				return alloc
+			result := child.TryReservedAllocate(iterator)
+			if result != nil {
+				return result
 			}
 		}
 	}
@@ -1609,7 +1644,7 @@ func (sq *Queue) getSortType() policies.SortPolicy {
 }
 
 // SupportTaskGroup returns true if the queue supports task groups.
-// FIFO and StateAware sorting policies can support this.
+// FIFO policy is required to support this.
 // NOTE: this call does not make sense for a parent queue, and always returns false
 func (sq *Queue) SupportTaskGroup() bool {
 	sq.RLock()
@@ -1617,7 +1652,7 @@ func (sq *Queue) SupportTaskGroup() bool {
 	if !sq.isLeaf {
 		return false
 	}
-	return sq.sortType == policies.FifoSortPolicy || sq.sortType == policies.StateAwarePolicy
+	return sq.sortType == policies.FifoSortPolicy
 }
 
 // updateGuaranteedResourceMetrics updates guaranteed resource metrics.
@@ -1730,25 +1765,11 @@ func (sq *Queue) GetPreemptionPolicy() policies.PreemptionPolicy {
 	return sq.preemptionPolicy
 }
 
-// SetMaxRunningApps allows setting the maximum running apps on a queue
-// test only
-func (sq *Queue) SetMaxRunningApps(max int) {
-	if sq == nil {
-		return
-	}
-	if max < 0 {
-		return
-	}
-	sq.Lock()
-	defer sq.Unlock()
-	sq.maxRunningApps = uint64(max)
-}
-
 // FindEligiblePreemptionVictims is used to locate tasks which may be preempted for the given ask.
 // queuePath is the fully-qualified path of the queue where ask resides
 // ask is the ask we are attempting to preempt for
 // return is a map of potential victims keyed by queue path
-func (sq *Queue) FindEligiblePreemptionVictims(queuePath string, ask *AllocationAsk) map[string]*QueuePreemptionSnapshot {
+func (sq *Queue) FindEligiblePreemptionVictims(queuePath string, ask *Allocation) map[string]*QueuePreemptionSnapshot {
 	results := make(map[string]*QueuePreemptionSnapshot)
 	priorityMap := make(map[string]int64)
 
@@ -1766,6 +1787,15 @@ func (sq *Queue) FindEligiblePreemptionVictims(queuePath string, ask *Allocation
 		return nil
 	}
 
+	// create snapshot for ask or preemptor queue
+	sq.createPreemptionSnapshot(results, queuePath)
+	c := sq
+	// set the ask queue for all queues in the ask queue hierarchy
+	for c.parent != nil {
+		results[c.QueuePath].AskQueue = results[queuePath]
+		c = c.parent
+	}
+
 	// walk the subtree contained within the preemption fence and collect potential victims organized by nodeID
 	fence.findEligiblePreemptionVictims(results, queuePath, ask, priorityMap, queuePriority, false)
 
@@ -1773,7 +1803,7 @@ func (sq *Queue) FindEligiblePreemptionVictims(queuePath string, ask *Allocation
 }
 
 // createPreemptionSnapshot is used to create a snapshot of the current queue's resource usage and potential preemption victims
-func (sq *Queue) createPreemptionSnapshot(cache map[string]*QueuePreemptionSnapshot) *QueuePreemptionSnapshot {
+func (sq *Queue) createPreemptionSnapshot(cache map[string]*QueuePreemptionSnapshot, askQueuePath string) *QueuePreemptionSnapshot {
 	if sq == nil {
 		return nil
 	}
@@ -1783,7 +1813,7 @@ func (sq *Queue) createPreemptionSnapshot(cache map[string]*QueuePreemptionSnaps
 		return snapshot
 	}
 
-	parentSnapshot := sq.parent.createPreemptionSnapshot(cache)
+	parentSnapshot := sq.parent.createPreemptionSnapshot(cache, askQueuePath)
 	sq.RLock()
 	defer sq.RUnlock()
 	snapshot = &QueuePreemptionSnapshot{
@@ -1795,41 +1825,43 @@ func (sq *Queue) createPreemptionSnapshot(cache map[string]*QueuePreemptionSnaps
 		MaxResource:        sq.maxResource.Clone(),
 		GuaranteedResource: sq.guaranteedResource.Clone(),
 		PotentialVictims:   make([]*Allocation, 0),
+		AskQueue:           cache[askQueuePath],
 	}
 	cache[sq.QueuePath] = snapshot
 	return snapshot
 }
 
-func (sq *Queue) findEligiblePreemptionVictims(results map[string]*QueuePreemptionSnapshot, queuePath string, ask *AllocationAsk, priorityMap map[string]int64, askPriority int64, fenced bool) {
+func (sq *Queue) findEligiblePreemptionVictims(results map[string]*QueuePreemptionSnapshot, queuePath string, ask *Allocation, priorityMap map[string]int64, askPriority int64, fenced bool) {
 	if sq == nil {
 		return
 	}
-
-	// if this is the target queue, return it but with an empty victim list so we can use it in calculations
-	if sq.QueuePath == queuePath {
-		sq.createPreemptionSnapshot(results)
+	if sq.GetQueuePath() == queuePath {
 		return
 	}
-
 	if sq.IsLeafQueue() {
 		// leaf queue, skip queue if preemption is disabled
 		if sq.GetPreemptionPolicy() == policies.DisabledPreemptionPolicy {
 			return
 		}
 
+		victims := sq.createPreemptionSnapshot(results, queuePath)
+
 		// skip this queue if we are within guaranteed limits
-		guaranteed := resources.ComponentWiseMinPermissive(sq.GetActualGuaranteedResource(), sq.GetMaxResource())
-		if guaranteed.FitInMaxUndef(sq.GetAllocatedResource()) {
+		remaining := results[sq.QueuePath].GetRemainingGuaranteedResource()
+		if remaining != nil && resources.StrictlyGreaterThanOrEquals(remaining, resources.Zero) {
 			return
 		}
 
-		victims := sq.createPreemptionSnapshot(results)
-
 		// walk allocations and select those that are equal or lower than current priority
 		for _, app := range sq.GetCopyOfApps() {
 			for _, alloc := range app.GetAllAllocations() {
+				// at least any one of the ask resource type should match with potential victim
+				if !ask.GetAllocatedResource().MatchAny(alloc.GetAllocatedResource()) {
+					continue
+				}
+
 				// skip tasks which require a specific node
-				if alloc.GetAsk().GetRequiredNode() != "" {
+				if alloc.GetRequiredNode() != "" {
 					continue
 				}
 
@@ -1892,7 +1924,8 @@ func (sq *Queue) findPreemptionFenceRoot(priorityMap map[string]int64, currentPr
 	}
 	priorityMap[sq.QueuePath] = currentPriority
 
-	if sq.parent == nil || sq.GetPreemptionPolicy() == policies.FencePreemptionPolicy {
+	// Return this queue as fence root if: 1. FencePreemptionPolicy is set 2. root queue 3. allocations in the queue reached maximum resources
+	if sq.parent == nil || sq.GetPreemptionPolicy() == policies.FencePreemptionPolicy || resources.Equals(sq.maxResource, sq.allocatedResource) {
 		return sq
 	}
 	return sq.parent.findPreemptionFenceRoot(priorityMap, currentPriority)
