@@ -26,24 +26,28 @@ import (
 	"github.com/google/btree"
 	"gotest.tools/v3/assert"
 
+	"github.com/apache/yunikorn-core/pkg/common"
 	"github.com/apache/yunikorn-core/pkg/common/configs"
 	"github.com/apache/yunikorn-core/pkg/common/resources"
 	"github.com/apache/yunikorn-core/pkg/common/security"
 	"github.com/apache/yunikorn-core/pkg/events"
 	"github.com/apache/yunikorn-core/pkg/rmproxy"
+	schedEvt "github.com/apache/yunikorn-core/pkg/scheduler/objects/events"
 	"github.com/apache/yunikorn-core/pkg/scheduler/ugm"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
 )
 
 const (
-	appID0        = "app-0"
-	appID1        = "app-1"
-	appID2        = "app-2"
-	aKey          = "alloc-1"
-	aAllocationID = "alloc-allocationid-1"
-	nodeID1       = "node-1"
-	nodeID2       = "node-2"
-	instType1     = "itype-1"
+	appID0    = "app-0"
+	appID1    = "app-1"
+	appID2    = "app-2"
+	appID3    = "app-3"
+	aKey      = "alloc-1"
+	aKey2     = "alloc-2"
+	nodeID1   = "node-1"
+	nodeID2   = "node-2"
+	instType1 = "itype-1"
+	testgroup = "testgroup"
 )
 
 // Create the root queue, base for all testing
@@ -176,8 +180,8 @@ func newNodeInternal(nodeID string, total, occupied *resources.Resource) *Node {
 		allocations:       make(map[string]*Allocation),
 		schedulable:       true,
 		reservations:      make(map[string]*reservation),
+		nodeEvents:        schedEvt.NewNodeEvents(events.GetEventSystem()),
 	}
-	sn.nodeEvents = newNodeEvents(sn, events.GetEventSystem())
 	return sn
 }
 
@@ -209,49 +213,60 @@ func newProto(nodeID string, totalResource, occupiedResource *resources.Resource
 	return &proto
 }
 
-// Create a new Allocation with a random ask key
+// Create a new Allocation with a random allocation key
 func newAllocation(appID, nodeID string, res *resources.Resource) *Allocation {
-	askKey := strconv.FormatInt((time.Now()).UnixNano(), 10)
-	ask := newAllocationAsk(askKey, appID, res)
-	return NewAllocation(nodeID, ask)
+	allocKey := strconv.FormatInt((time.Now()).UnixNano(), 10)
+	return newAllocationAll(allocKey, appID, nodeID, "", res, false, 0)
+}
+
+// Create a new Allocation with a specified allocation key
+func newAllocationWithKey(allocKey, appID, nodeID string, res *resources.Resource) *Allocation {
+	return newAllocationAll(allocKey, appID, nodeID, "", res, false, 0)
 }
 
 // Create a new Allocation with a random ask key
-func newPlaceholderAlloc(appID, nodeID string, res *resources.Resource) *Allocation {
-	askKey := strconv.FormatInt((time.Now()).UnixNano(), 10)
-	ask := newAllocationAsk(askKey, appID, res)
-	ask.placeholder = true
-	return NewAllocation(nodeID, ask)
+func newPlaceholderAlloc(appID, nodeID string, res *resources.Resource, taskGroup string) *Allocation {
+	allocKey := strconv.FormatInt((time.Now()).UnixNano(), 10)
+	return newAllocationAll(allocKey, appID, nodeID, taskGroup, res, true, 0)
 }
 
-func newAllocationAsk(allocKey, appID string, res *resources.Resource) *AllocationAsk {
-	return newAllocationAskAll(allocKey, appID, "", res, 1, false, 0)
+func newAllocationAsk(allocKey, appID string, res *resources.Resource) *Allocation {
+	return newAllocationAskAll(allocKey, appID, "", res, false, 0)
 }
 
-func newAllocationAskPriority(allocKey, appID string, res *resources.Resource, priority int32) *AllocationAsk {
-	return newAllocationAskAll(allocKey, appID, "", res, 1, false, priority)
+func newAllocationAskPriority(allocKey, appID string, res *resources.Resource, priority int32) *Allocation {
+	return newAllocationAskAll(allocKey, appID, "", res, false, priority)
 }
 
-func newAllocationAskRepeat(allocKey, appID string, res *resources.Resource, repeat int) *AllocationAsk {
-	return newAllocationAskAll(allocKey, appID, "", res, repeat, false, 0)
+func newAllocationAskTG(allocKey, appID, taskGroup string, res *resources.Resource) *Allocation {
+	return newAllocationAskAll(allocKey, appID, taskGroup, res, taskGroup != "", 0)
 }
 
-func newAllocationAskTG(allocKey, appID, taskGroup string, res *resources.Resource, repeat int) *AllocationAsk {
-	return newAllocationAskAll(allocKey, appID, taskGroup, res, repeat, taskGroup != "", 0)
-}
-
-func newAllocationAskAll(allocKey, appID, taskGroup string, res *resources.Resource, repeat int, placeholder bool, priority int32) *AllocationAsk {
-	ask := &si.AllocationAsk{
-		AllocationKey:  allocKey,
-		ApplicationID:  appID,
-		PartitionName:  "default",
-		ResourceAsk:    res.ToProto(),
-		MaxAllocations: int32(repeat),
-		TaskGroupName:  taskGroup,
-		Placeholder:    placeholder,
-		Priority:       priority,
+func newAllocationAskAll(allocKey, appID, taskGroup string, res *resources.Resource, placeholder bool, priority int32) *Allocation {
+	ask := &si.Allocation{
+		AllocationKey:    allocKey,
+		ApplicationID:    appID,
+		PartitionName:    "default",
+		ResourcePerAlloc: res.ToProto(),
+		TaskGroupName:    taskGroup,
+		Placeholder:      placeholder,
+		Priority:         priority,
 	}
-	return NewAllocationAskFromSI(ask)
+	return NewAllocationFromSI(ask)
+}
+
+func newAllocationAll(allocKey, appID, nodeID, taskGroup string, res *resources.Resource, placeholder bool, priority int32) *Allocation {
+	alloc := &si.Allocation{
+		AllocationKey:    allocKey,
+		ApplicationID:    appID,
+		PartitionName:    "default",
+		ResourcePerAlloc: res.ToProto(),
+		TaskGroupName:    taskGroup,
+		Placeholder:      placeholder,
+		Priority:         priority,
+		NodeID:           nodeID,
+	}
+	return NewAllocationFromSI(alloc)
 }
 
 func getTestUserGroup() security.UserGroup {
@@ -278,6 +293,21 @@ func assertUserResourcesAndGroupResources(t *testing.T, userGroup security.UserG
 	assert.Equal(t, resources.Equals(groupResource, expectedGroupResources), true)
 }
 
+func assertAllocationLog(t *testing.T, ask *Allocation) {
+	log := ask.GetAllocationLog()
+	preemptionPreconditionsFailed := false
+	PreemptionDoesNotHelp := false
+	for _, l := range log {
+		if l.Message == common.PreemptionPreconditionsFailed {
+			preemptionPreconditionsFailed = true
+		} else if l.Message == common.PreemptionDoesNotHelp {
+			PreemptionDoesNotHelp = true
+		}
+	}
+	assert.Assert(t, preemptionPreconditionsFailed)
+	assert.Assert(t, PreemptionDoesNotHelp)
+}
+
 func getNodeIteratorFn(nodes ...*Node) func() NodeIterator {
 	tree := btree.New(7)
 	for _, node := range nodes {
