@@ -19,12 +19,14 @@
 package tests
 
 import (
+	"fmt"
 	"testing"
 
 	"gotest.tools/v3/assert"
 
 	"github.com/apache/yunikorn-core/pkg/common/resources"
 	"github.com/apache/yunikorn-core/pkg/entrypoint"
+	"github.com/apache/yunikorn-core/pkg/scheduler"
 	"github.com/apache/yunikorn-core/pkg/scheduler/objects"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/common"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
@@ -124,17 +126,26 @@ func TestSchedulerRecovery(t *testing.T) {
 	assert.Equal(t, app01.CurrentState(), objects.New.String())
 
 	err = ms.proxy.UpdateAllocation(&si.AllocationRequest{
-		Asks: []*si.AllocationAsk{
+		Allocations: []*si.Allocation{
 			{
-				AllocationKey: "alloc-1",
-				ResourceAsk: &si.Resource{
+				AllocationKey: "alloc-0",
+				ResourcePerAlloc: &si.Resource{
 					Resources: map[string]*si.Quantity{
 						"memory": {Value: 10},
 						"vcore":  {Value: 1},
 					},
 				},
-				MaxAllocations: 2,
-				ApplicationID:  appID1,
+				ApplicationID: appID1,
+			},
+			{
+				AllocationKey: "alloc-1",
+				ResourcePerAlloc: &si.Resource{
+					Resources: map[string]*si.Quantity{
+						"memory": {Value: 10},
+						"vcore":  {Value: 1},
+					},
+				},
+				ApplicationID: appID1,
 			},
 		},
 		RmID: "rm:123",
@@ -170,33 +181,24 @@ func TestSchedulerRecovery(t *testing.T) {
 	waitForAllocatedNodeResource(t, ms.scheduler.GetClusterContext(), "[rm:123]default",
 		[]string{"node-1:1234", "node-2:1234"}, 20, 1000)
 
-	// ask for two more resources
+	// ask for 4 more allocations
+	asks := make([]*si.Allocation, 4)
+	mem := [4]int64{50, 100, 50, 100}
+	for i := 0; i < 4; i++ {
+		asks[i] = &si.Allocation{
+			AllocationKey: fmt.Sprintf("alloc-%d", i+2),
+			ResourcePerAlloc: &si.Resource{
+				Resources: map[string]*si.Quantity{
+					"memory": {Value: mem[i]},
+					"vcore":  {Value: 5},
+				},
+			},
+			ApplicationID: appID1,
+		}
+	}
 	err = ms.proxy.UpdateAllocation(&si.AllocationRequest{
-		Asks: []*si.AllocationAsk{
-			{
-				AllocationKey: "alloc-2",
-				ResourceAsk: &si.Resource{
-					Resources: map[string]*si.Quantity{
-						"memory": {Value: 50},
-						"vcore":  {Value: 5},
-					},
-				},
-				MaxAllocations: 2,
-				ApplicationID:  appID1,
-			},
-			{
-				AllocationKey: "alloc-3",
-				ResourceAsk: &si.Resource{
-					Resources: map[string]*si.Quantity{
-						"memory": {Value: 100},
-						"vcore":  {Value: 5},
-					},
-				},
-				MaxAllocations: 2,
-				ApplicationID:  appID1,
-			},
-		},
-		RmID: "rm:123",
+		Allocations: asks,
+		RmID:        "rm:123",
 	})
 
 	assert.NilError(t, err, "UpdateRequest further alloc on existing app failed")
@@ -255,8 +257,7 @@ func TestSchedulerRecovery(t *testing.T) {
 						"vcore":  {Value: 20},
 					},
 				},
-				Action:              si.NodeInfo_CREATE,
-				ExistingAllocations: mockRM.nodeAllocations["node-1:1234"],
+				Action: si.NodeInfo_CREATE,
 			},
 			{
 				NodeID:     "node-2:1234",
@@ -267,8 +268,7 @@ func TestSchedulerRecovery(t *testing.T) {
 						"vcore":  {Value: 20},
 					},
 				},
-				Action:              si.NodeInfo_CREATE,
-				ExistingAllocations: mockRM.nodeAllocations["node-2:1234"],
+				Action: si.NodeInfo_CREATE,
 			},
 		},
 		RmID: "rm:123",
@@ -279,6 +279,15 @@ func TestSchedulerRecovery(t *testing.T) {
 
 	// verify partition info
 	part = ms.scheduler.GetClusterContext().GetPartition(ms.partitionName)
+
+	// add allocs to partition
+	node1Allocations := mockRM.nodeAllocations["node-1:1234"]
+	err = registerAllocations(part, node1Allocations)
+	assert.NilError(t, err)
+	node2Allocations := mockRM.nodeAllocations["node-2:1234"]
+	err = registerAllocations(part, node2Allocations)
+	assert.NilError(t, err)
+
 	// verify apps in this partition
 	assert.Equal(t, 1, len(part.GetApplications()))
 	assert.Equal(t, appID1, part.GetApplications()[0].ApplicationID)
@@ -288,8 +297,6 @@ func TestSchedulerRecovery(t *testing.T) {
 
 	// verify nodes
 	assert.Equal(t, 2, part.GetTotalNodeCount(), "incorrect recovered node count")
-	node1Allocations := mockRM.nodeAllocations["node-1:1234"]
-	node2Allocations := mockRM.nodeAllocations["node-2:1234"]
 
 	assert.Equal(t, len(node1Allocations), len(part.GetNode("node-1:1234").GetAllAllocations()), "allocations on node-1 not as expected")
 	assert.Equal(t, len(node2Allocations), len(part.GetNode("node-2:1234").GetAllAllocations()), "allocations on node-1 not as expected")
@@ -382,17 +389,26 @@ func TestSchedulerRecovery2Allocations(t *testing.T) {
 	assert.Equal(t, app01.CurrentState(), objects.New.String())
 
 	err = ms.proxy.UpdateAllocation(&si.AllocationRequest{
-		Asks: []*si.AllocationAsk{
+		Allocations: []*si.Allocation{
 			{
 				AllocationKey: "alloc-1",
-				ResourceAsk: &si.Resource{
+				ResourcePerAlloc: &si.Resource{
 					Resources: map[string]*si.Quantity{
 						"memory": {Value: 10},
 						"vcore":  {Value: 1},
 					},
 				},
-				MaxAllocations: 2,
-				ApplicationID:  appID1,
+				ApplicationID: appID1,
+			},
+			{
+				AllocationKey: "alloc-2",
+				ResourcePerAlloc: &si.Resource{
+					Resources: map[string]*si.Quantity{
+						"memory": {Value: 10},
+						"vcore":  {Value: 1},
+					},
+				},
+				ApplicationID: appID1,
 			},
 		},
 		RmID: "rm:123",
@@ -433,14 +449,17 @@ func TestSchedulerRecovery2Allocations(t *testing.T) {
 						"vcore":  {Value: 20},
 					},
 				},
-				Action:              si.NodeInfo_CREATE,
-				ExistingAllocations: mockRM.nodeAllocations["node-1:1234"],
+				Action: si.NodeInfo_CREATE,
 			},
 		},
 		RmID: "rm:123",
 	})
 	assert.NilError(t, err, "NodeRequest nodes and app for recovery failed")
 	ms.mockRM.waitForAcceptedNode(t, "node-1:1234", 1000)
+	allocs := mockRM.nodeAllocations["node-1:1234"]
+	err = ms.proxy.UpdateAllocation(&si.AllocationRequest{Allocations: allocs, RmID: "rm:123"})
+	assert.NilError(t, err, "failed to update allocations")
+	ms.mockRM.waitForAllocations(t, len(allocs), 1000)
 	recoveredApp := ms.getApplication(appID1)
 	// verify app state
 	assert.Equal(t, recoveredApp.CurrentState(), objects.Running.String())
@@ -448,6 +467,8 @@ func TestSchedulerRecovery2Allocations(t *testing.T) {
 
 // test scheduler recovery when shim doesn't report existing application
 // but only include existing allocations of this app.
+//
+//nolint:funlen
 func TestSchedulerRecoveryWithoutAppInfo(t *testing.T) {
 	// Register RM
 	ms := &mockScheduler{}
@@ -470,25 +491,6 @@ func TestSchedulerRecoveryWithoutAppInfo(t *testing.T) {
 					},
 				},
 				Action: si.NodeInfo_CREATE,
-				ExistingAllocations: []*si.Allocation{
-					{
-						AllocationKey: "allocation-key-01",
-						AllocationID:  "ALLOCATIONID01",
-						ApplicationID: "app-01",
-						PartitionName: "default",
-						NodeID:        "node-1:1234",
-						ResourcePerAlloc: &si.Resource{
-							Resources: map[string]*si.Quantity{
-								common.Memory: {
-									Value: 1024,
-								},
-								common.CPU: {
-									Value: 1,
-								},
-							},
-						},
-					},
-				},
 			},
 			{
 				NodeID:     "node-2:1234",
@@ -507,13 +509,36 @@ func TestSchedulerRecoveryWithoutAppInfo(t *testing.T) {
 	assert.NilError(t, err, "NodeRequest nodes and apps failed")
 
 	// waiting for recovery
-	// node-1 should be rejected as some of allocations cannot be recovered
-	ms.mockRM.waitForRejectedNode(t, "node-1:1234", 1000)
+	ms.mockRM.waitForAcceptedNode(t, "node-1:1234", 1000)
 	ms.mockRM.waitForAcceptedNode(t, "node-2:1234", 1000)
 
-	// verify partition resources
 	part := ms.scheduler.GetClusterContext().GetPartition("[rm:123]default")
-	assert.Equal(t, part.GetTotalNodeCount(), 1)
+	err = ms.proxy.UpdateAllocation(&si.AllocationRequest{
+		Allocations: []*si.Allocation{
+			{
+				AllocationKey: "allocation-key-01",
+				ApplicationID: "app-01",
+				PartitionName: "default",
+				NodeID:        "node-1:1234",
+				ResourcePerAlloc: &si.Resource{
+					Resources: map[string]*si.Quantity{
+						common.Memory: {
+							Value: 1024,
+						},
+						common.CPU: {
+							Value: 1,
+						},
+					},
+				},
+			},
+		},
+		RmID: "rm:123",
+	})
+
+	assert.NilError(t, err)
+
+	// verify partition resources
+	assert.Equal(t, part.GetTotalNodeCount(), 2)
 	assert.Equal(t, part.GetTotalAllocationCount(), 0)
 	assert.Equal(t, part.GetNode("node-2:1234").GetAllocatedResource().Resources[common.Memory],
 		resources.Quantity(0))
@@ -538,22 +563,27 @@ func TestSchedulerRecoveryWithoutAppInfo(t *testing.T) {
 					},
 				},
 				Action: si.NodeInfo_CREATE,
-				ExistingAllocations: []*si.Allocation{
-					{
-						AllocationKey: "allocation-key-01",
-						AllocationID:  "ALLOCATIONID01",
-						ApplicationID: "app-01",
-						PartitionName: "default",
-						NodeID:        "node-1:1234",
-						ResourcePerAlloc: &si.Resource{
-							Resources: map[string]*si.Quantity{
-								common.Memory: {
-									Value: 100,
-								},
-								common.CPU: {
-									Value: 1,
-								},
-							},
+			},
+		},
+		RmID: "rm:123",
+	})
+	assert.NilError(t, err, "NodeRequest re-register nodes and app failed")
+	ms.mockRM.waitForAcceptedNode(t, "node-1:1234", 1000)
+
+	err = ms.proxy.UpdateAllocation(&si.AllocationRequest{
+		Allocations: []*si.Allocation{
+			{
+				AllocationKey: "allocation-key-01",
+				ApplicationID: "app-01",
+				PartitionName: "default",
+				NodeID:        "node-1:1234",
+				ResourcePerAlloc: &si.Resource{
+					Resources: map[string]*si.Quantity{
+						common.Memory: {
+							Value: 100,
+						},
+						common.CPU: {
+							Value: 1,
 						},
 					},
 				},
@@ -561,8 +591,8 @@ func TestSchedulerRecoveryWithoutAppInfo(t *testing.T) {
 		},
 		RmID: "rm:123",
 	})
-	assert.NilError(t, err, "NodeRequest re-register nodes and app failed")
-	ms.mockRM.waitForAcceptedNode(t, "node-1:1234", 1000)
+	assert.NilError(t, err)
+	ms.mockRM.waitForAllocations(t, 1, 1000)
 
 	assert.Equal(t, part.GetTotalNodeCount(), 2)
 	assert.Equal(t, part.GetTotalAllocationCount(), 1)
@@ -583,6 +613,7 @@ func TestSchedulerRecoveryWithoutAppInfo(t *testing.T) {
 // test scheduler recovery that only registers nodes and apps
 func TestAppRecovery(t *testing.T) {
 	serviceContext := entrypoint.StartAllServicesWithManualScheduler()
+	defer serviceContext.StopAll()
 	proxy := serviceContext.RMProxy
 
 	BuildInfoMap := make(map[string]string)
@@ -654,6 +685,7 @@ func TestAppRecovery(t *testing.T) {
 // test scheduler recovery that only registers apps
 func TestAppRecoveryAlone(t *testing.T) {
 	serviceContext := entrypoint.StartAllServicesWithManualScheduler()
+	defer serviceContext.StopAll()
 	proxy := serviceContext.RMProxy
 
 	BuildInfoMap := make(map[string]string)
@@ -706,6 +738,8 @@ func TestAppRecoveryAlone(t *testing.T) {
 // new allocations. But during the recovery, when we recover existing
 // allocations on node, we need to ensure the placement rule is still
 // enforced.
+//
+//nolint:funlen
 func TestAppRecoveryPlacement(t *testing.T) {
 	// Register RM
 	configData := `
@@ -789,17 +823,26 @@ partitions:
 	assert.Assert(t, appQueue != nil, "application queue was not created")
 
 	err = ms.proxy.UpdateAllocation(&si.AllocationRequest{
-		Asks: []*si.AllocationAsk{
+		Allocations: []*si.Allocation{
 			{
 				AllocationKey: "alloc-1",
-				ResourceAsk: &si.Resource{
+				ResourcePerAlloc: &si.Resource{
 					Resources: map[string]*si.Quantity{
 						"memory": {Value: 10},
 						"vcore":  {Value: 1},
 					},
 				},
-				MaxAllocations: 2,
-				ApplicationID:  appID1,
+				ApplicationID: appID1,
+			},
+			{
+				AllocationKey: "alloc-2",
+				ResourcePerAlloc: &si.Resource{
+					Resources: map[string]*si.Quantity{
+						"memory": {Value: 10},
+						"vcore":  {Value: 1},
+					},
+				},
+				ApplicationID: appID1,
 			},
 		},
 		RmID: "rm:123",
@@ -843,7 +886,6 @@ partitions:
 			existingAllocations = append(existingAllocations, &si.Allocation{
 				AllocationKey:    alloc.AllocationKey,
 				AllocationTags:   alloc.AllocationTags,
-				AllocationID:     alloc.AllocationID,
 				ResourcePerAlloc: alloc.ResourcePerAlloc,
 				Priority:         alloc.Priority,
 				NodeID:           alloc.NodeID,
@@ -899,8 +941,7 @@ partitions:
 						"vcore":  {Value: 20},
 					},
 				},
-				Action:              si.NodeInfo_CREATE,
-				ExistingAllocations: toRecover["node-1:1234"],
+				Action: si.NodeInfo_CREATE,
 			},
 			{
 				NodeID:     "node-2:1234",
@@ -911,8 +952,7 @@ partitions:
 						"vcore":  {Value: 20},
 					},
 				},
-				Action:              si.NodeInfo_CREATE,
-				ExistingAllocations: toRecover["node-2:1234"],
+				Action: si.NodeInfo_CREATE,
 			},
 		},
 		RmID: "rm:123",
@@ -924,6 +964,12 @@ partitions:
 	ms.mockRM.waitForAcceptedNode(t, "node-1:1234", 1000)
 	ms.mockRM.waitForAcceptedNode(t, "node-2:1234", 1000)
 	ms.mockRM.waitForAcceptedApplication(t, appID1, 1000)
+
+	err = registerAllocations(part, toRecover["node-1:1234"])
+	assert.NilError(t, err)
+	err = registerAllocations(part, toRecover["node-2:1234"])
+	assert.NilError(t, err)
+
 	// now the queue should have been created under root.app-1-namespace
 	assert.Equal(t, len(rootQ.GetCopyOfChildren()), 1)
 	appQueue = part.GetQueue("root.app-1-namespace")
@@ -938,7 +984,6 @@ func TestPlaceholderRecovery(t *testing.T) { //nolint:funlen
 		NodeID:        "node-1:1234",
 		ApplicationID: appID1,
 		TaskGroupName: "tg-1",
-		AllocationID:  "ph-alloc-1-0",
 		ResourcePerAlloc: &si.Resource{
 			Resources: map[string]*si.Quantity{
 				"memory": {
@@ -984,8 +1029,7 @@ func TestPlaceholderRecovery(t *testing.T) { //nolint:funlen
 						"vcore":  {Value: 20},
 					},
 				},
-				Action:              si.NodeInfo_CREATE,
-				ExistingAllocations: existingAllocations,
+				Action: si.NodeInfo_CREATE,
 			},
 		},
 		RmID: "rm:123",
@@ -993,54 +1037,59 @@ func TestPlaceholderRecovery(t *testing.T) { //nolint:funlen
 	assert.NilError(t, err, "NodeRequest nodes and app for recovery failed")
 	ms.mockRM.waitForAcceptedNode(t, "node-1:1234", 1000)
 
+	// Add existing allocations
+	err = ms.proxy.UpdateAllocation(&si.AllocationRequest{
+		Allocations: existingAllocations,
+		RmID:        "rm:123",
+	})
+	assert.NilError(t, err, "AllocationRequest failed for existing allocations")
+	ms.mockRM.waitForAllocations(t, len(existingAllocations), 1000)
+
 	// Add a new placeholder ask with a different task group
 	err = ms.proxy.UpdateAllocation(&si.AllocationRequest{
-		Asks: []*si.AllocationAsk{
+		Allocations: []*si.Allocation{
 			{
 				AllocationKey: "ph-alloc-2",
-				ResourceAsk: &si.Resource{
+				ResourcePerAlloc: &si.Resource{
 					Resources: map[string]*si.Quantity{
 						"memory": {Value: 10},
 						"vcore":  {Value: 1},
 					},
 				},
-				MaxAllocations: 1,
-				ApplicationID:  appID1,
-				TaskGroupName:  "tg-2",
-				Placeholder:    true,
+				ApplicationID: appID1,
+				TaskGroupName: "tg-2",
+				Placeholder:   true,
 			},
 		},
 		RmID: "rm:123",
 	})
 	assert.NilError(t, err, "AllocationRequest failed for placeholder ask")
-	ms.mockRM.waitForAllocations(t, 1, 1000)
+	ms.mockRM.waitForAllocations(t, len(existingAllocations)+1, 1000)
 
 	// Add two real asks
 	err = ms.proxy.UpdateAllocation(&si.AllocationRequest{
-		Asks: []*si.AllocationAsk{
+		Allocations: []*si.Allocation{
 			{
 				AllocationKey: "real-alloc-1",
-				ResourceAsk: &si.Resource{
+				ResourcePerAlloc: &si.Resource{
 					Resources: map[string]*si.Quantity{
 						"memory": {Value: 10},
 						"vcore":  {Value: 1},
 					},
 				},
-				MaxAllocations: 1,
-				ApplicationID:  appID1,
-				TaskGroupName:  "tg-1",
+				ApplicationID: appID1,
+				TaskGroupName: "tg-1",
 			},
 			{
 				AllocationKey: "real-alloc-2",
-				ResourceAsk: &si.Resource{
+				ResourcePerAlloc: &si.Resource{
 					Resources: map[string]*si.Quantity{
 						"memory": {Value: 10},
 						"vcore":  {Value: 1},
 					},
 				},
-				MaxAllocations: 1,
-				ApplicationID:  appID1,
-				TaskGroupName:  "tg-2",
+				ApplicationID: appID1,
+				TaskGroupName: "tg-2",
 			},
 		},
 		RmID: "rm:123",
@@ -1055,13 +1104,13 @@ func TestPlaceholderRecovery(t *testing.T) { //nolint:funlen
 				{
 					ApplicationID:   appID1,
 					PartitionName:   "default",
-					AllocationID:    "ph-alloc-1-0",
+					AllocationKey:   "ph-alloc-1",
 					TerminationType: si.TerminationType_PLACEHOLDER_REPLACED,
 				},
 				{
 					ApplicationID:   appID1,
 					PartitionName:   "default",
-					AllocationID:    "ph-alloc-2-0",
+					AllocationKey:   "ph-alloc-2",
 					TerminationType: si.TerminationType_PLACEHOLDER_REPLACED,
 				},
 			},
@@ -1077,13 +1126,13 @@ func TestPlaceholderRecovery(t *testing.T) { //nolint:funlen
 				{
 					ApplicationID:   appID1,
 					PartitionName:   "default",
-					AllocationID:    "real-alloc-1-0",
+					AllocationKey:   "real-alloc-1",
 					TerminationType: si.TerminationType_STOPPED_BY_RM,
 				},
 				{
 					ApplicationID:   appID1,
 					PartitionName:   "default",
-					AllocationID:    "real-alloc-2-0",
+					AllocationKey:   "real-alloc-2",
 					TerminationType: si.TerminationType_STOPPED_BY_RM,
 				},
 			},
@@ -1094,3 +1143,16 @@ func TestPlaceholderRecovery(t *testing.T) { //nolint:funlen
 
 	ms.mockRM.waitForApplicationState(t, appID1, "Completing", 1000)
 }
+
+func registerAllocations(partition *scheduler.PartitionContext, allocs []*si.Allocation) error {
+	for _, alloc := range allocs {
+		_, allocCreated, err := partition.UpdateAllocation(objects.NewAllocationFromSI(alloc))
+		if err != nil {
+			return err
+		}
+		if !allocCreated {
+			return fmt.Errorf("no alloc created")
+		}
+	}
+	return nil
+}
