@@ -21,7 +21,9 @@ package cache
 import (
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
+	"reflect"
 	"sort"
 	"strconv"
 	"strings"
@@ -56,6 +58,11 @@ import (
 
 const registerNodeContextHandler = "RegisterNodeContextHandler"
 
+var (
+	ErrorPodNotFound  = errors.New("predicates were not run because pod was not found in cache")
+	ErrorNodeNotFound = errors.New("predicates were not run because node was not found in cache")
+)
+
 // context maintains scheduling state, like apps and apps' tasks.
 type Context struct {
 	applications   map[string]*Application        // apps
@@ -86,7 +93,7 @@ func NewContextWithBootstrapConfigMaps(apis client.APIProvider, bootstrapConfigM
 	ctx := &Context{
 		applications: make(map[string]*Application),
 		apiProvider:  apis,
-		namespace:    apis.GetAPIs().GetConf().Namespace,
+		namespace:    schedulerconf.GetSchedulerConf().Namespace,
 		configMaps:   bootstrapConfigMaps,
 		lock:         &locking.RWMutex{},
 		klogger:      klog.NewKlogr(),
@@ -104,33 +111,50 @@ func NewContextWithBootstrapConfigMaps(apis client.APIProvider, bootstrapConfigM
 	return ctx
 }
 
-func (ctx *Context) AddSchedulingEventHandlers() {
-	ctx.apiProvider.AddEventHandler(&client.ResourceEventHandlers{
+func (ctx *Context) AddSchedulingEventHandlers() error {
+	err := ctx.apiProvider.AddEventHandler(&client.ResourceEventHandlers{
 		Type:     client.ConfigMapInformerHandlers,
 		FilterFn: ctx.filterConfigMaps,
 		AddFn:    ctx.addConfigMaps,
 		UpdateFn: ctx.updateConfigMaps,
 		DeleteFn: ctx.deleteConfigMaps,
 	})
-	ctx.apiProvider.AddEventHandler(&client.ResourceEventHandlers{
+	if err != nil {
+		return err
+	}
+
+	err = ctx.apiProvider.AddEventHandler(&client.ResourceEventHandlers{
 		Type:     client.PriorityClassInformerHandlers,
 		FilterFn: ctx.filterPriorityClasses,
 		AddFn:    ctx.addPriorityClass,
 		UpdateFn: ctx.updatePriorityClass,
 		DeleteFn: ctx.deletePriorityClass,
 	})
-	ctx.apiProvider.AddEventHandler(&client.ResourceEventHandlers{
+	if err != nil {
+		return err
+	}
+
+	err = ctx.apiProvider.AddEventHandler(&client.ResourceEventHandlers{
 		Type:     client.NodeInformerHandlers,
 		AddFn:    ctx.addNode,
 		UpdateFn: ctx.updateNode,
 		DeleteFn: ctx.deleteNode,
 	})
-	ctx.apiProvider.AddEventHandler(&client.ResourceEventHandlers{
+	if err != nil {
+		return err
+	}
+
+	err = ctx.apiProvider.AddEventHandler(&client.ResourceEventHandlers{
 		Type:     client.PodInformerHandlers,
 		AddFn:    ctx.AddPod,
 		UpdateFn: ctx.UpdatePod,
 		DeleteFn: ctx.DeletePod,
 	})
+	if err != nil {
+		return err
+	}
+
+	return nil
 }
 
 func (ctx *Context) IsPluginMode() bool {
@@ -142,8 +166,6 @@ func (ctx *Context) addNode(obj interface{}) {
 }
 
 func (ctx *Context) updateNode(_, obj interface{}) {
-	ctx.lock.Lock()
-	defer ctx.lock.Unlock()
 	node, err := convertToNode(obj)
 	if err != nil {
 		log.Log(log.ShimContext).Error("node conversion failed", zap.Error(err))
@@ -178,7 +200,7 @@ func (ctx *Context) updateNodeInternal(node *v1.Node, register bool) {
 			if applicationID == "" {
 				ctx.updateForeignPod(pod)
 			} else {
-				ctx.updateYuniKornPod(pod)
+				ctx.updateYuniKornPod(applicationID, pod)
 			}
 		}
 
@@ -190,34 +212,21 @@ func (ctx *Context) updateNodeInternal(node *v1.Node, register bool) {
 		// existing node
 		prevCapacity := common.GetNodeResource(&prevNode.Status)
 		newCapacity := common.GetNodeResource(&node.Status)
-		prevReady := hasReadyCondition(prevNode)
-		newReady := hasReadyCondition(node)
 
 		if !common.Equals(prevCapacity, newCapacity) {
 			// update capacity
 			if capacity, occupied, ok := ctx.schedulerCache.UpdateCapacity(node.Name, newCapacity); ok {
-				if err := ctx.updateNodeResources(node, capacity, occupied, newReady); err != nil {
+				if err := ctx.updateNodeResources(node, capacity, occupied); err != nil {
 					log.Log(log.ShimContext).Warn("Failed to update node capacity", zap.Error(err))
 				}
 			} else {
 				log.Log(log.ShimContext).Warn("Failed to update cached node capacity", zap.String("nodeName", node.Name))
 			}
-		} else if newReady != prevReady {
-			// update readiness
-			if capacity, occupied, ok := ctx.schedulerCache.SnapshotResources(node.Name); ok {
-				if err := ctx.updateNodeResources(node, capacity, occupied, newReady); err != nil {
-					log.Log(log.ShimContext).Warn("Failed to update node readiness", zap.Error(err))
-				}
-			} else {
-				log.Log(log.ShimContext).Warn("Failed to snapshot cached node capacity", zap.String("nodeName", node.Name))
-			}
 		}
 	}
 }
 
 func (ctx *Context) deleteNode(obj interface{}) {
-	ctx.lock.Lock()
-	defer ctx.lock.Unlock()
 	var node *v1.Node
 	switch t := obj.(type) {
 	case *v1.Node:
@@ -237,9 +246,6 @@ func (ctx *Context) deleteNode(obj interface{}) {
 }
 
 func (ctx *Context) addNodesWithoutRegistering(nodes []*v1.Node) {
-	ctx.lock.Lock()
-	defer ctx.lock.Unlock()
-
 	for _, node := range nodes {
 		ctx.updateNodeInternal(node, false)
 	}
@@ -275,30 +281,31 @@ func (ctx *Context) AddPod(obj interface{}) {
 }
 
 func (ctx *Context) UpdatePod(_, newObj interface{}) {
-	ctx.lock.Lock()
-	defer ctx.lock.Unlock()
-
 	pod, err := utils.Convert2Pod(newObj)
 	if err != nil {
 		log.Log(log.ShimContext).Error("failed to update pod", zap.Error(err))
 		return
 	}
-	if utils.GetApplicationIDFromPod(pod) == "" {
+	applicationID := utils.GetApplicationIDFromPod(pod)
+	if applicationID == "" {
 		ctx.updateForeignPod(pod)
 	} else {
-		ctx.updateYuniKornPod(pod)
+		ctx.updateYuniKornPod(applicationID, pod)
 	}
 }
 
-func (ctx *Context) updateYuniKornPod(pod *v1.Pod) {
+func (ctx *Context) updateYuniKornPod(appID string, pod *v1.Pod) {
+	taskID := string(pod.UID)
+	app := ctx.getApplication(appID)
+	if app != nil {
+		if task := app.GetTask(taskID); task != nil {
+			task.SetTaskPod(pod)
+		}
+	}
+
 	// treat terminated pods like a remove
 	if utils.IsPodTerminated(pod) {
-		if taskMeta, ok := getTaskMetadata(pod); ok {
-			if app := ctx.getApplication(taskMeta.ApplicationID); app != nil {
-				ctx.notifyTaskComplete(taskMeta.ApplicationID, taskMeta.TaskID)
-			}
-		}
-
+		ctx.notifyTaskComplete(app, taskID)
 		log.Log(log.ShimContext).Debug("Request to update terminated pod, removing from cache", zap.String("podName", pod.Name))
 		ctx.schedulerCache.RemovePod(pod)
 		return
@@ -306,24 +313,22 @@ func (ctx *Context) updateYuniKornPod(pod *v1.Pod) {
 
 	if ctx.schedulerCache.UpdatePod(pod) {
 		// pod was accepted; ensure the application and task objects have been created
-		ctx.ensureAppAndTaskCreated(pod)
+		ctx.ensureAppAndTaskCreated(pod, app)
 	}
 }
 
-func (ctx *Context) ensureAppAndTaskCreated(pod *v1.Pod) {
-	// get app metadata
-	appMeta, ok := getAppMetadata(pod)
-	if !ok {
-		log.Log(log.ShimContext).Warn("BUG: Unable to retrieve application metadata from YuniKorn-managed Pod",
-			zap.String("namespace", pod.Namespace),
-			zap.String("name", pod.Name))
-		return
-	}
-
+func (ctx *Context) ensureAppAndTaskCreated(pod *v1.Pod, app *Application) {
 	// add app if it doesn't already exist
-	app := ctx.getApplication(appMeta.ApplicationID)
 	if app == nil {
-		app = ctx.addApplication(&AddApplicationRequest{
+		// get app metadata
+		appMeta, ok := getAppMetadata(pod)
+		if !ok {
+			log.Log(log.ShimContext).Warn("BUG: Unable to retrieve application metadata from YuniKorn-managed Pod",
+				zap.String("namespace", pod.Namespace),
+				zap.String("name", pod.Name))
+			return
+		}
+		app = ctx.AddApplication(&AddApplicationRequest{
 			Metadata: appMeta,
 		})
 	}
@@ -338,7 +343,7 @@ func (ctx *Context) ensureAppAndTaskCreated(pod *v1.Pod) {
 	}
 
 	// add task if it doesn't already exist
-	if _, taskErr := app.GetTask(string(pod.UID)); taskErr != nil {
+	if task := app.GetTask(string(pod.UID)); task == nil {
 		ctx.addTask(&AddTaskRequest{
 			Metadata: taskMeta,
 		})
@@ -347,8 +352,8 @@ func (ctx *Context) ensureAppAndTaskCreated(pod *v1.Pod) {
 
 func (ctx *Context) updateForeignPod(pod *v1.Pod) {
 	podStatusBefore := ""
-	oldPod, ok := ctx.schedulerCache.GetPod(string(pod.UID))
-	if ok {
+	oldPod := ctx.schedulerCache.GetPod(string(pod.UID))
+	if oldPod != nil {
 		podStatusBefore = string(oldPod.Status.Phase)
 	}
 
@@ -427,12 +432,8 @@ func (ctx *Context) DeletePod(obj interface{}) {
 }
 
 func (ctx *Context) deleteYuniKornPod(pod *v1.Pod) {
-	ctx.lock.Lock()
-	defer ctx.lock.Unlock()
 	if taskMeta, ok := getTaskMetadata(pod); ok {
-		if app := ctx.getApplication(taskMeta.ApplicationID); app != nil {
-			ctx.notifyTaskComplete(taskMeta.ApplicationID, taskMeta.TaskID)
-		}
+		ctx.notifyTaskComplete(ctx.GetApplication(taskMeta.ApplicationID), taskMeta.TaskID)
 	}
 
 	log.Log(log.ShimContext).Debug("removing pod from cache", zap.String("podName", pod.Name))
@@ -440,11 +441,8 @@ func (ctx *Context) deleteYuniKornPod(pod *v1.Pod) {
 }
 
 func (ctx *Context) deleteForeignPod(pod *v1.Pod) {
-	ctx.lock.Lock()
-	defer ctx.lock.Unlock()
-
-	oldPod, ok := ctx.schedulerCache.GetPod(string(pod.UID))
-	if !ok {
+	oldPod := ctx.schedulerCache.GetPod(string(pod.UID))
+	if oldPod == nil {
 		// if pod is not in scheduler cache, no node updates are needed
 		log.Log(log.ShimContext).Debug("unknown foreign pod deleted, no resource updated needed",
 			zap.String("namespace", pod.Namespace),
@@ -456,7 +454,7 @@ func (ctx *Context) deleteForeignPod(pod *v1.Pod) {
 	//   1. pod is already assigned to a node
 	//   2. pod was not in a terminal state before
 	//   3. pod references a known node
-	if oldPod != nil && !utils.IsPodTerminated(oldPod) {
+	if !utils.IsPodTerminated(oldPod) {
 		if !ctx.schedulerCache.IsPodOrphaned(string(oldPod.UID)) {
 			log.Log(log.ShimContext).Debug("foreign pod deleted, triggering occupied resource update",
 				zap.String("namespace", pod.Namespace),
@@ -482,7 +480,7 @@ func (ctx *Context) updateNodeOccupiedResources(nodeName string, namespace strin
 		return
 	}
 	if node, capacity, occupied, ok := ctx.schedulerCache.UpdateOccupiedResource(nodeName, namespace, podName, resource, opt); ok {
-		if err := ctx.updateNodeResources(node, capacity, occupied, hasReadyCondition(node)); err != nil {
+		if err := ctx.updateNodeResources(node, capacity, occupied); err != nil {
 			log.Log(log.ShimContext).Warn("scheduler rejected update to node occupied resources", zap.Error(err))
 		}
 	} else {
@@ -573,8 +571,6 @@ func (ctx *Context) addPriorityClass(obj interface{}) {
 }
 
 func (ctx *Context) updatePriorityClass(_, newObj interface{}) {
-	ctx.lock.Lock()
-	defer ctx.lock.Unlock()
 	if priorityClass := utils.Convert2PriorityClass(newObj); priorityClass != nil {
 		ctx.updatePriorityClassInternal(priorityClass)
 	}
@@ -585,9 +581,6 @@ func (ctx *Context) updatePriorityClassInternal(priorityClass *schedulingv1.Prio
 }
 
 func (ctx *Context) deletePriorityClass(obj interface{}) {
-	ctx.lock.Lock()
-	defer ctx.lock.Unlock()
-
 	log.Log(log.ShimContext).Debug("priorityClass deleted")
 	var priorityClass *schedulingv1.PriorityClass
 	switch t := obj.(type) {
@@ -651,32 +644,29 @@ func (ctx *Context) EventsToRegister(queueingHintFn framework.QueueingHintFn) []
 	return ctx.predManager.EventsToRegister(queueingHintFn)
 }
 
-// evaluate given predicates based on current context
+// IsPodFitNode evaluates given predicates based on current context
 func (ctx *Context) IsPodFitNode(name, node string, allocate bool) error {
-	ctx.lock.RLock()
-	defer ctx.lock.RUnlock()
-	if pod, ok := ctx.schedulerCache.GetPod(name); ok {
-		// if pod exists in cache, try to run predicates
-		if targetNode := ctx.schedulerCache.GetNode(node); targetNode != nil {
-			// need to lock cache here as predicates need a stable view into the cache
-			ctx.schedulerCache.LockForReads()
-			defer ctx.schedulerCache.UnlockForReads()
-			_, err := ctx.predManager.Predicates(pod, targetNode, allocate)
-			return err
-		}
+	pod := ctx.schedulerCache.GetPod(name)
+	if pod == nil {
+		return ErrorPodNotFound
 	}
-	return fmt.Errorf("predicates were not running because pod or node was not found in cache")
+	// if pod exists in cache, try to run predicates
+	targetNode := ctx.schedulerCache.GetNode(node)
+	if targetNode == nil {
+		return ErrorNodeNotFound
+	}
+	// need to lock cache here as predicates need a stable view into the cache
+	ctx.schedulerCache.LockForReads()
+	defer ctx.schedulerCache.UnlockForReads()
+	plugin, err := ctx.predManager.Predicates(pod, targetNode, allocate)
+	if err != nil {
+		err = errors.Join(fmt.Errorf("failed plugin: '%s'", plugin), err)
+	}
+	return err
 }
 
-func (ctx *Context) IsPodFitNodeViaPreemption(name, node string, allocations []string, startIndex int) (index int, ok bool) {
-	// assume minimal pods need killing if running in testing mode
-	if ctx.apiProvider.IsTestingMode() {
-		return startIndex, ok
-	}
-
-	ctx.lock.RLock()
-	defer ctx.lock.RUnlock()
-	if pod, ok := ctx.schedulerCache.GetPod(name); ok {
+func (ctx *Context) IsPodFitNodeViaPreemption(name, node string, allocations []string, startIndex int) (int, bool) {
+	if pod := ctx.schedulerCache.GetPod(name); pod != nil {
 		// if pod exists in cache, try to run predicates
 		if targetNode := ctx.schedulerCache.GetNode(node); targetNode != nil {
 			// need to lock cache here as predicates need a stable view into the cache
@@ -684,19 +674,15 @@ func (ctx *Context) IsPodFitNodeViaPreemption(name, node string, allocations []s
 			defer ctx.schedulerCache.UnlockForReads()
 
 			// look up each victim in the scheduler cache
-			victims := make([]*v1.Pod, 0)
-			for _, uid := range allocations {
-				if victim, ok := ctx.schedulerCache.GetPodNoLock(uid); ok {
-					victims = append(victims, victim)
-				} else {
-					// if pod isn't found, add a placeholder so that the list is still the same size
-					victims = append(victims, nil)
-				}
+			victims := make([]*v1.Pod, len(allocations))
+			for index, uid := range allocations {
+				victim := ctx.schedulerCache.GetPodNoLock(uid)
+				victims[index] = victim
 			}
 
 			// check predicates for a match
-			if index, ok := ctx.predManager.PreemptionPredicates(pod, targetNode, victims, startIndex); ok {
-				return index, ok
+			if index := ctx.predManager.PreemptionPredicates(pod, targetNode, victims, startIndex); index != -1 {
+				return index, true
 			}
 		}
 	}
@@ -712,7 +698,7 @@ func (ctx *Context) bindPodVolumes(pod *v1.Pod) error {
 	// during scheduling process as they have directly impact to other scheduling processes.
 	// when assumePodVolumes was called, we caches the value if all pod volumes are bound in schedulerCache,
 	// then here we just need to retrieve that value from cache, to skip bindings if volumes are already bound.
-	if assumedPod, exist := ctx.schedulerCache.GetPod(podKey); exist {
+	if assumedPod := ctx.schedulerCache.GetPod(podKey); assumedPod != nil {
 		if ctx.schedulerCache.ArePodVolumesAllBound(podKey) {
 			log.Log(log.ShimContext).Info("Binding Pod Volumes skipped: all volumes already bound",
 				zap.String("podName", pod.Name))
@@ -788,9 +774,7 @@ func (ctx *Context) bindPodVolumes(pod *v1.Pod) error {
 // this way, the core can make allocation decisions with consideration of
 // other assumed pods before they are actually bound to the node (bound is slow).
 func (ctx *Context) AssumePod(name, node string) error {
-	ctx.lock.Lock()
-	defer ctx.lock.Unlock()
-	if pod, ok := ctx.schedulerCache.GetPod(name); ok {
+	if pod := ctx.schedulerCache.GetPod(name); pod != nil {
 		// when add assumed pod, we make a copy of the pod to avoid
 		// modifying its original reference. otherwise, it may have
 		// race when some other go-routines accessing it in parallel.
@@ -799,7 +783,7 @@ func (ctx *Context) AssumePod(name, node string) error {
 			// assume pod volumes, this will update bindings info in cache
 			// assume pod volumes before assuming the pod
 			// this will update scheduler cache with essential PV/PVC binding info
-			var allBound = true
+			var allBound bool
 			var err error
 			// retrieve the volume claims
 			podVolumeClaims, err := ctx.apiProvider.GetAPIs().VolumeBinder.GetPodVolumeClaims(ctx.klogger, pod)
@@ -820,9 +804,9 @@ func (ctx *Context) AssumePod(name, node string) error {
 				return err
 			}
 			if len(reasons) > 0 {
-				sReasons := make([]string, 0)
-				for _, reason := range reasons {
-					sReasons = append(sReasons, string(reason))
+				sReasons := make([]string, len(reasons))
+				for i, reason := range reasons {
+					sReasons[i] = string(reason)
 				}
 				sReason := strings.Join(sReasons, ", ")
 				err = fmt.Errorf("pod %s has conflicting volume claims: %s", pod.Name, sReason)
@@ -849,10 +833,7 @@ func (ctx *Context) AssumePod(name, node string) error {
 // forget pod must be called when a pod is assumed to be running on a node,
 // but then for some reason it is failed to bind or released.
 func (ctx *Context) ForgetPod(name string) {
-	ctx.lock.Lock()
-	defer ctx.lock.Unlock()
-
-	if pod, ok := ctx.schedulerCache.GetPod(name); ok {
+	if pod := ctx.schedulerCache.GetPod(name); pod != nil {
 		log.Log(log.ShimContext).Debug("forget pod", zap.String("pod", pod.Name))
 		ctx.schedulerCache.ForgetPod(pod)
 		return
@@ -860,12 +841,6 @@ func (ctx *Context) ForgetPod(name string) {
 	log.Log(log.ShimContext).Debug("unable to forget pod: not found in cache", zap.String("pod", name))
 }
 
-func (ctx *Context) UpdateApplication(app *Application) {
-	ctx.lock.Lock()
-	defer ctx.lock.Unlock()
-	ctx.applications[app.applicationID] = app
-}
-
 // IsTaskMaybeSchedulable returns true if a task might be currently able to be scheduled. This uses a bloom filter
 // cached from a set of taskIDs to perform efficient negative lookups.
 func (ctx *Context) IsTaskMaybeSchedulable(taskID string) bool {
@@ -894,47 +869,19 @@ func (ctx *Context) StartPodAllocation(podKey string, nodeID string) bool {
 	return ctx.schedulerCache.StartPodAllocation(podKey, nodeID)
 }
 
-// inform the scheduler that the application is completed,
-// the complete state may further explained to completed_with_errors(failed) or successfully_completed,
-// either way we need to release all allocations (if exists) for this application
-func (ctx *Context) NotifyApplicationComplete(appID string) {
-	if app := ctx.GetApplication(appID); app != nil {
-		log.Log(log.ShimContext).Debug("NotifyApplicationComplete",
-			zap.String("appID", appID),
-			zap.String("currentAppState", app.GetApplicationState()))
-		ev := NewSimpleApplicationEvent(appID, CompleteApplication)
-		dispatcher.Dispatch(ev)
-	}
-}
-
-func (ctx *Context) NotifyApplicationFail(appID string) {
-	if app := ctx.GetApplication(appID); app != nil {
-		log.Log(log.ShimContext).Debug("NotifyApplicationFail",
-			zap.String("appID", appID),
-			zap.String("currentAppState", app.GetApplicationState()))
-		ev := NewSimpleApplicationEvent(appID, FailApplication)
-		dispatcher.Dispatch(ev)
-	}
-}
-
-func (ctx *Context) NotifyTaskComplete(appID, taskID string) {
-	ctx.lock.Lock()
-	defer ctx.lock.Unlock()
-	ctx.notifyTaskComplete(appID, taskID)
-}
-
-func (ctx *Context) notifyTaskComplete(appID, taskID string) {
-	log.Log(log.ShimContext).Debug("NotifyTaskComplete",
-		zap.String("appID", appID),
-		zap.String("taskID", taskID))
-	if app := ctx.getApplication(appID); app != nil {
-		log.Log(log.ShimContext).Debug("release allocation",
-			zap.String("appID", appID),
+func (ctx *Context) notifyTaskComplete(app *Application, taskID string) {
+	if app == nil {
+		log.Log(log.ShimContext).Debug("In notifyTaskComplete but app is nil",
 			zap.String("taskID", taskID))
-		ev := NewSimpleTaskEvent(appID, taskID, CompleteTask)
-		dispatcher.Dispatch(ev)
-		appEv := NewSimpleApplicationEvent(appID, AppTaskCompleted)
-		dispatcher.Dispatch(appEv)
+		return
+	}
+	log.Log(log.ShimContext).Debug("release allocation in notifyTaskComplete",
+		zap.String("appID", app.applicationID),
+		zap.String("taskID", taskID))
+	ev := NewSimpleTaskEvent(app.applicationID, taskID, CompleteTask)
+	dispatcher.Dispatch(ev)
+	if app.GetApplicationState() == ApplicationStates().Resuming {
+		dispatcher.Dispatch(NewSimpleApplicationEvent(app.applicationID, AppTaskCompleted))
 	}
 }
 
@@ -942,6 +889,8 @@ func (ctx *Context) notifyTaskComplete(appID, taskID string) {
 // adds the following tags to the request based on annotations (if exist):
 //   - namespace.resourcequota
 //   - namespace.parentqueue
+//   - namespace.resourceguaranteed
+//   - namespace.resourcemaxapps
 func (ctx *Context) updateApplicationTags(request *AddApplicationRequest, namespace string) {
 	namespaceObj := ctx.getNamespaceObject(namespace)
 	if namespaceObj == nil {
@@ -963,6 +912,12 @@ func (ctx *Context) updateApplicationTags(request *AddApplicationRequest, namesp
 		}
 	}
 
+	// add maxApps resource info as an app tag
+	maxApps := utils.GetNamespaceMaxAppsFromAnnotation(namespaceObj)
+	if maxApps != "" {
+		request.Metadata.Tags[siCommon.AppTagNamespaceResourceMaxApps] = maxApps
+	}
+
 	// add parent queue info as an app tag
 	parentQueue := utils.GetNameSpaceAnnotationValue(namespaceObj, constants.AnnotationParentQueue)
 	if parentQueue != "" {
@@ -994,10 +949,6 @@ func (ctx *Context) AddApplication(request *AddApplicationRequest) *Application
 	ctx.lock.Lock()
 	defer ctx.lock.Unlock()
 
-	return ctx.addApplication(request)
-}
-
-func (ctx *Context) addApplication(request *AddApplicationRequest) *Application {
 	log.Log(log.ShimContext).Debug("AddApplication", zap.Any("Request", request))
 	if app := ctx.getApplication(request.Metadata.ApplicationID); app != nil {
 		return app
@@ -1063,31 +1014,7 @@ func (ctx *Context) getApplication(appID string) *Application {
 	return nil
 }
 
-func (ctx *Context) RemoveApplication(appID string) error {
-	ctx.lock.Lock()
-	defer ctx.lock.Unlock()
-	if app, exist := ctx.applications[appID]; exist {
-		// get the non-terminated task alias
-		nonTerminatedTaskAlias := app.getNonTerminatedTaskAlias()
-		// check there are any non-terminated task or not
-		if len(nonTerminatedTaskAlias) > 0 {
-			return fmt.Errorf("failed to remove application %s because it still has task in non-terminated task, tasks: %s", appID, strings.Join(nonTerminatedTaskAlias, ","))
-		}
-		// send the update request to scheduler core
-		rr := common.CreateUpdateRequestForRemoveApplication(app.applicationID, app.partition)
-		if err := ctx.apiProvider.GetAPIs().SchedulerAPI.UpdateApplication(rr); err != nil {
-			log.Log(log.ShimContext).Error("failed to send remove application request to core", zap.Error(err))
-		}
-		delete(ctx.applications, appID)
-		log.Log(log.ShimContext).Info("app removed",
-			zap.String("appID", appID))
-
-		return nil
-	}
-	return fmt.Errorf("application %s is not found in the context", appID)
-}
-
-func (ctx *Context) RemoveApplicationInternal(appID string) {
+func (ctx *Context) RemoveApplication(appID string) {
 	ctx.lock.Lock()
 	defer ctx.lock.Unlock()
 	if _, exist := ctx.applications[appID]; !exist {
@@ -1099,8 +1026,6 @@ func (ctx *Context) RemoveApplicationInternal(appID string) {
 
 // this implements ApplicationManagementProtocol
 func (ctx *Context) AddTask(request *AddTaskRequest) *Task {
-	ctx.lock.Lock()
-	defer ctx.lock.Unlock()
 	return ctx.addTask(request)
 }
 
@@ -1109,8 +1034,8 @@ func (ctx *Context) addTask(request *AddTaskRequest) *Task {
 		zap.String("appID", request.Metadata.ApplicationID),
 		zap.String("taskID", request.Metadata.TaskID))
 	if app := ctx.getApplication(request.Metadata.ApplicationID); app != nil {
-		existingTask, err := app.GetTask(request.Metadata.TaskID)
-		if err != nil {
+		existingTask := app.GetTask(request.Metadata.TaskID)
+		if existingTask == nil {
 			var originator bool
 
 			// Is this task the originator of the application?
@@ -1160,16 +1085,14 @@ func (ctx *Context) RemoveTask(appID, taskID string) {
 }
 
 func (ctx *Context) getTask(appID string, taskID string) *Task {
-	ctx.lock.RLock()
-	defer ctx.lock.RUnlock()
-	app := ctx.getApplication(appID)
+	app := ctx.GetApplication(appID)
 	if app == nil {
 		log.Log(log.ShimContext).Debug("application is not found in the context",
 			zap.String("appID", appID))
 		return nil
 	}
-	task, err := app.GetTask(taskID)
-	if err != nil {
+	task := app.GetTask(taskID)
+	if task == nil {
 		log.Log(log.ShimContext).Debug("task is not found in applications",
 			zap.String("taskID", taskID),
 			zap.String("appID", appID))
@@ -1271,7 +1194,7 @@ func (ctx *Context) HandleContainerStateUpdate(request *si.UpdateContainerSchedu
 					Reason:  "SchedulingSkipped",
 					Message: request.Reason,
 				}) {
-				events.GetRecorder().Eventf(task.pod.DeepCopy(), nil,
+				events.GetRecorder().Eventf(task.GetTaskPod().DeepCopy(), nil,
 					v1.EventTypeNormal, "PodUnschedulable", "PodUnschedulable",
 					"Task %s is skipped from scheduling because the queue quota has been exceed", task.alias)
 			}
@@ -1286,7 +1209,7 @@ func (ctx *Context) HandleContainerStateUpdate(request *si.UpdateContainerSchedu
 					Reason:  v1.PodReasonUnschedulable,
 					Message: request.Reason,
 				}) {
-				events.GetRecorder().Eventf(task.pod.DeepCopy(), nil,
+				events.GetRecorder().Eventf(task.GetTaskPod().DeepCopy(), nil,
 					v1.EventTypeNormal, "PodUnschedulable", "PodUnschedulable",
 					"Task %s is pending for the requested resources become available", task.alias)
 			}
@@ -1300,41 +1223,70 @@ func (ctx *Context) HandleContainerStateUpdate(request *si.UpdateContainerSchedu
 func (ctx *Context) ApplicationEventHandler() func(obj interface{}) {
 	return func(obj interface{}) {
 		if event, ok := obj.(events.ApplicationEvent); ok {
-			app := ctx.GetApplication(event.GetApplicationID())
+			appID := event.GetApplicationID()
+			app := ctx.GetApplication(appID)
 			if app == nil {
-				log.Log(log.ShimContext).Error("failed to handle application event",
-					zap.String("reason", "application not exist"))
+				log.Log(log.ShimContext).Error("failed to handle application event, application does not exist",
+					zap.String("applicationID", appID))
 				return
 			}
+
 			if app.canHandle(event) {
 				if err := app.handle(event); err != nil {
 					log.Log(log.ShimContext).Error("failed to handle application event",
 						zap.String("event", event.GetEvent()),
+						zap.String("applicationID", appID),
 						zap.Error(err))
 				}
+				return
 			}
+
+			log.Log(log.ShimContext).Error("application event cannot be handled in the current state",
+				zap.String("applicationID", appID),
+				zap.String("event", event.GetEvent()),
+				zap.String("state", app.sm.Current()))
+			return
 		}
+
+		log.Log(log.ShimContext).Error("could not handle application event",
+			zap.String("type", reflect.TypeOf(obj).Name()))
 	}
 }
 
 func (ctx *Context) TaskEventHandler() func(obj interface{}) {
 	return func(obj interface{}) {
 		if event, ok := obj.(events.TaskEvent); ok {
-			task := ctx.getTask(event.GetApplicationID(), event.GetTaskID())
+			appID := event.GetApplicationID()
+			taskID := event.GetTaskID()
+			task := ctx.getTask(appID, taskID)
 			if task == nil {
-				log.Log(log.ShimContext).Error("failed to handle application event")
+				log.Log(log.ShimContext).Debug("failed to handle task event, task does not exist",
+					zap.String("applicationID", appID),
+					zap.String("taskID", taskID))
 				return
 			}
+
 			if task.canHandle(event) {
 				if err := task.handle(event); err != nil {
 					log.Log(log.ShimContext).Error("failed to handle task event",
-						zap.String("applicationID", task.applicationID),
-						zap.String("taskID", task.taskID),
+						zap.String("applicationID", appID),
+						zap.String("taskID", taskID),
 						zap.String("event", event.GetEvent()),
 						zap.Error(err))
 				}
+				return
 			}
+
+			log.Log(log.ShimContext).Error("task event cannot be handled in the current state",
+				zap.String("applicationID", appID),
+				zap.String("taskID", taskID),
+				zap.String("event", event.GetEvent()),
+				zap.String("state", task.sm.Current()))
+			return
 		}
+
+		log.Log(log.ShimContext).Error("could not handle task event",
+			zap.String("type", reflect.TypeOf(obj).Name()))
 	}
 }
 
@@ -1402,7 +1354,7 @@ func (ctx *Context) InitializeState() error {
 		log.Log(log.ShimContext).Error("failed to load nodes", zap.Error(err))
 		return err
 	}
-	acceptedNodes, err := ctx.RegisterNodes(nodes)
+	acceptedNodes, err := ctx.registerNodes(nodes)
 	if err != nil {
 		log.Log(log.ShimContext).Error("failed to register nodes", zap.Error(err))
 		return err
@@ -1427,7 +1379,11 @@ func (ctx *Context) InitializeState() error {
 
 	// Step 5: Start scheduling event handlers. At this point, initialization is mostly complete, and any existing
 	// objects will show up as newly added objects. Since the add/update event handlers are idempotent, this is fine.
-	ctx.AddSchedulingEventHandlers()
+	err = ctx.AddSchedulingEventHandlers()
+	if err != nil {
+		log.Log(log.Admission).Error("failed to add scheduling event handlers", zap.Error(err))
+		return err
+	}
 
 	// Step 6: Finalize priority classes. Between the start of initialization and when the informer event handlers are
 	// registered, it is possible that a priority class object was deleted. Process them again and remove
@@ -1518,17 +1474,11 @@ func (ctx *Context) registerNode(node *v1.Node) error {
 	return nil
 }
 
-func (ctx *Context) RegisterNodes(nodes []*v1.Node) ([]*v1.Node, error) {
-	ctx.lock.Lock()
-	defer ctx.lock.Unlock()
-	return ctx.registerNodes(nodes)
-}
-
-// registerNodes registers the nodes to the scheduler core.
-// This method must be called while holding the Context write lock.
 func (ctx *Context) registerNodes(nodes []*v1.Node) ([]*v1.Node, error) {
 	nodesToRegister := make([]*si.NodeInfo, 0)
 	pendingNodes := make(map[string]*v1.Node)
+	acceptedNodes := make([]*v1.Node, 0)
+	rejectedNodes := make([]*v1.Node, 0)
 
 	// Generate a NodeInfo object for each node and add to the registration request
 	for _, node := range nodes {
@@ -1540,43 +1490,19 @@ func (ctx *Context) registerNodes(nodes []*v1.Node) ([]*v1.Node, error) {
 			Attributes: map[string]string{
 				constants.DefaultNodeAttributeHostNameKey: node.Name,
 				constants.DefaultNodeAttributeRackNameKey: constants.DefaultRackName,
-				siCommon.NodeReadyAttribute:               strconv.FormatBool(hasReadyCondition(node)),
 			},
 			SchedulableResource: common.GetNodeResource(&nodeStatus),
 			OccupiedResource:    common.NewResourceBuilder().Build(),
-			ExistingAllocations: make([]*si.Allocation, 0),
 		})
 		pendingNodes[node.Name] = node
 	}
 
-	acceptedNodes, rejectedNodes, err := ctx.registerNodesInternal(nodesToRegister, pendingNodes)
-	if err != nil {
-		log.Log(log.ShimContext).Error("Failed to register nodes", zap.Error(err))
-		return nil, err
-	}
-
-	for _, node := range acceptedNodes {
-		// post a successful event to the node
-		events.GetRecorder().Eventf(node.DeepCopy(), nil, v1.EventTypeNormal, "NodeAccepted", "NodeAccepted",
-			fmt.Sprintf("node %s is accepted by the scheduler", node.Name))
-	}
-	for _, node := range rejectedNodes {
-		// post a failure event to the node
-		events.GetRecorder().Eventf(node.DeepCopy(), nil, v1.EventTypeWarning, "NodeRejected", "NodeRejected",
-			fmt.Sprintf("node %s is rejected by the scheduler", node.Name))
-	}
-
-	return acceptedNodes, nil
-}
-
-func (ctx *Context) registerNodesInternal(nodesToRegister []*si.NodeInfo, pendingNodes map[string]*v1.Node) ([]*v1.Node, []*v1.Node, error) {
-	acceptedNodes := make([]*v1.Node, 0)
-	rejectedNodes := make([]*v1.Node, 0)
-
 	var wg sync.WaitGroup
+
 	// initialize wait group with the number of responses we expect
 	wg.Add(len(pendingNodes))
 
+	// register with the dispatcher so that we can track our response
 	handlerID := fmt.Sprintf("%s-%d", registerNodeContextHandler, ctx.txnID.Add(1))
 	dispatcher.RegisterEventHandler(handlerID, dispatcher.EventTypeNode, func(event interface{}) {
 		nodeEvent, ok := event.(CachedSchedulerNodeEvent)
@@ -1608,17 +1534,24 @@ func (ctx *Context) registerNodesInternal(nodesToRegister []*si.NodeInfo, pendin
 		RmID:  schedulerconf.GetSchedulerConf().ClusterID,
 	}); err != nil {
 		log.Log(log.ShimContext).Error("Failed to register nodes", zap.Error(err))
-		return nil, nil, err
+		return nil, err
 	}
 
-	// write lock must always be held at this point, releasing it while waiting to avoid any potential deadlocks
-	ctx.lock.Unlock()
-	defer ctx.lock.Lock()
-
 	// wait for all responses to accumulate
 	wg.Wait()
 
-	return acceptedNodes, rejectedNodes, nil
+	for _, node := range acceptedNodes {
+		// post a successful event to the node
+		events.GetRecorder().Eventf(node.DeepCopy(), nil, v1.EventTypeNormal, "NodeAccepted", "NodeAccepted",
+			fmt.Sprintf("node %s is accepted by the scheduler", node.Name))
+	}
+	for _, node := range rejectedNodes {
+		// post a failure event to the node
+		events.GetRecorder().Eventf(node.DeepCopy(), nil, v1.EventTypeWarning, "NodeRejected", "NodeRejected",
+			fmt.Sprintf("node %s is rejected by the scheduler", node.Name))
+	}
+
+	return acceptedNodes, nil
 }
 
 func (ctx *Context) decommissionNode(node *v1.Node) error {
@@ -1626,8 +1559,8 @@ func (ctx *Context) decommissionNode(node *v1.Node) error {
 	return ctx.apiProvider.GetAPIs().SchedulerAPI.UpdateNode(request)
 }
 
-func (ctx *Context) updateNodeResources(node *v1.Node, capacity *si.Resource, occupied *si.Resource, ready bool) error {
-	request := common.CreateUpdateRequestForUpdatedNode(node.Name, capacity, occupied, ready)
+func (ctx *Context) updateNodeResources(node *v1.Node, capacity *si.Resource, occupied *si.Resource) error {
+	request := common.CreateUpdateRequestForUpdatedNode(node.Name, capacity, occupied)
 	return ctx.apiProvider.GetAPIs().SchedulerAPI.UpdateNode(request)
 }
 
@@ -1642,11 +1575,9 @@ func (ctx *Context) enableNodes(nodes []*v1.Node) error {
 	for _, node := range nodes {
 		log.Log(log.ShimContext).Info("Enabling node", zap.String("name", node.Name))
 		nodesToEnable = append(nodesToEnable, &si.NodeInfo{
-			NodeID: node.Name,
-			Action: si.NodeInfo_DRAIN_TO_SCHEDULABLE,
-			Attributes: map[string]string{
-				siCommon.NodeReadyAttribute: strconv.FormatBool(hasReadyCondition(node)),
-			},
+			NodeID:     node.Name,
+			Action:     si.NodeInfo_DRAIN_TO_SCHEDULABLE,
+			Attributes: map[string]string{},
 		})
 	}
 
@@ -1675,9 +1606,6 @@ func (ctx *Context) finalizeNodes(existingNodes []*v1.Node) error {
 		nodeMap[node.Name] = node
 	}
 
-	ctx.lock.Lock()
-	defer ctx.lock.Unlock()
-
 	// find any existing nodes that no longer exist
 	for _, node := range existingNodes {
 		if _, ok := nodeMap[node.Name]; !ok {
@@ -1777,7 +1705,6 @@ func getExistingAllocation(pod *v1.Pod) *si.Allocation {
 		return &si.Allocation{
 			AllocationKey:    string(pod.UID),
 			AllocationTags:   meta.Tags,
-			AllocationID:     string(pod.UID),
 			ResourcePerAlloc: common.GetPodResource(pod),
 			NodeID:           pod.Spec.NodeName,
 			ApplicationID:    meta.ApplicationID,
@@ -1795,14 +1722,3 @@ func convertToNode(obj interface{}) (*v1.Node, error) {
 	}
 	return nil, fmt.Errorf("cannot convert to *v1.Node: %v", obj)
 }
-
-func hasReadyCondition(node *v1.Node) bool {
-	if node != nil {
-		for _, condition := range node.Status.Conditions {
-			if condition.Type == v1.NodeReady && condition.Status == v1.ConditionTrue {
-				return true
-			}
-		}
-	}
-	return false
-}
