@@ -51,44 +51,29 @@ func (callback *AsyncRMCallback) UpdateAllocation(response *si.AllocationRespons
 		// got allocation for pod, bind pod to the scheduled node
 		log.Log(log.ShimRMCallback).Debug("callback: response to new allocation",
 			zap.String("allocationKey", alloc.AllocationKey),
-			zap.String("allocationID", alloc.AllocationID),
 			zap.String("applicationID", alloc.ApplicationID),
 			zap.String("nodeID", alloc.NodeID))
 
 		// update cache
 		task := callback.context.getTask(alloc.ApplicationID, alloc.AllocationKey)
-		if task != nil {
-			task.setAllocationID(alloc.AllocationID)
-		} else {
+		if task == nil {
 			log.Log(log.ShimRMCallback).Warn("Unable to get task", zap.String("taskID", alloc.AllocationKey))
+			continue
 		}
+
+		task.setAllocationKey(alloc.AllocationKey)
+
 		if err := callback.context.AssumePod(alloc.AllocationKey, alloc.NodeID); err != nil {
-			if task != nil {
-				task.failWithEvent(err.Error(), "AssumePodError")
-			}
+			task.FailWithEvent(err.Error(), "AssumePodError")
 			return err
 		}
-		if app := callback.context.GetApplication(alloc.ApplicationID); app != nil {
-			if task != nil {
-				if utils.IsAssignedPod(task.GetTaskPod()) {
-					// task is already bound, fixup state and continue
-					task.MarkPreviouslyAllocated(alloc.AllocationID, alloc.NodeID)
-				} else {
-					ev := NewAllocateTaskEvent(app.GetApplicationID(), alloc.AllocationKey, alloc.AllocationID, alloc.NodeID)
-					dispatcher.Dispatch(ev)
-				}
-			}
-		}
-	}
 
-	for _, reject := range response.Rejected {
-		// request rejected by the scheduler, put it back and try scheduling again
-		log.Log(log.ShimRMCallback).Debug("callback: response to rejected ask",
-			zap.String("allocationKey", reject.AllocationKey))
-		if app := callback.context.GetApplication(reject.ApplicationID); app != nil {
-			dispatcher.Dispatch(NewRejectTaskEvent(app.GetApplicationID(), reject.AllocationKey,
-				fmt.Sprintf("task %s ask from application %s is rejected by scheduler",
-					reject.AllocationKey, reject.ApplicationID)))
+		if utils.IsAssignedPod(task.GetTaskPod()) {
+			// task is already bound, fixup state and continue
+			task.MarkPreviouslyAllocated(alloc.AllocationKey, alloc.NodeID)
+		} else {
+			ev := NewAllocateTaskEvent(alloc.ApplicationID, task.taskID, alloc.AllocationKey, alloc.NodeID)
+			dispatcher.Dispatch(ev)
 		}
 	}
 
@@ -96,16 +81,14 @@ func (callback *AsyncRMCallback) UpdateAllocation(response *si.AllocationRespons
 		// request rejected by the scheduler, reject it
 		log.Log(log.ShimRMCallback).Debug("callback: response to rejected allocation",
 			zap.String("allocationKey", reject.AllocationKey))
-		if app := callback.context.GetApplication(reject.ApplicationID); app != nil {
-			dispatcher.Dispatch(NewRejectTaskEvent(app.GetApplicationID(), reject.AllocationKey,
-				fmt.Sprintf("task %s allocation from application %s is rejected by scheduler",
-					reject.AllocationKey, reject.ApplicationID)))
-		}
+		dispatcher.Dispatch(NewRejectTaskEvent(reject.ApplicationID, reject.AllocationKey,
+			fmt.Sprintf("task %s allocation from application %s is rejected by scheduler",
+				reject.AllocationKey, reject.ApplicationID)))
 	}
 
 	for _, release := range response.Released {
 		log.Log(log.ShimRMCallback).Debug("callback: response to released allocations",
-			zap.String("AllocationID", release.AllocationID))
+			zap.String("AllocationKey", release.AllocationKey))
 
 		// update cache
 		callback.context.ForgetPod(release.GetAllocationKey())
@@ -113,17 +96,7 @@ func (callback *AsyncRMCallback) UpdateAllocation(response *si.AllocationRespons
 		// TerminationType 0 mean STOPPED_BY_RM
 		if release.TerminationType != si.TerminationType_STOPPED_BY_RM {
 			// send release app allocation to application states machine
-			ev := NewReleaseAppAllocationEvent(release.ApplicationID, release.TerminationType, release.AllocationID)
-			dispatcher.Dispatch(ev)
-		}
-	}
-
-	for _, ask := range response.ReleasedAsks {
-		log.Log(log.ShimRMCallback).Debug("callback: response to released allocations",
-			zap.String("allocation key", ask.AllocationKey))
-
-		if ask.TerminationType == si.TerminationType_TIMEOUT {
-			ev := NewReleaseAppAllocationAskEvent(ask.ApplicationID, ask.TerminationType, ask.AllocationKey)
+			ev := NewReleaseAppAllocationEvent(release.ApplicationID, release.TerminationType, release.AllocationKey)
 			dispatcher.Dispatch(ev)
 		}
 	}
@@ -166,23 +139,16 @@ func (callback *AsyncRMCallback) UpdateApplication(response *si.ApplicationRespo
 			zap.String("new status", updated.State))
 		switch updated.State {
 		case ApplicationStates().Completed:
-			callback.context.RemoveApplicationInternal(updated.ApplicationID)
+			callback.context.RemoveApplication(updated.ApplicationID)
 		case ApplicationStates().Resuming:
 			app := callback.context.GetApplication(updated.ApplicationID)
 			if app != nil && app.GetApplicationState() == ApplicationStates().Reserving {
 				ev := NewResumingApplicationEvent(updated.ApplicationID)
 				dispatcher.Dispatch(ev)
-
-				// handle status update
-				dispatcher.Dispatch(NewApplicationStatusChangeEvent(updated.ApplicationID, AppStateChange, updated.State))
 			}
-		default:
-			if updated.State == ApplicationStates().Failing || updated.State == ApplicationStates().Failed {
-				ev := NewFailApplicationEvent(updated.ApplicationID, updated.Message)
-				dispatcher.Dispatch(ev)
-			}
-			// handle status update
-			dispatcher.Dispatch(NewApplicationStatusChangeEvent(updated.ApplicationID, AppStateChange, updated.State))
+		case ApplicationStates().Failing, ApplicationStates().Failed:
+			ev := NewFailApplicationEvent(updated.ApplicationID, updated.Message)
+			dispatcher.Dispatch(ev)
 		}
 	}
 	return nil
