@@ -19,6 +19,7 @@
 package client
 
 import (
+	"fmt"
 	"time"
 
 	"go.uber.org/zap"
@@ -26,13 +27,13 @@ import (
 	schedv1 "k8s.io/api/scheduling/v1"
 	"k8s.io/client-go/informers"
 	k8fake "k8s.io/client-go/kubernetes/fake"
+	appsv1 "k8s.io/client-go/listers/apps/v1"
 	corev1 "k8s.io/client-go/listers/core/v1"
 	storagev1 "k8s.io/client-go/listers/storage/v1"
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/kubernetes/pkg/scheduler/framework/plugins/volumebinding"
 
 	"github.com/apache/yunikorn-k8shim/pkg/common/test"
-	"github.com/apache/yunikorn-k8shim/pkg/conf"
 	"github.com/apache/yunikorn-k8shim/pkg/locking"
 	"github.com/apache/yunikorn-k8shim/pkg/log"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
@@ -66,31 +67,18 @@ const (
 func NewMockedAPIProvider(showError bool) *MockedAPIProvider {
 	return &MockedAPIProvider{
 		clients: &Clients{
-			conf: &conf.SchedulerConf{
-				ClusterID:            "yk-test-cluster",
-				ClusterVersion:       "0.1",
-				PolicyGroup:          "queues",
-				Interval:             0,
-				KubeConfig:           "",
-				VolumeBindTimeout:    0,
-				TestMode:             true,
-				EventChannelCapacity: 0,
-				DispatchTimeout:      0,
-				KubeQPS:              0,
-				KubeBurst:            0,
-				Namespace:            "yunikorn",
-			},
 			KubeClient:            NewKubeClientMock(showError),
 			SchedulerAPI:          test.NewSchedulerAPIMock(),
 			PodInformer:           test.NewMockedPodInformer(),
 			NodeInformer:          test.NewMockedNodeInformer(),
 			ConfigMapInformer:     test.NewMockedConfigMapInformer(),
-			PVInformer:            &MockedPersistentVolumeInformer{},
-			PVCInformer:           &MockedPersistentVolumeClaimInformer{},
-			StorageInformer:       &MockedStorageClassInformer{},
+			PVInformer:            NewMockedPersistentVolumeInformer(),
+			PVCInformer:           NewMockedPersistentVolumeClaimInformer(),
+			StorageInformer:       NewMockedStorageClassInformer(),
 			VolumeBinder:          test.NewVolumeBinderMock(),
 			NamespaceInformer:     test.NewMockNamespaceInformer(false),
 			PriorityClassInformer: test.NewMockPriorityClassInformer(),
+			CSINodeInformer:       NewMockedCSINodeInformer(),
 			InformerFactory:       informers.NewSharedInformerFactory(k8fake.NewSimpleClientset(), time.Second*60),
 		},
 		events:       make(chan informerEvent),
@@ -218,16 +206,18 @@ func (m *MockedAPIProvider) IsTestingMode() bool {
 	return true
 }
 
-func (m *MockedAPIProvider) AddEventHandler(handlers *ResourceEventHandlers) {
+func (m *MockedAPIProvider) AddEventHandler(handlers *ResourceEventHandlers) error {
 	m.Lock()
 	defer m.Unlock()
 
 	if !m.running {
-		return
+		return fmt.Errorf("mocked API provider is not running")
 	}
 
 	m.eventHandler <- handlers
 	log.Log(log.Test).Info("registering event handler", zap.Stringer("type", handlers.Type))
+
+	return nil
 }
 
 func (m *MockedAPIProvider) RunEventHandler() {
@@ -416,10 +406,18 @@ func (m *MockedAPIProvider) GetBoundPods(clear bool) []BoundPod {
 }
 
 // MockedPersistentVolumeInformer implements PersistentVolumeInformer interface
-type MockedPersistentVolumeInformer struct{}
+type MockedPersistentVolumeInformer struct {
+	informer cache.SharedIndexInformer
+}
+
+func NewMockedPersistentVolumeInformer() *MockedPersistentVolumeInformer {
+	return &MockedPersistentVolumeInformer{
+		informer: &test.SharedInformerMock{},
+	}
+}
 
 func (m *MockedPersistentVolumeInformer) Informer() cache.SharedIndexInformer {
-	return nil
+	return m.informer
 }
 
 func (m *MockedPersistentVolumeInformer) Lister() corev1.PersistentVolumeLister {
@@ -427,10 +425,18 @@ func (m *MockedPersistentVolumeInformer) Lister() corev1.PersistentVolumeLister
 }
 
 // MockedPersistentVolumeClaimInformer implements PersistentVolumeClaimInformer interface
-type MockedPersistentVolumeClaimInformer struct{}
+type MockedPersistentVolumeClaimInformer struct {
+	informer cache.SharedIndexInformer
+}
+
+func NewMockedPersistentVolumeClaimInformer() *MockedPersistentVolumeClaimInformer {
+	return &MockedPersistentVolumeClaimInformer{
+		informer: &test.SharedInformerMock{},
+	}
+}
 
 func (m *MockedPersistentVolumeClaimInformer) Informer() cache.SharedIndexInformer {
-	return nil
+	return m.informer
 }
 
 func (m *MockedPersistentVolumeClaimInformer) Lister() corev1.PersistentVolumeClaimLister {
@@ -438,16 +444,150 @@ func (m *MockedPersistentVolumeClaimInformer) Lister() corev1.PersistentVolumeCl
 }
 
 // MockedStorageClassInformer implements StorageClassInformer interface
-type MockedStorageClassInformer struct{}
+type MockedStorageClassInformer struct {
+	informer cache.SharedIndexInformer
+}
+
+func NewMockedStorageClassInformer() *MockedStorageClassInformer {
+	return &MockedStorageClassInformer{
+		informer: &test.SharedInformerMock{},
+	}
+}
 
 func (m *MockedStorageClassInformer) Informer() cache.SharedIndexInformer {
-	return nil
+	return m.informer
 }
 
 func (m *MockedStorageClassInformer) Lister() storagev1.StorageClassLister {
 	return nil
 }
 
+type MockedCSINodeInformer struct {
+	informer cache.SharedIndexInformer
+}
+
+func (m *MockedCSINodeInformer) Informer() cache.SharedIndexInformer {
+	return m.informer
+}
+
+func (m *MockedCSINodeInformer) Lister() storagev1.CSINodeLister {
+	return nil
+}
+
+func NewMockedCSINodeInformer() *MockedCSINodeInformer {
+	return &MockedCSINodeInformer{
+		informer: &test.SharedInformerMock{},
+	}
+}
+
+type MockedCSIDriverInformer struct {
+	informer cache.SharedIndexInformer
+}
+
+func (m *MockedCSIDriverInformer) Informer() cache.SharedIndexInformer {
+	return m.informer
+}
+
+func (m *MockedCSIDriverInformer) Lister() storagev1.CSIDriverLister {
+	return nil
+}
+
+func NewMockedCSIDriverInformer() *MockedCSIDriverInformer {
+	return &MockedCSIDriverInformer{
+		informer: &test.SharedInformerMock{},
+	}
+}
+
+type MockedCSIStorageCapacityInformer struct {
+	informer cache.SharedIndexInformer
+}
+
+func (m *MockedCSIStorageCapacityInformer) Informer() cache.SharedIndexInformer {
+	return m.informer
+}
+
+func (m *MockedCSIStorageCapacityInformer) Lister() storagev1.CSIStorageCapacityLister {
+	return nil
+}
+
+func NewMockedCSIStorageCapacityInformer() *MockedCSIStorageCapacityInformer {
+	return &MockedCSIStorageCapacityInformer{
+		informer: &test.SharedInformerMock{},
+	}
+}
+
+type MockedServiceInformer struct {
+	informer cache.SharedIndexInformer
+}
+
+func (m *MockedServiceInformer) Informer() cache.SharedIndexInformer {
+	return m.informer
+}
+
+func (m *MockedServiceInformer) Lister() corev1.ServiceLister {
+	return nil
+}
+
+func NewMockedServiceInformer() *MockedServiceInformer {
+	return &MockedServiceInformer{
+		informer: &test.SharedInformerMock{},
+	}
+}
+
+type MockedReplicationControllerInformer struct {
+	informer cache.SharedIndexInformer
+}
+
+func (m *MockedReplicationControllerInformer) Informer() cache.SharedIndexInformer {
+	return m.informer
+}
+
+func (m *MockedReplicationControllerInformer) Lister() corev1.ReplicationControllerLister {
+	return nil
+}
+
+func NewMockedReplicationControllerInformer() *MockedReplicationControllerInformer {
+	return &MockedReplicationControllerInformer{
+		informer: &test.SharedInformerMock{},
+	}
+}
+
+type MockedReplicaSetInformer struct {
+	informer cache.SharedIndexInformer
+}
+
+func (m *MockedReplicaSetInformer) Informer() cache.SharedIndexInformer {
+	return m.informer
+}
+
+func (m *MockedReplicaSetInformer) Lister() appsv1.ReplicaSetLister {
+	return nil
+}
+
+func NewMockedReplicaSetInformer() *MockedReplicaSetInformer {
+	return &MockedReplicaSetInformer{
+		informer: &test.SharedInformerMock{},
+	}
+}
+
+type MockedStatefulSetInformer struct {
+	informer cache.SharedIndexInformer
+}
+
+func (m *MockedStatefulSetInformer) Informer() cache.SharedIndexInformer {
+	return m.informer
+}
+
+func (m *MockedStatefulSetInformer) Lister() appsv1.StatefulSetLister {
+	return nil
+}
+
+func NewMockedStatefulSetInformer() *MockedStatefulSetInformer {
+	return &MockedStatefulSetInformer{
+		informer: &test.SharedInformerMock{},
+	}
+}
+
 func (m *MockedAPIProvider) SetVolumeBinder(binder volumebinding.SchedulerVolumeBinder) {
 	m.clients.VolumeBinder = binder
 }
