@@ -21,13 +21,10 @@ package user_group_limit_test
 import (
 	"encoding/json"
 	"fmt"
+	"net/url"
 	"runtime"
 	"time"
 
-	"github.com/onsi/ginkgo/v2"
-	"github.com/onsi/gomega"
-	v1 "k8s.io/api/core/v1"
-
 	"github.com/apache/yunikorn-core/pkg/common/configs"
 	"github.com/apache/yunikorn-core/pkg/common/resources"
 	"github.com/apache/yunikorn-core/pkg/webservice/dao"
@@ -35,11 +32,18 @@ import (
 	amconf "github.com/apache/yunikorn-k8shim/pkg/admission/conf"
 	"github.com/apache/yunikorn-k8shim/pkg/common/constants"
 	tests "github.com/apache/yunikorn-k8shim/test/e2e"
+	"github.com/apache/yunikorn-k8shim/test/e2e/framework/configmanager"
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/helpers/common"
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/helpers/k8s"
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/helpers/yunikorn"
 	siCommon "github.com/apache/yunikorn-scheduler-interface/lib/go/common"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
+
+	"github.com/onsi/ginkgo/v2"
+	"github.com/onsi/gomega"
+
+	v1 "k8s.io/api/core/v1"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 )
 
 type TestType int
@@ -67,7 +71,6 @@ var (
 	ns                    *v1.Namespace
 	dev                   = "dev" + common.RandSeq(5)
 	oldConfigMap          = new(v1.ConfigMap)
-	annotation            = "ann-" + common.RandSeq(10)
 	admissionCustomConfig = map[string]string{
 		"log.core.scheduler.ugm.level":   "debug",
 		amconf.AMAccessControlBypassAuth: constants.True,
@@ -109,10 +112,9 @@ var _ = ginkgo.AfterSuite(func() {
 var _ = ginkgo.Describe("UserGroupLimit", func() {
 	ginkgo.It("Verify_maxresources_with_a_specific_user_limit", func() {
 		ginkgo.By("Update config")
-		annotation = "ann-" + common.RandSeq(10)
 		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
 		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
-			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", annotation, admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
+			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
 				// remove placement rules so we can control queue
 				sc.Partitions[0].PlacementRules = nil
 
@@ -157,10 +159,9 @@ var _ = ginkgo.Describe("UserGroupLimit", func() {
 
 	ginkgo.It("Verify_maxapplications_with_a_specific_user_limit", func() {
 		ginkgo.By("Update config")
-		annotation = "ann-" + common.RandSeq(10)
 		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
 		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
-			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", annotation, admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
+			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
 				// remove placement rules so we can control queue
 				sc.Partitions[0].PlacementRules = nil
 
@@ -205,10 +206,9 @@ var _ = ginkgo.Describe("UserGroupLimit", func() {
 
 	ginkgo.It("Verify_maxresources_with_a_specific_group_limit", func() {
 		ginkgo.By("Update config")
-		annotation = "ann-" + common.RandSeq(10)
 		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
 		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
-			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", annotation, admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
+			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
 				// remove placement rules so we can control queue
 				sc.Partitions[0].PlacementRules = nil
 
@@ -251,10 +251,9 @@ var _ = ginkgo.Describe("UserGroupLimit", func() {
 
 	ginkgo.It("Verify_maxapplications_with_a_specific_group_limit", func() {
 		ginkgo.By("Update config")
-		annotation = "ann-" + common.RandSeq(10)
 		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
 		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
-			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", annotation, admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
+			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
 				// remove placement rules so we can control queue
 				sc.Partitions[0].PlacementRules = nil
 
@@ -297,10 +296,9 @@ var _ = ginkgo.Describe("UserGroupLimit", func() {
 
 	ginkgo.It("Verify_maxresources_with_user_limit_lower_than_group_limit", func() {
 		ginkgo.By("Update config")
-		annotation = "ann-" + common.RandSeq(10)
 		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
 		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
-			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", annotation, admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
+			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
 				// remove placement rules so we can control queue
 				sc.Partitions[0].PlacementRules = nil
 
@@ -342,10 +340,9 @@ var _ = ginkgo.Describe("UserGroupLimit", func() {
 
 	ginkgo.It("Verify_maxresources_with_group_limit_lower_than_user_limit", func() {
 		ginkgo.By("Update config")
-		annotation = "ann-" + common.RandSeq(10)
 		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
 		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
-			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", annotation, admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
+			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
 				// remove placement rules so we can control queue
 				sc.Partitions[0].PlacementRules = nil
 
@@ -387,10 +384,9 @@ var _ = ginkgo.Describe("UserGroupLimit", func() {
 
 	ginkgo.It("Verify_maxresources_with_a_wildcard_user_limit", func() {
 		ginkgo.By("Update config")
-		annotation = "ann-" + common.RandSeq(10)
 		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
 		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
-			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", annotation, admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
+			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
 				// remove placement rules so we can control queue
 				sc.Partitions[0].PlacementRules = nil
 
@@ -435,10 +431,9 @@ var _ = ginkgo.Describe("UserGroupLimit", func() {
 
 	ginkgo.It("Verify_maxapplications_with_a_wildcard_user_limit", func() {
 		ginkgo.By("Update config")
-		annotation = "ann-" + common.RandSeq(10)
 		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
 		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
-			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", annotation, admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
+			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
 				// remove placement rules so we can control queue
 				sc.Partitions[0].PlacementRules = nil
 
@@ -483,10 +478,9 @@ var _ = ginkgo.Describe("UserGroupLimit", func() {
 
 	ginkgo.It("Verify_maxresources_with_a_wildcard_group_limit", func() {
 		ginkgo.By("Update config")
-		annotation = "ann-" + common.RandSeq(10)
 		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
 		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
-			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", annotation, admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
+			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
 				// remove placement rules so we can control queue
 				sc.Partitions[0].PlacementRules = nil
 
@@ -531,10 +525,9 @@ var _ = ginkgo.Describe("UserGroupLimit", func() {
 
 	ginkgo.It("Verify_maxapplications_with_a_wildcard_group_limit", func() {
 		ginkgo.By("Update config")
-		annotation = "ann-" + common.RandSeq(10)
 		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
 		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
-			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", annotation, admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
+			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
 				// remove placement rules so we can control queue
 				sc.Partitions[0].PlacementRules = nil
 
@@ -577,6 +570,347 @@ var _ = ginkgo.Describe("UserGroupLimit", func() {
 		checkUsage(userTestType, user2, sandboxQueue1, []*v1.Pod{usergroup2Sandbox1Pod1})
 	})
 
+	ginkgo.It("Verify_maxresources_with_a_valid_user_name_and_specific_user_limit", func() {
+		ginkgo.By("Update config")
+		validUser := "user_Test-a_b_#_c_#_d_/_e@dom:ain.com"
+		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
+		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
+			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
+				// remove placement rules so we can control queue
+				sc.Partitions[0].PlacementRules = nil
+
+				if err := common.AddQueue(sc, constants.DefaultPartition, constants.RootQueue, configs.QueueConfig{
+					Name: "sandbox1",
+					Limits: []configs.Limit{
+						{
+							Limit:           "user entry",
+							Users:           []string{validUser},
+							MaxApplications: 2,
+							MaxResources: map[string]string{
+								siCommon.Memory: fmt.Sprintf("%dM", mediumMem),
+							},
+						},
+					},
+				}); err != nil {
+					return err
+				}
+				return common.AddQueue(sc, constants.DefaultPartition, constants.RootQueue, configs.QueueConfig{Name: "sandbox2"})
+			})
+		})
+
+		// usergroup1 can deploy the first sleep pod to root.sandbox1
+		usergroup1 := &si.UserGroupInformation{User: validUser, Groups: []string{group1}}
+
+		// usergroup1 can't deploy the second sleep pod to root.sandbox1
+		usergroup1Sandbox1Pod1 := deploySleepPod(usergroup1, sandboxQueue1, true, "because memory usage is less than maxresources")
+		deploySleepPod(usergroup1, sandboxQueue1, false, "because final memory usage is more than maxresources")
+		checkUsage(userTestType, url.QueryEscape(validUser), sandboxQueue1, []*v1.Pod{usergroup1Sandbox1Pod1})
+	})
+
+	ginkgo.It("Verify_maxresources_with_a_valid_group_name_and_specific_group_limit", func() {
+		ginkgo.By("Update config")
+		validGroup := "group_Test-a_b_dom:ain.com"
+		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
+		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
+			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
+				// remove placement rules so we can control queue
+				sc.Partitions[0].PlacementRules = nil
+
+				if err := common.AddQueue(sc, constants.DefaultPartition, constants.RootQueue, configs.QueueConfig{
+					Name: "sandbox1",
+					Limits: []configs.Limit{
+						{
+							Limit:           "group entry",
+							Groups:          []string{validGroup},
+							MaxApplications: 2,
+							MaxResources: map[string]string{
+								siCommon.Memory: fmt.Sprintf("%dM", mediumMem),
+							},
+						},
+					},
+				}); err != nil {
+					return err
+				}
+				return common.AddQueue(sc, constants.DefaultPartition, constants.RootQueue, configs.QueueConfig{Name: "sandbox2"})
+			})
+		})
+
+		// usergroup1 can deploy the first sleep pod to root.sandbox1
+		usergroup1 := &si.UserGroupInformation{User: user1, Groups: []string{validGroup}}
+
+		// usergroup1 can't deploy the second sleep pod to root.sandbox1
+		usergroup1Sandbox1Pod1 := deploySleepPod(usergroup1, sandboxQueue1, true, "because memory usage is less than maxresources")
+		_ = deploySleepPod(usergroup1, sandboxQueue1, false, "because final memory usage is more than maxresources")
+		checkUsage(groupTestType, url.QueryEscape(validGroup), sandboxQueue1, []*v1.Pod{usergroup1Sandbox1Pod1})
+	})
+
+	ginkgo.It("Verify_Queue_Name_With_Special_Characters", func() {
+		ginkgo.By("Create a queue with a name that includes all allowed special characters")
+		queueName := "root_test22-a_b_#_c_#_d_/_e@dom:ain"
+		yunikorn.UpdateCustomConfigMapWrapper(oldConfigMap, "", func(sc *configs.SchedulerConfig) error {
+			// remove placement rules so we can control queue
+			sc.Partitions[0].PlacementRules = nil
+			var err error
+			if err = common.AddQueue(sc, "default", "root", configs.QueueConfig{
+				Name:       queueName,
+				Resources:  configs.Resources{Guaranteed: map[string]string{"memory": fmt.Sprintf("%dM", 200)}},
+				Properties: map[string]string{"preemption.delay": "1s"},
+			}); err != nil {
+				return err
+			}
+			return nil
+		})
+		ginkgo.By("Fetch the queue information using the REST API")
+		queueInfo, err := restClient.GetQueue("default", "root."+url.QueryEscape(queueName), false)
+		gomega.Ω(err).NotTo(gomega.HaveOccurred())
+		gomega.Ω(queueInfo.QueueName).To(gomega.Equal("root." + queueName))
+	})
+
+	ginkgo.It("Verify_Queue_Name_With_Disallowed_Special_Characters", func() {
+		ginkgo.By("Attempt to create a queue with a name that includes disallowed special characters")
+		invalidConfig := `
+			partitions:
+  				- name: default
+				  placementrules:
+                       - name: tag
+                         value: namespace
+                         create: true
+                  queues:
+					   - name: root_test22-a_b_#_c_#_d__e@dom:ain$
+						 submitacl: '*'
+`
+		queueName := "root_test22-a_b_#_c_#_d__e@dom:ain$"
+		invalidConfigData := map[string]string{"queues.yaml": invalidConfig}
+		invalidConfigMap := &v1.ConfigMap{
+			ObjectMeta: metav1.ObjectMeta{
+				Name:      constants.ConfigMapName,
+				Namespace: configmanager.YuniKornTestConfig.YkNamespace,
+			},
+			Data: invalidConfigData,
+		}
+		_, invalidConfigErr := kClient.UpdateConfigMap(invalidConfigMap, configmanager.YuniKornTestConfig.YkNamespace)
+		gomega.Ω(invalidConfigErr).Should(gomega.HaveOccurred())
+		ginkgo.By("Verify that the queue was not created")
+		_, err := restClient.GetQueue("default", "root."+url.QueryEscape(queueName), false)
+		gomega.Ω(err).To(gomega.HaveOccurred()) // Expect an error
+		queueName2 := "root_test22"
+		yunikorn.UpdateCustomConfigMapWrapper(oldConfigMap, "", func(sc *configs.SchedulerConfig) error {
+			// remove placement rules so we can control queue
+			sc.Partitions[0].PlacementRules = nil
+			var err error
+			if err = common.AddQueue(sc, "default", "root", configs.QueueConfig{
+				Name:       queueName2,
+				Resources:  configs.Resources{Guaranteed: map[string]string{"memory": fmt.Sprintf("%dM", 200)}},
+				Properties: map[string]string{"preemption.delay": "1s"},
+			}); err != nil {
+				return err
+			}
+			return nil
+		})
+	})
+
+	ginkgo.It("Verify user limit and wildcard user limit", func() {
+		ginkgo.By("Update config")
+		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
+		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
+			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
+				// remove placement rules so we can control queue
+				sc.Partitions[0].PlacementRules = nil
+				err := common.AddQueue(sc, constants.DefaultPartition, constants.RootQueue, configs.QueueConfig{
+					Name: "sandbox1",
+					Limits: []configs.Limit{
+						{
+							Limit:           "user entry",
+							Users:           []string{user1},
+							MaxApplications: 1,
+							MaxResources: map[string]string{
+								siCommon.Memory: fmt.Sprintf("%dM", mediumMem),
+							},
+						},
+						{
+							Limit:           "wildcard user entry",
+							Users:           []string{"*"},
+							MaxApplications: 2,
+							MaxResources: map[string]string{
+								siCommon.Memory: fmt.Sprintf("%dM", largeMem),
+							},
+						},
+					},
+				})
+				if err != nil {
+					return err
+				}
+				return common.AddQueue(sc, constants.DefaultPartition, constants.RootQueue, configs.QueueConfig{Name: "sandbox2"})
+			})
+		})
+
+		// usergroup1 can deploy the first sleep pod to root.sandbox1
+		usergroup1 := &si.UserGroupInformation{User: user1, Groups: []string{group1}}
+
+		// usergroup1 can't deploy the second sleep pod to root.sandbox1
+		usergroup1Sandbox1Pod1 := deploySleepPod(usergroup1, sandboxQueue1, true, "because memory usage is less than user entry limit")
+		_ = deploySleepPod(usergroup1, sandboxQueue1, false, "because final memory usage is more than user entry limit")
+		checkUsage(userTestType, user1, sandboxQueue1, []*v1.Pod{usergroup1Sandbox1Pod1})
+
+		// usergroup2 can deploy 2 sleep pods to root.sandbox1
+		usergroup2 := &si.UserGroupInformation{User: user2, Groups: []string{group2}}
+		usergroup2Sandbox1Pod1 := deploySleepPod(usergroup2, sandboxQueue1, true, "because there is no limit for usergroup2")
+		checkUsage(userTestType, user2, sandboxQueue1, []*v1.Pod{usergroup2Sandbox1Pod1})
+
+		// usergroup2 can deploy the second sleep pod to root.sandbox1
+		usergroup2 = &si.UserGroupInformation{User: user2, Groups: []string{group2}}
+		usergroup2Sandbox1Pod2 := deploySleepPod(usergroup2, sandboxQueue1, true, "because there is no limit for usergroup2")
+		checkUsage(userTestType, user2, sandboxQueue1, []*v1.Pod{usergroup2Sandbox1Pod1, usergroup2Sandbox1Pod2})
+
+		// usergroup2 can't deploy the third sleep pod to root.sandbox1 because of max-application limit
+		usergroup2 = &si.UserGroupInformation{User: user2, Groups: []string{group2}}
+		usergroup2Sandbox1Pod3 := deploySleepPod(usergroup2, sandboxQueue1, false, "because final memory usage is more than wildcard maxapplications")
+		checkUsage(userTestType, user2, sandboxQueue1, []*v1.Pod{usergroup2Sandbox1Pod1, usergroup2Sandbox1Pod2})
+
+		// Update Wildcard user entry limit to 3
+		ginkgo.By("Update config")
+		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
+		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
+			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
+				// remove placement rules so we can control queue
+				sc.Partitions[0].PlacementRules = nil
+
+				err := common.AddQueue(sc, constants.DefaultPartition, constants.RootQueue, configs.QueueConfig{
+					Name: "sandbox1",
+					Limits: []configs.Limit{
+						{
+							Limit:           "user entry",
+							Users:           []string{user1},
+							MaxApplications: 1,
+							MaxResources: map[string]string{
+								siCommon.Memory: fmt.Sprintf("%dM", mediumMem),
+							},
+						},
+						{
+							Limit:           "wildcard user entry",
+							Users:           []string{"*"},
+							MaxApplications: 3,
+							MaxResources: map[string]string{
+								siCommon.Memory: fmt.Sprintf("%dM", largeMem),
+							},
+						},
+					},
+				})
+				if err != nil {
+					return err
+				}
+				return common.AddQueue(sc, constants.DefaultPartition, constants.RootQueue, configs.QueueConfig{Name: "sandbox2"})
+			})
+		})
+		// usergroup2 can deploy the third sleep pod to root.sandbox1 becuase of max-application limit updated to 3
+		checkUsage(userTestType, user2, sandboxQueue1, []*v1.Pod{usergroup2Sandbox1Pod1, usergroup2Sandbox1Pod2, usergroup2Sandbox1Pod3})
+		// usergroup2 can't deploy the fourth sleep pod to root.sandbox1 because of max-application limit
+		usergroup2 = &si.UserGroupInformation{User: user2, Groups: []string{group2}}
+		_ = deploySleepPod(usergroup2, sandboxQueue1, false, "because final memory usage is more than wildcard maxapplications")
+		checkUsage(userTestType, user2, sandboxQueue1, []*v1.Pod{usergroup2Sandbox1Pod1, usergroup2Sandbox1Pod2, usergroup2Sandbox1Pod3})
+
+	})
+
+	ginkgo.It("Verify group limit and wildcard group limit", func() {
+		ginkgo.By("Update config")
+		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
+		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
+			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
+				// remove placement rules so we can control queue
+				sc.Partitions[0].PlacementRules = nil
+
+				err := common.AddQueue(sc, constants.DefaultPartition, constants.RootQueue, configs.QueueConfig{
+					Name: "sandbox1",
+					Limits: []configs.Limit{
+						{
+							Limit:           "group entry",
+							Groups:          []string{group1},
+							MaxApplications: 1,
+							MaxResources: map[string]string{
+								siCommon.Memory: fmt.Sprintf("%dM", mediumMem),
+							},
+						},
+						{
+							Limit:           "wildcard group entry",
+							Groups:          []string{"*"},
+							MaxApplications: 2,
+							MaxResources: map[string]string{
+								siCommon.Memory: fmt.Sprintf("%dM", largeMem),
+							},
+						},
+					}})
+				if err != nil {
+					return err
+				}
+				return common.AddQueue(sc, constants.DefaultPartition, constants.RootQueue, configs.QueueConfig{Name: "sandbox2"})
+			})
+		})
+		// group1 can deploy the first sleep pod to root.sandbox1
+		usergroup1 := &si.UserGroupInformation{User: user1, Groups: []string{group1}}
+		group1Sandvox1Pod1 := deploySleepPod(usergroup1, sandboxQueue1, true, "because there is no limit for group1")
+		checkUsage(groupTestType, group1, sandboxQueue1, []*v1.Pod{group1Sandvox1Pod1})
+
+		// group1 can't deploy the second sleep pod to root.sandbox1
+		usergroup1 = &si.UserGroupInformation{User: user1, Groups: []string{group1}}
+		_ = deploySleepPod(usergroup1, sandboxQueue1, false, "because final memory usage is more than group entry limit")
+		checkUsage(groupTestType, group1, sandboxQueue1, []*v1.Pod{group1Sandvox1Pod1})
+
+		// group2 can deploy 2 sleep pods to root.sandbox1
+		usergroup2 := &si.UserGroupInformation{User: user2, Groups: []string{group2}}
+		group2Sandbox1Pod1 := deploySleepPod(usergroup2, sandboxQueue1, true, "because there is no limit for group2")
+		checkUsageWildcardGroups(groupTestType, group2, sandboxQueue1, []*v1.Pod{group2Sandbox1Pod1})
+
+		// group2 can deploy the second sleep pod to root.sandbox1
+		usergroup2 = &si.UserGroupInformation{User: user2, Groups: []string{group2}}
+		group2Sandbox1Pod2 := deploySleepPod(usergroup2, sandboxQueue1, true, "because there is no limit for group2")
+		checkUsageWildcardGroups(groupTestType, group2, sandboxQueue1, []*v1.Pod{group2Sandbox1Pod1, group2Sandbox1Pod2})
+
+		// group2 can't deploy the third sleep pod to root.sandbox1 because of max-application limit
+		usergroup2 = &si.UserGroupInformation{User: user2, Groups: []string{group2}}
+		group2Sandbox1Pod3 := deploySleepPod(usergroup2, sandboxQueue1, false, "because final memory usage is more than wildcard maxapplications")
+		checkUsageWildcardGroups(groupTestType, group2, sandboxQueue1, []*v1.Pod{group2Sandbox1Pod1, group2Sandbox1Pod2})
+		// Update Wildcard group entry limit to 3
+		ginkgo.By("Update config")
+		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
+		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
+			yunikorn.UpdateCustomConfigMapWrapperWithMap(oldConfigMap, "", admissionCustomConfig, func(sc *configs.SchedulerConfig) error {
+				// remove placement rules so we can control queue
+				sc.Partitions[0].PlacementRules = nil
+
+				err := common.AddQueue(sc, constants.DefaultPartition, constants.RootQueue, configs.QueueConfig{
+					Name: "sandbox1",
+					Limits: []configs.Limit{
+						{
+							Limit:           "group entry",
+							Groups:          []string{group1},
+							MaxApplications: 1,
+							MaxResources: map[string]string{
+								siCommon.Memory: fmt.Sprintf("%dM", mediumMem),
+							},
+						},
+						{
+							Limit:           "wildcard group entry",
+							Groups:          []string{"*"},
+							MaxApplications: 3,
+							MaxResources: map[string]string{
+								siCommon.Memory: fmt.Sprintf("%dM", largeMem),
+							},
+						},
+					}})
+				if err != nil {
+					return err
+				}
+				return common.AddQueue(sc, constants.DefaultPartition, constants.RootQueue, configs.QueueConfig{Name: "sandbox2"})
+			})
+		})
+		// group2 can deploy the third sleep pod to root.sandbox1 becuase of max-application limit updated to 3
+		checkUsageWildcardGroups(groupTestType, group2, sandboxQueue1, []*v1.Pod{group2Sandbox1Pod1, group2Sandbox1Pod2, group2Sandbox1Pod3})
+		// group2 can't deploy the fourth sleep pod to root.sandbox1 because of max-application limit
+		usergroup2 = &si.UserGroupInformation{User: user2, Groups: []string{group2}}
+		_ = deploySleepPod(usergroup2, sandboxQueue1, false, "because final memory usage is more than wildcard maxapplications")
+		checkUsageWildcardGroups(groupTestType, group2, sandboxQueue1, []*v1.Pod{group2Sandbox1Pod1, group2Sandbox1Pod2, group2Sandbox1Pod3})
+	})
+
 	ginkgo.AfterEach(func() {
 		tests.DumpClusterInfoIfSpecFailed(suiteName, []string{ns.Name})
 
@@ -589,7 +923,7 @@ var _ = ginkgo.Describe("UserGroupLimit", func() {
 
 		// reset config
 		ginkgo.By("Restoring YuniKorn configuration")
-		yunikorn.RestoreConfigMapWrapper(oldConfigMap, annotation)
+		yunikorn.RestoreConfigMapWrapper(oldConfigMap)
 	})
 })
 
@@ -656,3 +990,35 @@ func checkUsage(testType TestType, name string, queuePath string, expectedRunnin
 	Ω(resourceUsageDAO.ResourceUsage.Resources["pods"]).To(gomega.Equal(resources.Quantity(len(expectedRunningPods))))
 	Ω(resourceUsageDAO.RunningApplications).To(gomega.ConsistOf(appIDs...))
 }
+
+func checkUsageWildcardGroups(testType TestType, name string, queuePath string, expectedRunningPods []*v1.Pod) {
+	var rootQueueResourceUsageDAO *dao.ResourceUsageDAOInfo
+	if testType == groupTestType {
+		ginkgo.By(fmt.Sprintf("Check group resource usage for %s in queue %s", name, queuePath))
+		groupUsageDAOInfo, err := restClient.GetGroupsUsage(constants.DefaultPartition)
+		Ω(err).NotTo(gomega.HaveOccurred())
+		Ω(groupUsageDAOInfo).NotTo(gomega.BeNil())
+		for _, groupUsageDAOInfog := range groupUsageDAOInfo {
+			if groupUsageDAOInfog.GroupName == "*" {
+				rootQueueResourceUsageDAO = groupUsageDAOInfog.Queues
+			}
+		}
+	}
+	Ω(rootQueueResourceUsageDAO).NotTo(gomega.BeNil())
+	var resourceUsageDAO *dao.ResourceUsageDAOInfo
+	for _, queue := range rootQueueResourceUsageDAO.Children {
+		if queue.QueuePath == queuePath {
+			resourceUsageDAO = queue
+			break
+		}
+	}
+	Ω(resourceUsageDAO).NotTo(gomega.BeNil())
+
+	appIDs := make([]interface{}, 0, len(expectedRunningPods))
+	for _, pod := range expectedRunningPods {
+		appIDs = append(appIDs, pod.Labels[constants.LabelApplicationID])
+	}
+	Ω(resourceUsageDAO.ResourceUsage).NotTo(gomega.BeNil())
+	Ω(resourceUsageDAO.ResourceUsage.Resources["pods"]).To(gomega.Equal(resources.Quantity(len(expectedRunningPods))))
+	Ω(resourceUsageDAO.RunningApplications).To(gomega.ConsistOf(appIDs...))
+}
